@@ -1,3 +1,5 @@
+# encoding: utf-8
+
 from nose.plugins import Plugin
 from inspect import isclass
 import hashlib
@@ -6,7 +8,7 @@ import sys
 import re
 import pkg_resources
 from paste.deploy import loadapp
-from pylons import config
+from ckan.common import config
 import unittest
 import time
 
@@ -17,7 +19,7 @@ class CkanNose(Plugin):
         # import needs to be here or setup happens too early
         import ckan.model as model
 
-        if 'new_tests' in repr(ctx):
+        if 'legacy' not in repr(ctx):
             # We don't want to do the stuff below for new-style tests.
             if not CkanNose.settings.reset_database:
                 model.repo.tables_created_and_initialised = True
@@ -81,21 +83,18 @@ class CkanNose(Plugin):
             help='drop database and reinitialize before tests are run')
 
     def wantClass(self, cls):
-        name = cls.__name__
-
-        wanted = (not cls.__name__.startswith('_')
-                  and (issubclass(cls, unittest.TestCase)
-                       or re.search('(?:^|[\b_\./-])[Tt]est', name)
-                      ))
-        
-        if self.segments and str(hashlib.md5(name).hexdigest())[0] not in self.segments:
+        if self.segments and str(hashlib.md5(
+                cls.__name__).hexdigest())[0] not in self.segments:
             return False
 
-        return wanted
+    def wantFunction(self, fn):
+        if self.segments and hashlib.md5(
+                fn.__name__).hexdigest()[0] not in self.segments:
+            return False
 
     def finalize(self, report):
         if self.segments:
-            print 'Segments: %s' % self.segments
+            print('Segments: %s' % self.segments)
 
     def configure(self, settings, config):
         CkanNose.settings = settings
@@ -127,7 +126,7 @@ class CkanNose(Plugin):
 ##
 ##        testname = str(test)
 ##        #if ' ' in testname:
-##        #    testname = testname.split(' ')[1]
+##        #    testname = testname.split()[1]
 ##
 ##        f.write('%s,%s\n' % (testname, str(runtime)))
 ##
