@@ -1,3 +1,5 @@
+# encoding: utf-8
+
 import os.path
 import logging
 import cgi
@@ -7,6 +9,7 @@ import urllib
 
 from webob.multidict import UnicodeMultiDict
 from paste.util.multidict import MultiDict
+from six import text_type
 
 import ckan.model as model
 import ckan.logic as logic
@@ -17,6 +20,8 @@ import ckan.lib.navl.dictization_functions
 import ckan.lib.jsonp as jsonp
 import ckan.lib.munge as munge
 
+from ckan.views import identify_user
+
 from ckan.common import _, c, request, response
 
 
@@ -34,6 +39,7 @@ CONTENT_TYPES = {
     'text': 'text/plain;charset=utf-8',
     'html': 'text/html;charset=utf-8',
     'json': 'application/json;charset=utf-8',
+    'javascript': 'application/javascript;charset=utf-8',
 }
 
 
@@ -50,9 +56,9 @@ class ApiController(base.BaseController):
             api_version = api_version[1:]
             routes_dict['ver'] = int(api_version)
 
-        self._identify_user()
+        identify_user()
         try:
-            context = {'model': model, 'user': c.user or c.author,
+            context = {'model': model, 'user': c.user,
                        'auth_user_obj': c.userobj}
             logic.check_access('site_read', context)
         except NotAuthorized:
@@ -83,16 +89,19 @@ class ApiController(base.BaseController):
         if response_data is not None:
             response.headers['Content-Type'] = CONTENT_TYPES[content_type]
             if content_type == 'json':
-                response_msg = h.json.dumps(response_data)
+                response_msg = h.json.dumps(
+                    response_data,
+                    for_json=True)  # handle objects with for_json methods
             else:
                 response_msg = response_data
             # Support "JSONP" callback.
-            if status_int == 200 and 'callback' in request.params and \
-                (request.method == 'GET' or
-                 c.logic_function and request.method == 'POST'):
+            if (status_int == 200 and
+                    'callback' in request.params and
+                    request.method == 'GET'):
                 # escape callback to remove '<', '&', '>' chars
                 callback = cgi.escape(request.params['callback'])
                 response_msg = self._wrap_jsonp(callback, response_msg)
+                response.headers['Content-Type'] = CONTENT_TYPES['javascript']
         return response_msg
 
     def _finish_ok(self, response_data=None,
@@ -138,7 +147,7 @@ class ApiController(base.BaseController):
     def _set_response_header(self, name, value):
         try:
             value = str(value)
-        except Exception, inst:
+        except Exception as inst:
             msg = "Couldn't convert '%s' header value '%s' to string: %s" % \
                 (name, value, inst)
             raise Exception(msg)
@@ -149,35 +158,38 @@ class ApiController(base.BaseController):
         response_data['version'] = ver
         return self._finish_ok(response_data)
 
-    def snippet(self, snippet_path, ver=None):
-        ''' Renders and returns a snippet used by ajax calls '''
-        # we only allow snippets in templates/ajax_snippets and it's subdirs
-        snippet_path = u'ajax_snippets/' + snippet_path
-        return base.render(snippet_path, extra_vars=dict(request.params))
-
     def action(self, logic_function, ver=None):
         try:
             function = get_action(logic_function)
         except KeyError:
-            log.error('Can\'t find logic function: %s' % logic_function)
+            log.info('Can\'t find logic function: %s', logic_function)
             return self._finish_bad_request(
                 _('Action name not known: %s') % logic_function)
 
         context = {'model': model, 'session': model.Session, 'user': c.user,
                    'api_version': ver, 'auth_user_obj': c.userobj}
         model.Session()._context = context
-        return_dict = {'help': function.__doc__}
+
+        return_dict = {'help': h.url_for(controller='api',
+                                         action='action',
+                                         logic_function='help_show',
+                                         ver=ver,
+                                         name=logic_function,
+                                         qualified=True,
+                                         )
+                       }
         try:
             side_effect_free = getattr(function, 'side_effect_free', False)
-            request_data = self._get_request_data(try_url_params=
-                                                  side_effect_free)
-        except ValueError, inst:
-            log.error('Bad request data: %s' % inst)
+            request_data = self._get_request_data(
+                try_url_params=side_effect_free)
+        except ValueError as inst:
+            log.info('Bad Action API request data: %s', inst)
             return self._finish_bad_request(
                 _('JSON Error: %s') % inst)
         if not isinstance(request_data, dict):
             # this occurs if request_data is blank
-            log.error('Bad request data - not dict: %r' % request_data)
+            log.info('Bad Action API request data - not dict: %r',
+                     request_data)
             return self._finish_bad_request(
                 _('Bad request data: %s') %
                 'Request data JSON decoded to %r but '
@@ -185,56 +197,62 @@ class ApiController(base.BaseController):
         # if callback is specified we do not want to send that to the search
         if 'callback' in request_data:
             del request_data['callback']
+            c.user = None
+            c.userobj = None
+            context['user'] = None
+            context['auth_user_obj'] = None
         try:
             result = function(context, request_data)
             return_dict['success'] = True
             return_dict['result'] = result
-        except DataError, e:
-            log.error('Format incorrect: %s - %s' % (e.error, request_data))
+        except DataError as e:
+            log.info('Format incorrect (Action API): %s - %s',
+                     e.error, request_data)
             return_dict['error'] = {'__type': 'Integrity Error',
                                     'message': e.error,
                                     'data': request_data}
             return_dict['success'] = False
             return self._finish(400, return_dict, content_type='json')
-        except NotAuthorized, e:
+        except NotAuthorized as e:
             return_dict['error'] = {'__type': 'Authorization Error',
                                     'message': _('Access denied')}
             return_dict['success'] = False
-            
-            if e.extra_msg:
-                return_dict['error']['message'] += ': %s' % e.extra_msg
+
+            if text_type(e):
+                return_dict['error']['message'] += u': %s' % e
 
             return self._finish(403, return_dict, content_type='json')
-        except NotFound, e:
+        except NotFound as e:
             return_dict['error'] = {'__type': 'Not Found Error',
                                     'message': _('Not found')}
-            if e.extra_msg:
-                return_dict['error']['message'] += ': %s' % e.extra_msg
+            if text_type(e):
+                return_dict['error']['message'] += u': %s' % e
             return_dict['success'] = False
             return self._finish(404, return_dict, content_type='json')
-        except ValidationError, e:
+        except ValidationError as e:
             error_dict = e.error_dict
             error_dict['__type'] = 'Validation Error'
             return_dict['error'] = error_dict
             return_dict['success'] = False
             # CS nasty_string ignore
-            log.error('Validation error: %r' % str(e.error_dict))
+            log.info('Validation error (Action API): %r', str(e.error_dict))
             return self._finish(409, return_dict, content_type='json')
-        except search.SearchQueryError, e:
+        except search.SearchQueryError as e:
             return_dict['error'] = {'__type': 'Search Query Error',
                                     'message': 'Search Query is invalid: %r' %
                                     e.args}
             return_dict['success'] = False
             return self._finish(400, return_dict, content_type='json')
-        except search.SearchError, e:
+        except search.SearchError as e:
             return_dict['error'] = {'__type': 'Search Error',
                                     'message': 'Search error: %r' % e.args}
             return_dict['success'] = False
             return self._finish(409, return_dict, content_type='json')
-        except search.SearchIndexError, e:
-            return_dict['error'] = {'__type': 'Search Index Error',
-                    'message': 'Unable to add package to search index: %s' %
-                    str(e)}
+        except search.SearchIndexError as e:
+            return_dict['error'] = {
+                '__type': 'Search Index Error',
+                'message': 'Unable to add package to search index: %s' %
+                           str(e)}
             return_dict['success'] = False
             return self._finish(500, return_dict, content_type='json')
         return self._finish_ok(return_dict)
@@ -253,235 +271,9 @@ class ApiController(base.BaseController):
         if action:
             return get_action(action)
 
-    def list(self, ver=None, register=None, subregister=None, id=None):
-        context = {'model': model, 'session': model.Session,
-                   'user': c.user, 'api_version': ver,
-                   'auth_user_obj': c.userobj}
-        log.debug('listing: %s' % context)
-        action_map = {
-            'revision': 'revision_list',
-            'group': 'group_list',
-            'dataset': 'package_list',
-            'tag': 'tag_list',
-            'related': 'related_list',
-            'licenses': 'license_list',
-            ('dataset', 'relationships'): 'package_relationships_list',
-            ('dataset', 'revisions'): 'package_revision_list',
-            ('dataset', 'activity'): 'package_activity_list',
-            ('group', 'activity'): 'group_activity_list',
-            ('user', 'activity'): 'user_activity_list',
-            ('user', 'dashboard_activity'): 'dashboard_activity_list',
-            ('activity', 'details'): 'activity_detail_list',
-        }
-
-        action = self._get_action_from_map(action_map, register, subregister)
-        if not action:
-            return self._finish_bad_request(
-                _('Cannot list entity of this type: %s') % register)
-        try:
-            return self._finish_ok(action(context, {'id': id}))
-        except NotFound, e:
-            extra_msg = e.extra_msg
-            return self._finish_not_found(extra_msg)
-        except NotAuthorized, e:
-            extra_msg = e.extra_msg
-            return self._finish_not_authz(extra_msg)
-
-    def show(self, ver=None, register=None, subregister=None,
-             id=None, id2=None):
-        action_map = {
-            'revision': 'revision_show',
-            'group': 'group_show_rest',
-            'tag': 'tag_show_rest',
-            'related': 'related_show',
-            'dataset': 'package_show_rest',
-            ('dataset', 'relationships'): 'package_relationships_list',
-        }
-        for type in model.PackageRelationship.get_all_types():
-            action_map[('dataset', type)] = 'package_relationships_list'
-
-        context = {'model': model, 'session': model.Session, 'user': c.user,
-                   'api_version': ver, 'auth_user_obj': c.userobj}
-        data_dict = {'id': id, 'id2': id2, 'rel': subregister}
-
-        log.debug('show: %s' % context)
-
-        action = self._get_action_from_map(action_map, register, subregister)
-        if not action:
-            return self._finish_bad_request(
-                _('Cannot read entity of this type: %s') % register)
-        try:
-            return self._finish_ok(action(context, data_dict))
-        except NotFound, e:
-            extra_msg = e.extra_msg
-            return self._finish_not_found(extra_msg)
-        except NotAuthorized, e:
-            extra_msg = e.extra_msg
-            return self._finish_not_authz(extra_msg)
-
-    def _represent_package(self, package):
-        return package.as_dict(ref_package_by=self.ref_package_by,
-                               ref_group_by=self.ref_group_by)
-
-    def create(self, ver=None, register=None, subregister=None,
-               id=None, id2=None):
-
-        action_map = {
-            'group': 'group_create_rest',
-            'dataset': 'package_create_rest',
-            'rating': 'rating_create',
-            'related': 'related_create',
-            ('dataset', 'relationships'): 'package_relationship_create_rest',
-        }
-        for type in model.PackageRelationship.get_all_types():
-            action_map[('dataset', type)] = 'package_relationship_create_rest'
-
-        context = {'model': model, 'session': model.Session, 'user': c.user,
-                   'api_version': ver, 'auth_user_obj': c.userobj}
-        log.debug('create: %s' % (context))
-        try:
-            request_data = self._get_request_data()
-            data_dict = {'id': id, 'id2': id2, 'rel': subregister}
-            data_dict.update(request_data)
-        except ValueError, inst:
-            return self._finish_bad_request(
-                _('JSON Error: %s') % inst)
-
-        action = self._get_action_from_map(action_map, register, subregister)
-        if not action:
-            return self._finish_bad_request(
-                _('Cannot create new entity of this type: %s %s') %
-                (register, subregister))
-
-        try:
-            response_data = action(context, data_dict)
-            location = None
-            if "id" in data_dict:
-                location = str('%s/%s' % (request.path.replace('package',
-                                                               'dataset'),
-                                          data_dict.get("id")))
-            return self._finish_ok(response_data,
-                                   resource_location=location)
-        except NotAuthorized, e:
-            extra_msg = e.extra_msg
-            return self._finish_not_authz(extra_msg)
-        except NotFound, e:
-            extra_msg = e.extra_msg
-            return self._finish_not_found(extra_msg)
-        except ValidationError, e:
-            # CS: nasty_string ignore
-            log.error('Validation error: %r' % str(e.error_dict))
-            return self._finish(409, e.error_dict, content_type='json')
-        except DataError, e:
-            log.error('Format incorrect: %s - %s' % (e.error, request_data))
-            error_dict = {
-                'success': False,
-                'error': {'__type': 'Integrity Error',
-                                    'message': e.error,
-                                    'data': request_data}}
-            return self._finish(400, error_dict, content_type='json')
-        except search.SearchIndexError:
-            log.error('Unable to add package to search index: %s' %
-                      request_data)
-            return self._finish(500,
-                                _(u'Unable to add package to search index') %
-                                request_data)
-        except:
-            model.Session.rollback()
-            raise
-
-    def update(self, ver=None, register=None, subregister=None,
-               id=None, id2=None):
-        action_map = {
-            'dataset': 'package_update_rest',
-            'group': 'group_update_rest',
-            ('dataset', 'relationships'): 'package_relationship_update_rest',
-        }
-        for type in model.PackageRelationship.get_all_types():
-            action_map[('dataset', type)] = 'package_relationship_update_rest'
-
-        context = {'model': model, 'session': model.Session, 'user': c.user,
-                   'api_version': ver, 'id': id, 'auth_user_obj': c.userobj}
-        log.debug('update: %s' % (context))
-        try:
-            request_data = self._get_request_data()
-            data_dict = {'id': id, 'id2': id2, 'rel': subregister}
-            data_dict.update(request_data)
-        except ValueError, inst:
-            return self._finish_bad_request(
-                _('JSON Error: %s') % inst)
-
-        action = self._get_action_from_map(action_map, register, subregister)
-        if not action:
-            return self._finish_bad_request(
-                _('Cannot update entity of this type: %s') %
-                register.encode('utf-8'))
-        try:
-            response_data = action(context, data_dict)
-            return self._finish_ok(response_data)
-        except NotAuthorized, e:
-            extra_msg = e.extra_msg
-            return self._finish_not_authz(extra_msg)
-        except NotFound, e:
-            extra_msg = e.extra_msg
-            return self._finish_not_found(extra_msg)
-        except ValidationError, e:
-            # CS: nasty_string ignore
-            log.error('Validation error: %r' % str(e.error_dict))
-            return self._finish(409, e.error_dict, content_type='json')
-        except DataError, e:
-            log.error('Format incorrect: %s - %s' % (e.error, request_data))
-            error_dict = {
-                'success': False,
-                'error': {'__type': 'Integrity Error',
-                                    'message': e.error,
-                                    'data': request_data}}
-            return self._finish(400, error_dict, content_type='json')
-        except search.SearchIndexError:
-            log.error('Unable to update search index: %s' % request_data)
-            return self._finish(500, _(u'Unable to update search index') %
-                                request_data)
-
-    def delete(self, ver=None, register=None, subregister=None,
-               id=None, id2=None):
-        action_map = {
-            'group': 'group_delete',
-            'dataset': 'package_delete',
-            'related': 'related_delete',
-            ('dataset', 'relationships'): 'package_relationship_delete_rest',
-        }
-        for type in model.PackageRelationship.get_all_types():
-            action_map[('dataset', type)] = 'package_relationship_delete_rest'
-
-        context = {'model': model, 'session': model.Session, 'user': c.user,
-                   'api_version': ver, 'auth_user_obj': c.userobj}
-
-        data_dict = {'id': id, 'id2': id2, 'rel': subregister}
-
-        log.debug('delete %s/%s/%s/%s' % (register, id, subregister, id2))
-
-        action = self._get_action_from_map(action_map, register, subregister)
-        if not action:
-            return self._finish_bad_request(
-                _('Cannot delete entity of this type: %s %s') %
-                (register, subregister or ''))
-        try:
-            response_data = action(context, data_dict)
-            return self._finish_ok(response_data)
-        except NotAuthorized, e:
-            extra_msg = e.extra_msg
-            return self._finish_not_authz(extra_msg)
-        except NotFound, e:
-            extra_msg = e.extra_msg
-            return self._finish_not_found(extra_msg)
-        except ValidationError, e:
-            # CS: nasty_string ignore
-            log.error('Validation error: %r' % str(e.error_dict))
-            return self._finish(409, e.error_dict, content_type='json')
-
     def search(self, ver=None, register=None):
 
-        log.debug('search %s params: %r' % (register, request.params))
+        log.debug('search %s params: %r', register, request.params)
         if register == 'revision':
             since_time = None
             if 'since_id' in request.params:
@@ -498,19 +290,21 @@ class ApiController(base.BaseController):
                 since_time_str = request.params['since_time']
                 try:
                     since_time = h.date_str_to_datetime(since_time_str)
-                except ValueError, inst:
+                except ValueError as inst:
                     return self._finish_bad_request('ValueError: %s' % inst)
             else:
                 return self._finish_bad_request(
                     _("Missing search term ('since_id=UUID' or " +
-                            " 'since_time=TIMESTAMP')"))
-            revs = model.Session.query(model.Revision).\
-                filter(model.Revision.timestamp > since_time)
+                      " 'since_time=TIMESTAMP')"))
+            revs = model.Session.query(model.Revision) \
+                .filter(model.Revision.timestamp > since_time) \
+                .order_by(model.Revision.timestamp) \
+                .limit(50)  # reasonable enough for a page
             return self._finish_ok([rev.id for rev in revs])
         elif register in ['dataset', 'package', 'resource']:
             try:
                 params = MultiDict(self._get_search_params(request.params))
-            except ValueError, e:
+            except ValueError as e:
                 return self._finish_bad_request(
                     _('Could not read parameters: %r' % e))
 
@@ -570,7 +364,7 @@ class ApiController(base.BaseController):
                         del params['callback']
                     results = query.run(params)
                 return self._finish_ok(results)
-            except search.SearchError, e:
+            except search.SearchError as e:
                 log.exception(e)
                 return self._finish_bad_request(
                     _('Bad search option: %s') % e)
@@ -584,7 +378,7 @@ class ApiController(base.BaseController):
             try:
                 qjson_param = request_params['qjson'].replace('\\\\u', '\\u')
                 params = h.json.loads(qjson_param, encoding='utf8')
-            except ValueError, e:
+            except ValueError as e:
                 raise ValueError(_('Malformed qjson value: %r')
                                  % e)
         elif len(request_params) == 1 and \
@@ -600,44 +394,6 @@ class ApiController(base.BaseController):
             raise ValueError(msg)
         return params
 
-    def markdown(self, ver=None):
-        raw_markdown = request.params.get('q', '')
-        results = h.render_markdown(raw_markdown)
-
-        return self._finish_ok(results)
-
-    def tag_counts(self, ver=None):
-        c.q = request.params.get('q', '')
-
-        context = {'model': model, 'session': model.Session,
-                   'user': c.user or c.author, 'auth_user_obj': c.userobj}
-
-        tag_names = get_action('tag_list')(context, {})
-        results = []
-        for tag_name in tag_names:
-            tag_count = len(context['model'].Tag.get(tag_name).packages)
-            results.append((tag_name, tag_count))
-        return self._finish_ok(results)
-
-    def throughput(self, ver=None):
-        qos = self._calc_throughput()
-        qos = str(qos)
-        return self._finish_ok(qos)
-
-    def _calc_throughput(self, ver=None):
-        period = 10  # Seconds.
-        timing_cache_path = self._get_timing_cache_path()
-        call_count = 0
-        for t in range(0, period):
-            expr = '%s/%s*' % (
-                timing_cache_path,
-                (datetime.datetime.now() -
-                 datetime.timedelta(0, t)).isoformat()[0:19],
-            )
-            call_count += len(glob.glob(expr))
-        # Todo: Clear old records.
-        return float(call_count) / period
-
     @jsonp.jsonpify
     def user_autocomplete(self):
         q = request.params.get('q', '')
@@ -645,7 +401,7 @@ class ApiController(base.BaseController):
         user_list = []
         if q:
             context = {'model': model, 'session': model.Session,
-                       'user': c.user or c.author, 'auth_user_obj': c.userobj}
+                       'user': c.user, 'auth_user_obj': c.userobj}
 
             data_dict = {'q': q, 'limit': limit}
 
@@ -675,33 +431,18 @@ class ApiController(base.BaseController):
         out = map(convert_to_dict, query.all())
         return out
 
-    def is_slug_valid(self):
+    @jsonp.jsonpify
+    def organization_autocomplete(self):
+        q = request.params.get('q', '')
+        limit = request.params.get('limit', 20)
+        organization_list = []
 
-        def package_exists(val):
-            if model.Session.query(model.Package) \
-                .autoflush(False).filter_by(name=val).count():
-                return True
-            return False
-
-        def group_exists(val):
-            if model.Session.query(model.Group) \
-                    .autoflush(False).filter_by(name=val).count():
-                return True
-            return False
-
-        slug = request.params.get('slug') or ''
-        slugtype = request.params.get('type') or ''
-        # TODO: We need plugins to be able to register new disallowed names
-        disallowed = ['new', 'edit', 'search']
-        if slugtype == u'package':
-            response_data = dict(valid=not (package_exists(slug)
-                                 or slug in disallowed))
-            return self._finish_ok(response_data)
-        if slugtype == u'group':
-            response_data = dict(valid=not (group_exists(slug) or
-                                 slug in disallowed))
-            return self._finish_ok(response_data)
-        return self._finish_bad_request('Bad slug type: %s' % slugtype)
+        if q:
+            context = {'user': c.user, 'model': model}
+            data_dict = {'q': q, 'limit': limit}
+            organization_list = \
+                get_action('organization_autocomplete')(context, data_dict)
+        return organization_list
 
     def dataset_autocomplete(self):
         q = request.params.get('incomplete', '')
@@ -709,7 +450,7 @@ class ApiController(base.BaseController):
         package_dicts = []
         if q:
             context = {'model': model, 'session': model.Session,
-                       'user': c.user or c.author, 'auth_user_obj': c.userobj}
+                       'user': c.user, 'auth_user_obj': c.userobj}
 
             data_dict = {'q': q, 'limit': limit}
 
@@ -720,12 +461,13 @@ class ApiController(base.BaseController):
         return self._finish_ok(resultSet)
 
     def tag_autocomplete(self):
-        q = request.params.get('incomplete', '')
+        q = request.str_params.get('incomplete', '')
+        q = text_type(urllib.unquote(q), 'utf-8')
         limit = request.params.get('limit', 10)
         tag_names = []
         if q:
             context = {'model': model, 'session': model.Session,
-                       'user': c.user or c.author, 'auth_user_obj': c.userobj}
+                       'user': c.user, 'auth_user_obj': c.userobj}
 
             data_dict = {'q': q, 'limit': limit}
 
@@ -744,7 +486,7 @@ class ApiController(base.BaseController):
         formats = []
         if q:
             context = {'model': model, 'session': model.Session,
-                       'user': c.user or c.author, 'auth_user_obj': c.userobj}
+                       'user': c.user, 'auth_user_obj': c.userobj}
             data_dict = {'q': q, 'limit': limit}
             formats = get_action('format_autocomplete')(context, data_dict)
 
@@ -770,20 +512,6 @@ class ApiController(base.BaseController):
         munged_tag = munge.munge_tag(tag)
         return self._finish_ok(munged_tag)
 
-    def format_icon(self):
-        f = request.params.get('format')
-        out = {
-            'format': f,
-            'icon': h.icon_url(h.format_icon(f))
-        }
-        return self._finish_ok(out)
-
-    def status(self):
-        context = {'model': model, 'session': model.Session}
-        data_dict = {}
-        status = get_action('status_show')(context, data_dict)
-        return self._finish_ok(status)
-
     def i18n_js_translations(self, lang):
         ''' translation strings for front end '''
         ckan_path = os.path.join(os.path.dirname(__file__), '..')
@@ -821,7 +549,7 @@ class ApiController(base.BaseController):
         def make_unicode(entity):
             '''Cast bare strings and strings in lists or dicts to Unicode. '''
             if isinstance(entity, str):
-                return unicode(entity)
+                return text_type(entity)
             elif isinstance(entity, list):
                 new_items = []
                 for item in entity:
@@ -835,9 +563,9 @@ class ApiController(base.BaseController):
             else:
                 return entity
 
-        cls.log.debug('Retrieving request params: %r' % request.params)
-        cls.log.debug('Retrieving request POST: %r' % request.POST)
-        cls.log.debug('Retrieving request GET: %r' % request.GET)
+        cls.log.debug('Retrieving request params: %r', request.params)
+        cls.log.debug('Retrieving request POST: %r', request.POST)
+        cls.log.debug('Retrieving request GET: %r', request.GET)
         request_data = None
         if request.POST and request.content_type == 'multipart/form-data':
             request_data = dict(request.POST)
@@ -850,7 +578,7 @@ class ApiController(base.BaseController):
                     request_data = keys[0]
                 else:
                     request_data = urllib.unquote_plus(request.body)
-            except Exception, inst:
+            except Exception as inst:
                 msg = "Could not find the POST data: %r : %s" % \
                       (request.POST, inst)
                 raise ValueError(msg)
@@ -864,21 +592,22 @@ class ApiController(base.BaseController):
                     request_data = request.body
                 else:
                     request_data = None
-            except Exception, inst:
+            except Exception as inst:
                 msg = "Could not extract request body data: %s" % \
                       (inst)
                 raise ValueError(msg)
-            cls.log.debug('Retrieved request body: %r' % request.body)
+            cls.log.debug('Retrieved request body: %r', request.body)
             if not request_data:
                 if not try_url_params:
-                    msg = "No request body data"
+                    msg = "Invalid request. Please use POST method" \
+                        " for your request"
                     raise ValueError(msg)
                 else:
                     request_data = {}
         if request_data and request.content_type != 'multipart/form-data':
             try:
                 request_data = h.json.loads(request_data, encoding='utf8')
-            except ValueError, e:
+            except ValueError as e:
                 raise ValueError('Error decoding JSON data. '
                                  'Error: %r '
                                  'JSON data extracted from the request: %r' %
@@ -891,5 +620,5 @@ class ApiController(base.BaseController):
                 # if val is str then assume it is ascii, since json converts
                 # utf8 encoded JSON to unicode
                 request_data[key] = make_unicode(val)
-        cls.log.debug('Request data extracted: %r' % request_data)
+        cls.log.debug('Request data extracted: %r', request_data)
         return request_data
