@@ -1,10 +1,13 @@
+# encoding: utf-8
+
 import cgi
 
 from paste.urlparser import PkgResourcesParser
-from pylons import request, tmpl_context as c
+from pylons import request
 from pylons.controllers.util import forward
 from webhelpers.html.builder import literal
 
+from ckan.common import c
 from ckan.lib.base import BaseController
 from ckan.lib.base import render
 
@@ -29,15 +32,22 @@ class ErrorController(BaseController):
         if not original_response:
             return 'There is no error.'
         # Bypass error template for API operations.
-        if original_request and original_request.path.startswith('/api'):
+        if (original_request and
+                (original_request.path.startswith('/api') or
+                 original_request.path.startswith('/fanstatic'))):
             return original_response.body
+        # If the charset has been lost on the middleware stack, use the
+        # default one (utf-8)
+        if not original_response.charset and original_response.default_charset:
+            original_response.charset = original_response.default_charset
         # Otherwise, decorate original response with error template.
-        c.content = literal(original_response.unicode_body) or \
+        content = literal(original_response.unicode_body) or \
             cgi.escape(request.GET.get('message', ''))
-        c.prefix = request.environ.get('SCRIPT_NAME', ''),
-        c.code = cgi.escape(request.GET.get('code',
-                            str(original_response.status_int))),
-        return render('error_document_template.html')
+        prefix = request.environ.get('SCRIPT_NAME', ''),
+        code = cgi.escape(request.GET.get('code',
+                          str(original_response.status_int))),
+        extra_vars = {'code': code, 'content': content, 'prefix': prefix}
+        return render('error_document_template.html', extra_vars=extra_vars)
 
     def img(self, id):
         """Serve Pylons' stock images"""
