@@ -1,3 +1,5 @@
+# encoding: utf-8
+
 """
 The feed controller produces Atom feeds of datasets.
 
@@ -22,15 +24,16 @@ of the revision history, rather than a feed of datasets.
 import logging
 import urlparse
 
+from six import text_type
 import webhelpers.feedgenerator
-from pylons import config
 
-import ckan.model as model
 import ckan.lib.base as base
 import ckan.lib.helpers as h
 import ckan.logic as logic
+import ckan.model as model
+import ckan.plugins as plugins
 
-from ckan.common import _, g, c, request, response, json
+from ckan.common import _, config, c, request, response, json
 
 # TODO make the item list configurable
 ITEMS_LIMIT = 20
@@ -46,7 +49,7 @@ def _package_search(data_dict):
      * unless overridden, sets a default item limit
     """
     context = {'model': model, 'session': model.Session,
-               'user': c.user or c.author, 'auth_user_obj': c.userobj}
+               'user': c.user, 'auth_user_obj': c.userobj}
 
     if 'sort' not in data_dict or not data_dict['sort']:
         data_dict['sort'] = 'metadata_modified desc'
@@ -166,16 +169,19 @@ class FeedController(base.BaseController):
                               controller='package',
                               action='search')
 
-    def group(self, id):
-        try:
-            context = {'model': model, 'session': model.Session,
-                       'user': c.user or c.author, 'auth_user_obj': c.userobj}
-            group_dict = logic.get_action('group_show')(context, {'id': id})
-        except logic.NotFound:
-            base.abort(404, _('Group not found'))
+    def _group_or_organization(self, obj_dict, is_org):
 
         data_dict, params = self._parse_url_params()
-        data_dict['fq'] = 'groups:"%s"' % id
+        if is_org:
+            key = 'owner_org'
+            value = obj_dict['id']
+            group_type = 'organization'
+        else:
+            key = 'groups'
+            value = obj_dict['name']
+            group_type = 'group'
+
+        data_dict['fq'] = '{0}:"{1}"'.format(key, value)
 
         item_count, results = _package_search(data_dict)
 
@@ -183,28 +189,63 @@ class FeedController(base.BaseController):
                                                 item_count=item_count,
                                                 limit=data_dict['rows'],
                                                 controller='feed',
-                                                action='group',
-                                                id=id)
-
+                                                action=group_type,
+                                                id=obj_dict['name'])
         feed_url = self._feed_url(params,
                                   controller='feed',
-                                  action='group',
-                                  id=id)
+                                  action=group_type,
+                                  id=obj_dict['name'])
 
-        alternate_url = self._alternate_url(params, groups=id)
+        site_title = config.get('ckan.site_title', 'CKAN')
+        if is_org:
+            guid = _create_atom_id(u'/feeds/organization/%s.atom' %
+                                   obj_dict['name'])
+            alternate_url = self._alternate_url(params,
+                                                organization=obj_dict['name'])
+            desc = u'Recently created or updated datasets on %s '\
+                'by organization: "%s"' % (site_title, obj_dict['title'])
+            title = u'%s - Organization: "%s"' % (site_title,
+                                                  obj_dict['title'])
+
+        else:  # is group
+            guid = _create_atom_id(u'/feeds/group/%s.atom' %
+                                   obj_dict['name'])
+            alternate_url = self._alternate_url(params,
+                                                groups=obj_dict['name'])
+            desc = u'Recently created or updated datasets on %s '\
+                'by group: "%s"' % (site_title, obj_dict['title'])
+            title = u'%s - Group: "%s"' %\
+                (site_title, obj_dict['title'])
 
         return self.output_feed(results,
-                                feed_title=u'%s - Group: "%s"' %
-                                (g.site_title, group_dict['title']),
-                                feed_description=u'Recently created or '
-                                'updated datasets on %s by group: "%s"' %
-                                (g.site_title, group_dict['title']),
+                                feed_title=title,
+                                feed_description=desc,
                                 feed_link=alternate_url,
-                                feed_guid=_create_atom_id
-                                (u'/feeds/groups/%s.atom' % id),
+                                feed_guid=guid,
                                 feed_url=feed_url,
                                 navigation_urls=navigation_urls)
 
+    def group(self, id):
+        try:
+            context = {'model': model, 'session': model.Session,
+                       'user': c.user, 'auth_user_obj': c.userobj}
+            group_dict = logic.get_action('group_show')(context, {'id': id})
+        except logic.NotFound:
+            base.abort(404, _('Group not found'))
+
+        return self._group_or_organization(group_dict, is_org=False)
+
+    def organization(self, id):
+        try:
+            context = {'model': model, 'session': model.Session,
+                       'user': c.user, 'auth_user_obj': c.userobj}
+            group_dict = logic.get_action('organization_show')(context,
+                                                               {'id': id})
+        except logic.NotFound:
+            base.abort(404, _('Organization not found'))
+
+        return self._group_or_organization(group_dict, is_org=True)
+
     def tag(self, id):
         data_dict, params = self._parse_url_params()
         data_dict['fq'] = 'tags:"%s"' % id
@@ -225,12 +266,14 @@ class FeedController(base.BaseController):
 
         alternate_url = self._alternate_url(params, tags=id)
 
+        site_title = config.get('ckan.site_title', 'CKAN')
+
         return self.output_feed(results,
                                 feed_title=u'%s - Tag: "%s"' %
-                                (g.site_title, id),
+                                (site_title, id),
                                 feed_description=u'Recently created or '
                                 'updated datasets on %s by tag: "%s"' %
-                                (g.site_title, id),
+                                (site_title, id),
                                 feed_link=alternate_url,
                                 feed_guid=_create_atom_id
                                 (u'/feeds/tag/%s.atom' % id),
@@ -255,10 +298,12 @@ class FeedController(base.BaseController):
 
         alternate_url = self._alternate_url(params)
 
+        site_title = config.get('ckan.site_title', 'CKAN')
+
         return self.output_feed(results,
-                                feed_title=g.site_title,
+                                feed_title=site_title,
                                 feed_description=u'Recently created or '
-                                'updated datasets on %s' % g.site_title,
+                                'updated datasets on %s' % site_title,
                                 feed_link=alternate_url,
                                 feed_guid=_create_atom_id
                                 (u'/feeds/dataset.atom'),
@@ -276,12 +321,7 @@ class FeedController(base.BaseController):
                 search_params[param] = value
                 fq += ' %s:"%s"' % (param, value)
 
-        try:
-            page = int(request.params.get('page', 1))
-        except ValueError:
-            base.abort(400, _('"page" parameter must be a positive integer'))
-        if page < 0:
-            base.abort(400, _('"page" parameter must be a positive integer'))
+        page = h.get_page_number(request.params)
 
         limit = ITEMS_LIMIT
         data_dict = {
@@ -309,11 +349,13 @@ class FeedController(base.BaseController):
 
         alternate_url = self._alternate_url(request.params)
 
+        site_title = config.get('ckan.site_title', 'CKAN')
+
         return self.output_feed(results,
-                                feed_title=u'%s - Custom query' % g.site_title,
+                                feed_title=u'%s - Custom query' % site_title,
                                 feed_description=u'Recently created or updated'
                                 ' datasets on %s. Custom query: \'%s\'' %
-                                (g.site_title, q),
+                                (site_title, q),
                                 feed_link=alternate_url,
                                 feed_guid=_create_atom_id(atom_url),
                                 feed_url=feed_url,
@@ -327,10 +369,18 @@ class FeedController(base.BaseController):
             config.get('ckan.site_url', '').strip()
 
         # TODO language
-        feed = _FixedAtom1Feed(
-            title=feed_title,
-            link=feed_link,
-            description=feed_description,
+        feed_class = None
+        for plugin in plugins.PluginImplementations(plugins.IFeed):
+            if hasattr(plugin, 'get_feed_class'):
+                feed_class = plugin.get_feed_class()
+
+        if not feed_class:
+            feed_class = _FixedAtom1Feed
+
+        feed = feed_class(
+            feed_title,
+            feed_link,
+            feed_description,
             language=u'en',
             author_name=author_name,
             author_link=author_link,
@@ -343,6 +393,12 @@ class FeedController(base.BaseController):
         )
 
         for pkg in results:
+            additional_fields = {}
+
+            for plugin in plugins.PluginImplementations(plugins.IFeed):
+                if hasattr(plugin, 'get_item_additional_fields'):
+                    additional_fields = plugin.get_item_additional_fields(pkg)
+
             feed.add_item(
                 title=pkg.get('title', ''),
                 link=self.base_url + h.url_for(controller='package',
@@ -356,18 +412,20 @@ class FeedController(base.BaseController):
                 author_email=pkg.get('author_email', ''),
                 categories=[t['name'] for t in pkg.get('tags', [])],
                 enclosure=webhelpers.feedgenerator.Enclosure(
-                    self.base_url + h.url_for(controller='api',
-                                              register='package',
-                                              action='show',
-                                              id=pkg['name'],
-                                              ver='2'),
-                    unicode(len(json.dumps(pkg))),   # TODO fix this
-                    u'application/json')
+                    h.url_for(controller='api',
+                              register='package',
+                              action='show',
+                              id=pkg['name'],
+                              ver='3',
+                              qualified=True),
+                    text_type(len(json.dumps(pkg))),   # TODO fix this
+                    u'application/json'),
+                **additional_fields
             )
         response.content_type = feed.mime_type
         return feed.writeString('utf-8')
 
-    #### CLASS PRIVATE METHODS ####
+    # CLASS PRIVATE METHODS #
 
     def _feed_url(self, query, controller, action, **kwargs):
         """
@@ -426,12 +484,7 @@ class FeedController(base.BaseController):
         Returns the constructed search-query dict, and the valid URL
         query parameters.
         """
-        try:
-            page = int(request.params.get('page', 1)) or 1
-        except ValueError:
-            base.abort(400, _('"page" parameter must be a positive integer'))
-        if page < 0:
-            base.abort(400, _('"page" parameter must be a positive integer'))
+        page = h.get_page_number(request.params)
 
         limit = ITEMS_LIMIT
         data_dict = {
