@@ -1,6 +1,9 @@
+# encoding: utf-8
+
 from datetime import datetime, timedelta
 
 from pylons.i18n import get_lang
+from six import text_type
 
 import ckan.logic as logic
 import ckan.lib.base as base
@@ -15,7 +18,7 @@ class RevisionController(base.BaseController):
     def __before__(self, action, **env):
         base.BaseController.__before__(self, action, **env)
 
-        context = {'model': model, 'user': c.user or c.author,
+        context = {'model': model, 'user': c.user,
                    'auth_user_obj': c.userobj}
         if c.user:
             try:
@@ -28,7 +31,7 @@ class RevisionController(base.BaseController):
         try:
             logic.check_access('site_read', context)
         except logic.NotAuthorized:
-            base.abort(401, _('Not authorized to see this page'))
+            base.abort(403, _('Not authorized to see this page'))
 
     def index(self):
         return self.list()
@@ -42,7 +45,7 @@ class RevisionController(base.BaseController):
                 title=_(u'CKAN Repository Revision History'),
                 link=h.url_for(controller='revision', action='list', id=''),
                 description=_(u'Recent changes to the CKAN repository.'),
-                language=unicode(get_lang()),
+                language=text_type(get_lang()),
             )
             # TODO: make this configurable?
             # we do not want the system to fall over!
@@ -62,8 +65,6 @@ class RevisionController(base.BaseController):
                 package_indications = []
                 revision_changes = model.repo.list_changes(revision)
                 resource_revisions = revision_changes[model.Resource]
-                resource_group_revisions = \
-                    revision_changes[model.ResourceGroup]
                 package_extra_revisions = revision_changes[model.PackageExtra]
                 for package in revision.packages:
                     if not package:
@@ -89,16 +90,9 @@ class RevisionController(base.BaseController):
                     else:
                         transition = 'updated'
                         for resource_revision in resource_revisions:
-                            if resource_revision.continuity.resource_group.\
-                                    package_id == package.id:
+                            if resource_revision.package_id == package.id:
                                 transition += ':resources'
                                 break
-                        for resource_group_revision in \
-                                resource_group_revisions:
-                            if resource_group_revision.package_id == \
-                                    package.id:
-                                transition += ':resource_group'
-                                break
                         for package_extra_revision in package_extra_revisions:
                             if package_extra_revision.package_id == \
                                     package.id:
@@ -129,9 +123,20 @@ class RevisionController(base.BaseController):
             return feed.writeString('utf-8')
         else:
             query = model.Session.query(model.Revision)
+            revs = query.limit(20).all()
+            filtered_revs = []
+            for rev in list(revs):
+                private_rev = False
+                for pkg in rev.packages:
+                    if pkg.private:
+                        private_rev = True
+                        break
+                if not private_rev:
+                    filtered_revs.append(rev)
+
             c.page = h.Page(
-                collection=query,
-                page=request.params.get('page', 1),
+                collection=filtered_revs,
+                page=h.get_page_number(request.params),
                 url=h.pager_url,
                 items_per_page=20
             )
@@ -146,6 +151,14 @@ class RevisionController(base.BaseController):
 
         pkgs = model.Session.query(model.PackageRevision).\
             filter_by(revision=c.revision)
+
+        context = {'model': model, 'user': c.user}
+        for pkg in c.revision.packages:
+            try:
+                logic.check_access('package_show', context, {'id': pkg.id})
+            except logic.NotAuthorized:
+                base.abort(403, _('Not authorized to see this page'))
+
         c.packages = [pkg.continuity for pkg in pkgs if not pkg.private]
         pkgtags = model.Session.query(model.PackageTagRevision).\
             filter_by(revision=c.revision)
@@ -157,6 +170,7 @@ class RevisionController(base.BaseController):
         return base.render('revision/read.html')
 
     def diff(self, id=None):
+
         if 'diff' not in request.params or 'oldid' not in request.params:
             base.abort(400)
         c.revision_from = model.Session.query(model.Revision).get(
@@ -167,6 +181,13 @@ class RevisionController(base.BaseController):
         c.diff_entity = request.params.get('diff_entity')
         if c.diff_entity == 'package':
             c.pkg = model.Package.by_name(id)
+
+            context = {'model': model, 'user': c.user}
+            try:
+                logic.check_access('package_show', context, {'id': c.pkg.id})
+            except logic.NotAuthorized:
+                base.abort(403, _('Not authorized to see this page'))
+
             diff = c.pkg.diff(c.revision_to, c.revision_from)
         elif c.diff_entity == 'group':
             c.group = model.Group.by_name(id)
@@ -188,7 +209,7 @@ class RevisionController(base.BaseController):
         if action in ['delete', 'undelete']:
             # this should be at a lower level (e.g. logic layer)
             if not c.revision_change_state_allowed:
-                base.abort(401)
+                base.abort(403)
             if action == 'delete':
                 revision.state = model.State.DELETED
             elif action == 'undelete':
