@@ -1,12 +1,14 @@
-from genshi.template.loader import TemplateNotFound
+# encoding: utf-8
 
 import ckan.lib.base as base
+import ckan.lib.render
+from ckan.common import response
 
 
 class TemplateController(base.BaseController):
 
     def view(self, url):
-        """By default, the final controller tried to fulfill the request
+        u"""By default, the final controller tried to fulfill the request
         when no other routes match. It may be used to display a template
         when all else fails, e.g.::
 
@@ -27,13 +29,17 @@ class TemplateController(base.BaseController):
         By default this controller aborts the request with a 404 (Not
         Found)
         """
+        if url.endswith(u'.txt'):
+            response.headers[b'Content-Type'] = b'text/plain; charset=utf-8'
+        # Default content-type is text/html
         try:
             return base.render(url)
-        except TemplateNotFound:
-            if url.endswith('.html'):
+        except ckan.lib.render.TemplateNotFound:
+            if url.endswith(u'.html'):
                 base.abort(404)
-            url += '.html'
+            url += u'.html'
+            response.headers[b'Content-Type'] = b'text/html; charset=utf-8'
             try:
                 return base.render(url)
-            except TemplateNotFound:
+            except ckan.lib.render.TemplateNotFound:
                 base.abort(404)
