@@ -1,50 +1,63 @@
-import logging
-from urllib import quote
+# encoding: utf-8
 
-from pylons import config
+import logging
+
+from ckan.common import config
+from paste.deploy.converters import asbool
+from six import text_type
 
 import ckan.lib.base as base
 import ckan.model as model
 import ckan.lib.helpers as h
-import ckan.new_authz as new_authz
+import ckan.authz as authz
 import ckan.logic as logic
 import ckan.logic.schema as schema
 import ckan.lib.captcha as captcha
 import ckan.lib.mailer as mailer
 import ckan.lib.navl.dictization_functions as dictization_functions
+import ckan.lib.authenticator as authenticator
 import ckan.plugins as p
 
-from ckan.common import _, c, g, request, response
+from ckan.common import _, c, request, response
 
 log = logging.getLogger(__name__)
 
 
 abort = base.abort
 render = base.render
-validate = base.validate
 
 check_access = logic.check_access
 get_action = logic.get_action
 NotFound = logic.NotFound
 NotAuthorized = logic.NotAuthorized
 ValidationError = logic.ValidationError
+UsernamePasswordError = logic.UsernamePasswordError
 
 DataError = dictization_functions.DataError
 unflatten = dictization_functions.unflatten
 
 
+def set_repoze_user(user_id):
+    '''Set the repoze.who cookie to match a given user_id'''
+    if 'repoze.who.plugins' in request.environ:
+        rememberer = request.environ['repoze.who.plugins']['friendlyform']
+        identity = {'repoze.who.userid': user_id}
+        response.headerlist += rememberer.remember(request.environ,
+                                                   identity)
+
+
 class UserController(base.BaseController):
     def __before__(self, action, **env):
         base.BaseController.__before__(self, action, **env)
         try:
-            context = {'model': model, 'user': c.user or c.author,
+            context = {'model': model, 'user': c.user,
                        'auth_user_obj': c.userobj}
             check_access('site_read', context)
         except NotAuthorized:
             if c.action not in ('login', 'request_reset', 'perform_reset',):
-                abort(401, _('Not authorized to see this page'))
+                abort(403, _('Not authorized to see this page'))
 
-    ## hooks for subclasses
+    # hooks for subclasses
     new_user_form = 'user/new_user_form.html'
     edit_user_form = 'user/edit_user_form.html'
 
@@ -63,18 +76,20 @@ class UserController(base.BaseController):
         into a format suitable for the form (optional)'''
 
     def _setup_template_variables(self, context, data_dict):
-        c.is_sysadmin = new_authz.is_sysadmin(c.user)
+        c.is_sysadmin = authz.is_sysadmin(c.user)
         try:
             user_dict = get_action('user_show')(context, data_dict)
         except NotFound:
-            abort(404, _('User not found'))
+            h.flash_error(_('Not authorized to see this page'))
+            h.redirect_to(controller='user', action='login')
         except NotAuthorized:
-            abort(401, _('Not authorized to see this page'))
+            abort(403, _('Not authorized to see this page'))
+
         c.user_dict = user_dict
         c.is_myself = user_dict['name'] == c.user
         c.about_formatted = h.render_markdown(user_dict['about'])
 
-    ## end hooks
+    # end hooks
 
     def _get_repoze_handler(self, handler_name):
         '''Returns the URL that repoze.who will respond to and perform a
@@ -83,21 +98,23 @@ class UserController(base.BaseController):
                        handler_name)
 
     def index(self):
-        LIMIT = 20
-
-        page = int(request.params.get('page', 1))
+        page = h.get_page_number(request.params)
         c.q = request.params.get('q', '')
         c.order_by = request.params.get('order_by', 'name')
 
-        context = {'return_query': True, 'user': c.user or c.author,
+        context = {'return_query': True, 'user': c.user,
                    'auth_user_obj': c.userobj}
 
         data_dict = {'q': c.q,
                      'order_by': c.order_by}
+
+        limit = int(
+            request.params.get('limit', config.get('ckan.user_list_limit', 20))
+        )
         try:
             check_access('user_list', context, data_dict)
         except NotAuthorized:
-            abort(401, _('Not authorized to see this page'))
+            abort(403, _('Not authorized to see this page'))
 
         users_list = get_action('user_list')(context, data_dict)
 
@@ -106,24 +123,24 @@ class UserController(base.BaseController):
             page=page,
             url=h.pager_url,
             item_count=users_list.count(),
-            items_per_page=LIMIT
+            items_per_page=limit
         )
         return render('user/list.html')
 
     def read(self, id=None):
         context = {'model': model, 'session': model.Session,
-                   'user': c.user or c.author, 'auth_user_obj': c.userobj,
+                   'user': c.user, 'auth_user_obj': c.userobj,
                    'for_view': True}
         data_dict = {'id': id,
-                     'user_obj': c.userobj}
-
-        context['with_related'] = True
+                     'user_obj': c.userobj,
+                     'include_datasets': True,
+                     'include_num_followers': True}
 
         self._setup_template_variables(context, data_dict)
 
         # The legacy templates have the user's activity stream on the user
         # profile page, new templates do not.
-        if h.asbool(config.get('ckan.legacy_templates', False)):
+        if asbool(config.get('ckan.legacy_templates', False)):
             c.user_activity_stream = get_action('user_activity_list_html')(
                 context, {'id': c.user_dict['id']})
 
@@ -134,8 +151,7 @@ class UserController(base.BaseController):
             h.redirect_to(locale=locale, controller='user', action='login',
                           id=None)
         user_ref = c.userobj.get_reference_preferred_for_uri()
-        h.redirect_to(locale=locale, controller='user', action='dashboard',
-                      id=user_ref)
+        h.redirect_to(locale=locale, controller='user', action='dashboard')
 
     def register(self, data=None, errors=None, error_summary=None):
         context = {'model': model, 'session': model.Session, 'user': c.user,
@@ -143,7 +159,7 @@ class UserController(base.BaseController):
         try:
             check_access('user_create', context)
         except NotAuthorized:
-            abort(401, _('Unauthorized to register as a user.'))
+            abort(403, _('Unauthorized to register as a user.'))
 
         return self.new(data, errors, error_summary)
 
@@ -151,8 +167,9 @@ class UserController(base.BaseController):
         '''GET to display a form for registering a new user.
            or POST the form data to actually do the user registration.
         '''
-        context = {'model': model, 'session': model.Session,
-                   'user': c.user or c.author,
+        context = {'model': model,
+                   'session': model.Session,
+                   'user': c.user,
                    'auth_user_obj': c.userobj,
                    'schema': self._new_form_to_db_schema(),
                    'save': 'save' in request.params}
@@ -160,12 +177,12 @@ class UserController(base.BaseController):
         try:
             check_access('user_create', context)
         except NotAuthorized:
-            abort(401, _('Unauthorized to create a user'))
+            abort(403, _('Unauthorized to create a user'))
 
-        if context['save'] and not data:
+        if context['save'] and not data and request.method == 'POST':
             return self._save_new(context)
 
-        if c.user and not data:
+        if c.user and not data and not authz.is_sysadmin(c.user):
             # #1799 Don't offer the registration form if already logged in
             return render('user/logout_first.html')
 
@@ -174,7 +191,7 @@ class UserController(base.BaseController):
         error_summary = error_summary or {}
         vars = {'data': data, 'errors': errors, 'error_summary': error_summary}
 
-        c.is_sysadmin = new_authz.is_sysadmin(c.user)
+        c.is_sysadmin = authz.is_sysadmin(c.user)
         c.form = render(self.new_user_form, extra_vars=vars)
         return render('user/new.html')
 
@@ -192,7 +209,31 @@ class UserController(base.BaseController):
             h.redirect_to(user_index)
         except NotAuthorized:
             msg = _('Unauthorized to delete user with id "{user_id}".')
-            abort(401, msg.format(user_id=id))
+            abort(403, msg.format(user_id=id))
+
+    def generate_apikey(self, id):
+        '''Cycle the API key of a user'''
+        context = {'model': model,
+                   'session': model.Session,
+                   'user': c.user,
+                   'auth_user_obj': c.userobj,
+                   }
+        if id is None:
+            if c.userobj:
+                id = c.userobj.id
+            else:
+                abort(400, _('No user specified'))
+        data_dict = {'id': id}
+
+        try:
+            result = get_action('user_generate_apikey')(context, data_dict)
+        except NotAuthorized:
+            abort(403, _('Unauthorized to edit user %s') % '')
+        except NotFound:
+            abort(404, _('User not found'))
+
+        h.flash_success(_('Profile updated'))
+        h.redirect_to(controller='user', action='read', id=result['name'])
 
     def _save_new(self, context):
         try:
@@ -202,8 +243,8 @@ class UserController(base.BaseController):
             captcha.check_recaptcha(request)
             user = get_action('user_create')(context, data_dict)
         except NotAuthorized:
-            abort(401, _('Unauthorized to create user %s') % '')
-        except NotFound, e:
+            abort(403, _('Unauthorized to create user %s') % '')
+        except NotFound as e:
             abort(404, _('User not found'))
         except DataError:
             abort(400, _(u'Integrity Error'))
@@ -211,24 +252,28 @@ class UserController(base.BaseController):
             error_msg = _(u'Bad Captcha. Please try again.')
             h.flash_error(error_msg)
             return self.new(data_dict)
-        except ValidationError, e:
+        except ValidationError as e:
             errors = e.error_dict
             error_summary = e.error_summary
             return self.new(data_dict, errors, error_summary)
         if not c.user:
             # log the user in programatically
-            rememberer = request.environ['repoze.who.plugins']['friendlyform']
-            identity = {'repoze.who.userid': data_dict['name']}
-            response.headerlist += rememberer.remember(request.environ,
-                                                       identity)
-            h.redirect_to(controller='user', action='me', __ckan_no_root=True)
+            set_repoze_user(data_dict['name'])
+            h.redirect_to(controller='user', action='me')
         else:
             # #1799 User has managed to register whilst logged in - warn user
             # they are not re-logged in as new user.
             h.flash_success(_('User "%s" is now registered but you are still '
                             'logged in as "%s" from before') %
                             (data_dict['name'], c.user))
-            return render('user/logout_first.html')
+            if authz.is_sysadmin(c.user):
+                # the sysadmin created a new user. We redirect him to the
+                # activity page for the newly created user
+                h.redirect_to(controller='user',
+                              action='activity',
+                              id=data_dict['name'])
+            else:
+                return render('user/logout_first.html')
 
     def edit(self, id=None, data=None, errors=None, error_summary=None):
         context = {'save': 'save' in request.params,
@@ -246,9 +291,9 @@ class UserController(base.BaseController):
         try:
             check_access('user_update', context, data_dict)
         except NotAuthorized:
-            abort(401, _('Unauthorized to edit a user.'))
+            abort(403, _('Unauthorized to edit a user.'))
 
-        if (context['save']) and not data:
+        if context['save'] and not data and request.method == 'POST':
             return self._save_edit(id, context)
 
         try:
@@ -256,7 +301,8 @@ class UserController(base.BaseController):
 
             schema = self._db_to_edit_form_schema()
             if schema:
-                old_data, errors = validate(old_data, schema)
+                old_data, errors = \
+                    dictization_functions.validate(old_data, schema, context)
 
             c.display_name = old_data.get('display_name')
             c.user_name = old_data.get('name')
@@ -264,15 +310,15 @@ class UserController(base.BaseController):
             data = data or old_data
 
         except NotAuthorized:
-            abort(401, _('Unauthorized to edit user %s') % '')
+            abort(403, _('Unauthorized to edit user %s') % '')
         except NotFound:
             abort(404, _('User not found'))
 
         user_obj = context.get('user_obj')
 
-        if not (new_authz.is_sysadmin(c.user)
+        if not (authz.is_sysadmin(c.user)
                 or c.user == user_obj.name):
-            abort(401, _('User %s not authorized to edit %s') %
+            abort(403, _('User %s not authorized to edit %s') %
                   (str(c.user), id))
 
         errors = errors or {}
@@ -280,11 +326,11 @@ class UserController(base.BaseController):
 
         self._setup_template_variables({'model': model,
                                         'session': model.Session,
-                                        'user': c.user or c.author},
+                                        'user': c.user},
                                        data_dict)
 
         c.is_myself = True
-        c.show_email_notifications = h.asbool(
+        c.show_email_notifications = asbool(
             config.get('ckan.activity_streams_email_notifications'))
         c.form = render(self.edit_user_form, extra_vars=vars)
 
@@ -292,28 +338,54 @@ class UserController(base.BaseController):
 
     def _save_edit(self, id, context):
         try:
+            if id in (c.userobj.id, c.userobj.name):
+                current_user = True
+            else:
+                current_user = False
+            old_username = c.userobj.name
+
             data_dict = logic.clean_dict(unflatten(
                 logic.tuplize_dict(logic.parse_params(request.params))))
             context['message'] = data_dict.get('log_message', '')
             data_dict['id'] = id
 
+            email_changed = data_dict['email'] != c.userobj.email
+
+            if (data_dict['password1'] and data_dict['password2']) \
+                    or email_changed:
+                identity = {'login': c.user,
+                            'password': data_dict['old_password']}
+                auth = authenticator.UsernamePasswordAuthenticator()
+
+                if auth.authenticate(request.environ, identity) != c.user:
+                    raise UsernamePasswordError
+
             # MOAN: Do I really have to do this here?
             if 'activity_streams_email_notifications' not in data_dict:
                 data_dict['activity_streams_email_notifications'] = False
 
             user = get_action('user_update')(context, data_dict)
             h.flash_success(_('Profile updated'))
+
+            if current_user and data_dict['name'] != old_username:
+                # Changing currently logged in user's name.
+                # Update repoze.who cookie to match
+                set_repoze_user(data_dict['name'])
             h.redirect_to(controller='user', action='read', id=user['name'])
         except NotAuthorized:
-            abort(401, _('Unauthorized to edit user %s') % id)
-        except NotFound, e:
+            abort(403, _('Unauthorized to edit user %s') % id)
+        except NotFound as e:
             abort(404, _('User not found'))
         except DataError:
             abort(400, _(u'Integrity Error'))
-        except ValidationError, e:
+        except ValidationError as e:
             errors = e.error_dict
             error_summary = e.error_summary
             return self.edit(id, data_dict, errors, error_summary)
+        except UsernamePasswordError:
+            errors = {'oldpassword': [_('Password entered was incorrect')]}
+            error_summary = {_('Old Password'): _('incorrect password')}
+            return self.edit(id, data_dict, errors, error_summary)
 
     def login(self, error=None):
         # Do any plugin login stuff
@@ -323,16 +395,10 @@ class UserController(base.BaseController):
         if 'error' in request.params:
             h.flash_error(request.params['error'])
 
-        if request.environ['SCRIPT_NAME'] and g.openid_enabled:
-            # #1662 restriction
-            log.warn('Cannot mount CKAN at a URL and login with OpenID.')
-            g.openid_enabled = False
-
         if not c.user:
             came_from = request.params.get('came_from')
             if not came_from:
-                came_from = h.url_for(controller='user', action='logged_in',
-                                      __ckan_no_root=True)
+                came_from = h.url_for(controller='user', action='logged_in')
             c.login_handler = h.url_for(
                 self._get_repoze_handler('login_handler_path'),
                 came_from=came_from)
@@ -356,15 +422,10 @@ class UserController(base.BaseController):
 
             user_dict = get_action('user_show')(context, data_dict)
 
-            h.flash_success(_("%s is now logged in") %
-                            user_dict['display_name'])
             return self.me()
         else:
             err = _('Login failed. Bad username or password.')
-            if g.openid_enabled:
-                err += _(' (Or if using OpenID, it hasn\'t been associated '
-                         'with a user account.)')
-            if h.asbool(config.get('ckan.legacy_templates', 'false')):
+            if asbool(config.get('ckan.legacy_templates', 'false')):
                 h.flash_error(err)
                 h.redirect_to(controller='user',
                               action='login', came_from=came_from)
@@ -375,10 +436,9 @@ class UserController(base.BaseController):
         # Do any plugin logout stuff
         for item in p.PluginImplementations(p.IAuthenticator):
             item.logout()
-        url = h.url_for(controller='user', action='logged_out_page',
-                        __ckan_no_root=True)
+        url = h.url_for(controller='user', action='logged_out_page')
         h.redirect_to(self._get_repoze_handler('logout_handler_path') +
-                      '?came_from=' + url)
+                      '?came_from=' + url, parse_url=True)
 
     def logged_out(self):
         # redirect if needed
@@ -397,56 +457,64 @@ class UserController(base.BaseController):
         try:
             check_access('request_reset', context)
         except NotAuthorized:
-            abort(401, _('Unauthorized to request reset password.'))
+            abort(403, _('Unauthorized to request reset password.'))
 
         if request.method == 'POST':
             id = request.params.get('user')
-
+            if id in (None, u''):
+                h.flash_error(_(u'Email is required'))
+                return h.redirect_to(u'/user/reset')
             context = {'model': model,
-                       'user': c.user}
+                       'user': c.user,
+                       u'ignore_auth': True}
+            user_objs = []
 
-            data_dict = {'id': id}
-            user_obj = None
-            try:
-                user_dict = get_action('user_show')(context, data_dict)
-                user_obj = context['user_obj']
-            except NotFound:
-                # Try searching the user
-                del data_dict['id']
-                data_dict['q'] = id
+            if u'@' not in id:
+                try:
+                    user_dict = get_action('user_show')(context, {'id': id})
+                    user_objs.append(context['user_obj'])
+                except NotFound:
+                    pass
+            else:
+                user_list = logic.get_action(u'user_list')(context, {
+                    u'email': id
+                })
+                if user_list:
+                    # send reset emails for *all* user accounts with this email
+                    # (otherwise we'd have to silently fail - we can't tell the
+                    # user, as that would reveal the existence of accounts with
+                    # this email address)
+                    for user_dict in user_list:
+                        logic.get_action(u'user_show')(
+                            context, {u'id': user_dict[u'id']})
+                        user_objs.append(context[u'user_obj'])
 
-                if id and len(id) > 2:
-                    user_list = get_action('user_list')(context, data_dict)
-                    if len(user_list) == 1:
-                        # This is ugly, but we need the user object for the
-                        # mailer,
-                        # and user_list does not return them
-                        del data_dict['q']
-                        data_dict['id'] = user_list[0]['id']
-                        user_dict = get_action('user_show')(context, data_dict)
-                        user_obj = context['user_obj']
-                    elif len(user_list) > 1:
-                        h.flash_error(_('"%s" matched several users') % (id))
-                    else:
-                        h.flash_error(_('No such user: %s') % id)
-                else:
-                    h.flash_error(_('No such user: %s') % id)
+            if not user_objs:
+                log.info(u'User requested reset link for unknown user: {}'
+                         .format(id))
 
-            if user_obj:
+            for user_obj in user_objs:
+                log.info(u'Emailing reset link to user: {}'
+                         .format(user_obj.name))
                 try:
                     mailer.send_reset_link(user_obj)
-                    h.flash_success(_('Please check your inbox for '
-                                    'a reset code.'))
-                    h.redirect_to('/')
-                except mailer.MailerException, e:
-                    h.flash_error(_('Could not send reset link: %s') %
-                                  unicode(e))
+                except mailer.MailerException as e:
+                    h.flash_error(
+                        _(u'Error sending the email. Try again later '
+                          'or contact an administrator for help')
+                    )
+                    log.exception(e)
+                    return h.redirect_to(u'/')
+            # always tell the user it succeeded, because otherwise we reveal
+            # which accounts exist or not
+            h.flash_success(
+                _(u'A reset link has been emailed to you '
+                  '(unless the account specified does not exist)'))
+            return h.redirect_to(u'/')
         return render('user/request_reset.html')
 
     def perform_reset(self, id):
         # FIXME 403 error for invalid key is a non helpful page
-        # FIXME We should reset the reset key when it is used to prevent
-        # reuse of the url
         context = {'model': model, 'session': model.Session,
                    'user': id,
                    'keep_email': True}
@@ -454,14 +522,14 @@ class UserController(base.BaseController):
         try:
             check_access('user_reset', context)
         except NotAuthorized:
-            abort(401, _('Unauthorized to reset password.'))
+            abort(403, _('Unauthorized to reset password.'))
 
         try:
             data_dict = {'id': id}
             user_dict = get_action('user_show')(context, data_dict)
 
             user_obj = context['user_obj']
-        except NotFound, e:
+        except NotFound as e:
             abort(404, _('User not found'))
 
         c.reset_key = request.params.get('key')
@@ -472,24 +540,30 @@ class UserController(base.BaseController):
         if request.method == 'POST':
             try:
                 context['reset_password'] = True
+                user_state = user_dict['state']
                 new_password = self._get_form_password()
                 user_dict['password'] = new_password
+                username = request.params.get('name')
+                if (username is not None and username != ''):
+                    user_dict['name'] = username
                 user_dict['reset_key'] = c.reset_key
                 user_dict['state'] = model.State.ACTIVE
                 user = get_action('user_update')(context, user_dict)
+                mailer.create_reset_key(user_obj)
 
                 h.flash_success(_("Your password has been reset."))
-                h.redirect_to('/')
+                h.redirect_to(u'home.index')
             except NotAuthorized:
                 h.flash_error(_('Unauthorized to edit user %s') % id)
-            except NotFound, e:
+            except NotFound as e:
                 h.flash_error(_('User not found'))
             except DataError:
                 h.flash_error(_(u'Integrity Error'))
-            except ValidationError, e:
+            except ValidationError as e:
                 h.flash_error(u'%r' % e.error_dict)
-            except ValueError, ve:
-                h.flash_error(unicode(ve))
+            except ValueError as ve:
+                h.flash_error(text_type(ve))
+            user_dict['state'] = user_state
 
         c.user_dict = user_dict
         return render('user/perform_reset.html')
@@ -508,33 +582,38 @@ class UserController(base.BaseController):
         raise ValueError(_('You must provide a password'))
 
     def followers(self, id=None):
-        context = {'for_view': True, 'user': c.user or c.author,
+        context = {'for_view': True, 'user': c.user,
                    'auth_user_obj': c.userobj}
-        data_dict = {'id': id, 'user_obj': c.userobj}
+        data_dict = {'id': id, 'user_obj': c.userobj,
+                     'include_num_followers': True}
         self._setup_template_variables(context, data_dict)
         f = get_action('user_follower_list')
         try:
             c.followers = f(context, {'id': c.user_dict['id']})
         except NotAuthorized:
-            abort(401, _('Unauthorized to view followers %s') % '')
+            abort(403, _('Unauthorized to view followers %s') % '')
         return render('user/followers.html')
 
     def activity(self, id, offset=0):
         '''Render this user's public activity stream page.'''
 
         context = {'model': model, 'session': model.Session,
-                   'user': c.user or c.author, 'auth_user_obj': c.userobj,
+                   'user': c.user, 'auth_user_obj': c.userobj,
                    'for_view': True}
-        data_dict = {'id': id, 'user_obj': c.userobj}
+        data_dict = {'id': id, 'user_obj': c.userobj,
+                     'include_num_followers': True}
         try:
             check_access('user_show', context, data_dict)
         except NotAuthorized:
-            abort(401, _('Not authorized to see this page'))
+            abort(403, _('Not authorized to see this page'))
 
         self._setup_template_variables(context, data_dict)
 
-        c.user_activity_stream = get_action('user_activity_list_html')(
-            context, {'id': c.user_dict['id'], 'offset': offset})
+        try:
+            c.user_activity_stream = get_action('user_activity_list_html')(
+                context, {'id': c.user_dict['id'], 'offset': offset})
+        except ValidationError:
+            base.abort(400)
 
         return render('user/activity_stream.html')
 
@@ -552,10 +631,10 @@ class UserController(base.BaseController):
         if (filter_type and filter_id):
             context = {
                 'model': model, 'session': model.Session,
-                'user': c.user or c.author, 'auth_user_obj': c.userobj,
+                'user': c.user, 'auth_user_obj': c.userobj,
                 'for_view': True
             }
-            data_dict = {'id': filter_id}
+            data_dict = {'id': filter_id, 'include_num_followers': True}
             followee = None
 
             action_functions = {
@@ -571,11 +650,8 @@ class UserController(base.BaseController):
                 abort(404, _('Follow item not found'))
             try:
                 followee = action_function(context, data_dict)
-            except NotFound:
+            except (NotFound, NotAuthorized):
                 abort(404, _('{0} not found').format(filter_type))
-            except NotAuthorized:
-                abort(401, _('Unauthorized to read {0} {1}').format(
-                    filter_type, id))
 
             if followee is not None:
                 return {
@@ -596,7 +672,7 @@ class UserController(base.BaseController):
 
     def dashboard(self, id=None, offset=0):
         context = {'model': model, 'session': model.Session,
-                   'user': c.user or c.author, 'auth_user_obj': c.userobj,
+                   'user': c.user, 'auth_user_obj': c.userobj,
                    'for_view': True}
         data_dict = {'id': id, 'user_obj': c.userobj, 'offset': offset}
         self._setup_template_variables(context, data_dict)
@@ -620,21 +696,21 @@ class UserController(base.BaseController):
         return render('user/dashboard.html')
 
     def dashboard_datasets(self):
-        context = {'for_view': True, 'user': c.user or c.author,
+        context = {'for_view': True, 'user': c.user,
                    'auth_user_obj': c.userobj}
-        data_dict = {'user_obj': c.userobj}
+        data_dict = {'user_obj': c.userobj, 'include_datasets': True}
         self._setup_template_variables(context, data_dict)
         return render('user/dashboard_datasets.html')
 
     def dashboard_organizations(self):
-        context = {'for_view': True, 'user': c.user or c.author,
+        context = {'for_view': True, 'user': c.user,
                    'auth_user_obj': c.userobj}
         data_dict = {'user_obj': c.userobj}
         self._setup_template_variables(context, data_dict)
         return render('user/dashboard_organizations.html')
 
     def dashboard_groups(self):
-        context = {'for_view': True, 'user': c.user or c.author,
+        context = {'for_view': True, 'user': c.user,
                    'auth_user_obj': c.userobj}
         data_dict = {'user_obj': c.userobj}
         self._setup_template_variables(context, data_dict)
@@ -644,39 +720,39 @@ class UserController(base.BaseController):
         '''Start following this user.'''
         context = {'model': model,
                    'session': model.Session,
-                   'user': c.user or c.author,
+                   'user': c.user,
                    'auth_user_obj': c.userobj}
-        data_dict = {'id': id}
+        data_dict = {'id': id, 'include_num_followers': True}
         try:
             get_action('follow_user')(context, data_dict)
             user_dict = get_action('user_show')(context, data_dict)
             h.flash_success(_("You are now following {0}").format(
                 user_dict['display_name']))
         except ValidationError as e:
-            error_message = (e.extra_msg or e.message or e.error_summary
+            error_message = (e.message or e.error_summary
                              or e.error_dict)
             h.flash_error(error_message)
         except NotAuthorized as e:
-            h.flash_error(e.extra_msg)
+            h.flash_error(e.message)
         h.redirect_to(controller='user', action='read', id=id)
 
     def unfollow(self, id):
         '''Stop following this user.'''
         context = {'model': model,
                    'session': model.Session,
-                   'user': c.user or c.author,
+                   'user': c.user,
                    'auth_user_obj': c.userobj}
-        data_dict = {'id': id}
+        data_dict = {'id': id, 'include_num_followers': True}
         try:
             get_action('unfollow_user')(context, data_dict)
             user_dict = get_action('user_show')(context, data_dict)
             h.flash_success(_("You are no longer following {0}").format(
                 user_dict['display_name']))
         except (NotFound, NotAuthorized) as e:
-            error_message = e.extra_msg or e.message
+            error_message = e.message
             h.flash_error(error_message)
         except ValidationError as e:
-            error_message = (e.error_summary or e.message or e.extra_msg
+            error_message = (e.error_summary or e.message
                              or e.error_dict)
             h.flash_error(error_message)
         h.redirect_to(controller='user', action='read', id=id)
