@@ -1,25 +1,14 @@
+# encoding: utf-8
+
 import logging
 
 from zope.interface import implements
 from repoze.who.interfaces import IAuthenticator
 
-from ckan.model import User, Session
+from ckan.model import User
 
 log = logging.getLogger(__name__)
 
-class OpenIDAuthenticator(object):
-    implements(IAuthenticator)
-
-    def authenticate(self, environ, identity):
-        if 'repoze.who.plugins.openid.userid' in identity:
-            openid = identity['repoze.who.plugins.openid.userid']
-            user = User.by_openid(openid)
-            if user is None or not user.is_active():
-                return None
-            else:
-                return user.name
-        return None
-
 
 class UsernamePasswordAuthenticator(object):
     implements(IAuthenticator)
@@ -41,4 +30,3 @@ class UsernamePasswordAuthenticator(object):
             return user.name
 
         return None
-
