@@ -1,16 +1,22 @@
+# encoding: utf-8
+
 import datetime
 import uuid
 import logging
+import six
 
 from sqlalchemy.orm import class_mapper
+from six import string_types
 
 import ckan.lib.dictization as d
 import ckan.lib.helpers as h
-import ckan.new_authz as new_authz
+import ckan.authz as authz
 
 log = logging.getLogger(__name__)
 
+
 def resource_dict_save(res_dict, context):
+
     model = context["model"]
     session = context["session"]
 
@@ -27,39 +33,37 @@ def resource_dict_save(res_dict, context):
     table = class_mapper(model.Resource).mapped_table
     fields = [field.name for field in table.c]
 
-    for key, value in res_dict.iteritems():
-        if isinstance(value, list):
-            continue
+    # Strip the full url for resources of type 'upload'
+    if res_dict.get('url') and res_dict.get('url_type') == u'upload':
+        res_dict['url'] = res_dict['url'].rsplit('/')[-1]
+
+    # Resource extras not submitted will be removed from the existing extras
+    # dict
+    new_extras = {}
+    has_changed = False
+    for key, value in six.iteritems(res_dict):
         if key in ('extras', 'revision_timestamp', 'tracking_summary'):
             continue
         if key in fields:
             if isinstance(getattr(obj, key), datetime.datetime):
-                if getattr(obj, key).isoformat() == value:
+                if isinstance(value, string_types):
+                    db_value = getattr(obj, key).isoformat()
+                else:
+                    db_value = getattr(obj, key)
+                if  db_value == value:
                     continue
-            if key == 'url' and not new and obj.url <> value:
+                if key == 'last_modified' and not new:
+                    obj.url_changed = True
+            if key == 'url' and not new and obj.url != value:
                 obj.url_changed = True
-            # this is an internal field so ignore
-            # FIXME This helps get the tests to pass but is a hack and should
-            # be fixed properly. basically don't update the format if not needed
-            if (key == 'format' and (value == obj.format
-                    or value == d.model_dictize._unified_resource_format(obj.format))):
-                continue
             setattr(obj, key, value)
         else:
             # resources save extras directly onto the object, instead
             # of in a separate extras field like packages and groups
-            obj.extras[key] = value
+            new_extras[key] = value
 
-    # Resource extras not submitted should be removed from the extras dict
-    extras_to_delete = set(obj.extras.keys()) - set(res_dict.keys())
-    for delete_me in extras_to_delete:
-        del obj.extras[delete_me]
-
-    if context.get('pending'):
-        if session.is_modified(obj, include_collections=False, passive=True):
-            obj.state = u'pending'
-    else:
-        obj.state = u'active'
+    obj.state = u'active'
+    obj.extras = new_extras
 
     session.add(obj)
     return obj
@@ -69,23 +73,30 @@ def package_resource_list_save(res_dicts, package, context):
     if res_dicts is None and allow_partial_update:
         return
 
-    pending = context.get('pending')
-
-    resource_list = package.resource_groups_all[0].resources_all
-    old_list = package.resource_groups_all[0].resources_all[:]
+    resource_list = package.resources_all
+    old_list = package.resources_all[:]
 
     obj_list = []
     for res_dict in res_dicts or []:
+        if not u'package_id' in res_dict or not res_dict[u'package_id']:
+            res_dict[u'package_id'] = package.id
         obj = resource_dict_save(res_dict, context)
         obj_list.append(obj)
 
+    # Set the package's resources. resource_list is an ORM relation - the
+    # package's resources. If we didn't have the slice operator "[:]" then it
+    # would reassign the variable "resource_list" to be the obj_list. But with
+    # the slice operator it changes the contents of the relation, setting the
+    # package's resources.
+    # At the table level, for each resource in the obj_list, its
+    # resource.package_id is changed to this package (which is needed for new
+    # resources), and every resource.position is set to ascending integers,
+    # according to their ordering in the obj_list.
     resource_list[:] = obj_list
 
+    # Mark any left-over resources as deleted
     for resource in set(old_list) - set(obj_list):
-        if pending and resource.state != 'deleted':
-            resource.state = 'pending-deleted'
-        else:
-            resource.state = 'deleted'
+        resource.state = 'deleted'
         resource_list.append(resource)
 
 
@@ -111,17 +122,16 @@ def package_extras_save(extra_dicts, obj, context):
             new_extras[extra_dict["key"]] = extra_dict["value"]
     #new
     for key in set(new_extras.keys()) - set(old_extras.keys()):
-        state = 'pending' if context.get('pending') else 'active'
+        state = 'active'
         extra = model.PackageExtra(state=state, key=key, value=new_extras[key])
         session.add(extra)
         extras_list.append(extra)
     #changed
     for key in set(new_extras.keys()) & set(old_extras.keys()):
         extra = old_extras[key]
-        #dont change state to pending if nothing has changed
         if new_extras[key] == extra.value and extra.state != 'deleted':
             continue
-        state = 'pending' if context.get('pending') else 'active'
+        state = 'active'
         extra.value = new_extras[key]
         extra.state = state
         session.add(extra)
@@ -130,22 +140,9 @@ def package_extras_save(extra_dicts, obj, context):
         extra = old_extras[key]
         if extra.state == 'deleted':
             continue
-        state = 'pending-deleted' if context.get('pending') else 'deleted'
+        state = 'deleted'
         extra.state = state
 
-def group_extras_save(extras_dicts, context):
-
-    model = context["model"]
-    session = context["session"]
-
-    result_dict = {}
-    for extra_dict in extras_dicts:
-        if extra_dict.get("deleted"):
-            continue
-        result_dict[extra_dict["key"]] = extra_dict["value"]
-
-    return result_dict
-
 def package_tag_list_save(tag_dicts, package, context):
     allow_partial_update = context.get("allow_partial_update", False)
     if tag_dicts is None and allow_partial_update:
@@ -153,16 +150,13 @@ def package_tag_list_save(tag_dicts, package, context):
 
     model = context["model"]
     session = context["session"]
-    pending = context.get('pending')
 
     tag_package_tag = dict((package_tag.tag, package_tag)
                             for package_tag in
                             package.package_tag_all)
 
-    tag_package_tag_inactive = dict(
-        [ (tag,pt) for tag,pt in tag_package_tag.items() if
-            pt.state in ['deleted', 'pending-deleted'] ]
-        )
+    tag_package_tag_inactive = {tag: pt for tag,pt in tag_package_tag.items() if
+            pt.state in ['deleted']}
 
     tag_name_vocab = set()
     tags = set()
@@ -176,21 +170,18 @@ def package_tag_list_save(tag_dicts, package, context):
     # case 1: currently active but not in new list
     for tag in set(tag_package_tag.keys()) - tags:
         package_tag = tag_package_tag[tag]
-        if pending and package_tag.state != 'deleted':
-            package_tag.state = 'pending-deleted'
-        else:
-            package_tag.state = 'deleted'
+        package_tag.state = 'deleted'
 
     # case 2: in new list but never used before
     for tag in tags - set(tag_package_tag.keys()):
-        state = 'pending' if pending else 'active'
+        state = 'active'
         package_tag_obj = model.PackageTag(package, tag, state)
         session.add(package_tag_obj)
         tag_package_tag[tag] = package_tag_obj
 
     # case 3: in new list and already used but in deleted state
     for tag in tags.intersection(set(tag_package_tag_inactive.keys())):
-        state = 'pending' if pending else 'active'
+        state = 'active'
         package_tag = tag_package_tag[tag]
         package_tag.state = state
 
@@ -205,7 +196,6 @@ def package_membership_list_save(group_dicts, package, context):
     capacity = 'public'
     model = context["model"]
     session = context["session"]
-    pending = context.get('pending')
     user = context.get('user')
 
     members = session.query(model.Member) \
@@ -237,7 +227,7 @@ def package_membership_list_save(group_dicts, package, context):
         member_obj = group_member[group]
         if member_obj and member_obj.state == 'deleted':
             continue
-        if new_authz.has_user_permission_for_group_or_org(
+        if authz.has_user_permission_for_group_or_org(
                 member_obj.group_id, user, 'read'):
             member_obj.capacity = capacity
             member_obj.state = 'deleted'
@@ -248,7 +238,7 @@ def package_membership_list_save(group_dicts, package, context):
         member_obj = group_member.get(group)
         if member_obj and member_obj.state == 'active':
             continue
-        if new_authz.has_user_permission_for_group_or_org(
+        if authz.has_user_permission_for_group_or_org(
                 group.id, user, 'read'):
             member_obj = group_member.get(group)
             if member_obj:
@@ -272,7 +262,6 @@ def relationship_list_save(relationship_dicts, package, attr, context):
 
     model = context["model"]
     session = context["session"]
-    pending = context.get('pending')
 
     relationship_list = getattr(package, attr)
     old_list = relationship_list[:]
@@ -286,10 +275,7 @@ def relationship_list_save(relationship_dicts, package, attr, context):
     relationship_list[:] = relationships
 
     for relationship in set(old_list) - set(relationship_list):
-        if pending and relationship.state <> 'deleted':
-            relationship.state = 'pending-deleted'
-        else:
-            relationship.state = 'deleted'
+        relationship.state = 'deleted'
         relationship_list.append(relationship)
 
 def package_dict_save(pkg_dict, context):
@@ -387,14 +373,13 @@ def group_member_save(context, group_dict, member_table_name):
     return processed
 
 
-def group_dict_save(group_dict, context):
+def group_dict_save(group_dict, context, prevent_packages_update=False):
     from ckan.lib.search import rebuild
 
     model = context["model"]
     session = context["session"]
     group = context.get("group")
     allow_partial_update = context.get("allow_partial_update", False)
-    prevent_packages_update = context.get("prevent_packages_update", False)
 
     Group = model.Group
     if group:
@@ -424,6 +409,18 @@ def group_dict_save(group_dict, context):
             'Groups: %r  Tags: %r', pkgs_edited, group_users_changed,
             group_groups_changed, group_tags_changed)
 
+    extras = group_dict.get("extras", [])
+    new_extras = {i['key'] for i in extras}
+    if extras:
+        old_extras = group.extras
+        for key in set(old_extras) - new_extras:
+            del group.extras[key]
+        for x in extras:
+            if 'deleted' in x and x['key'] in old_extras:
+                del group.extras[x['key']]
+                continue
+            group.extras[x['key']] = x['value']
+
     # We will get a list of packages that we have either added or
     # removed from the group, and trigger a re-index.
     package_ids = pkgs_edited['removed']
@@ -432,15 +429,6 @@ def group_dict_save(group_dict, context):
         session.commit()
         map( rebuild, package_ids )
 
-    extras = group_extras_save(group_dict.get("extras", {}), context)
-    if extras or not allow_partial_update:
-        old_extras = set(group.extras.keys())
-        new_extras = set(extras.keys())
-        for key in old_extras - new_extras:
-            del group.extras[key]
-        for key in new_extras:
-            group.extras[key] = extras[key]
-
     return group
 
 
@@ -462,13 +450,6 @@ def user_dict_save(user_dict, context):
     return user
 
 
-def related_dict_save(related_dict, context):
-    model = context['model']
-    session = context['session']
-
-    return d.table_dict_save(related_dict,model.Related, context)
-
-
 def package_api_to_dict(api1_dict, context):
 
     package = context.get("package")
@@ -480,7 +461,7 @@ def package_api_to_dict(api1_dict, context):
     for key, value in api1_dict.iteritems():
         new_value = value
         if key == 'tags':
-            if isinstance(value, basestring):
+            if isinstance(value, string_types):
                 new_value = [{"name": item} for item in value.split()]
             else:
                 new_value = [{"name": item} for item in value]
@@ -620,3 +601,18 @@ def follower_dict_save(data_dict, context, FollowerClass):
             object_id=data_dict['id'])
     session.add(follower_obj)
     return follower_obj
+
+
+def resource_view_dict_save(data_dict, context):
+    model = context['model']
+    resource_view = context.get('resource_view')
+    if resource_view:
+        data_dict['id'] = resource_view.id
+    config = {}
+    for key, value in data_dict.iteritems():
+        if key not in model.ResourceView.get_columns():
+            config[key]  = value
+    data_dict['config'] = config
+
+
+    return d.table_dict_save(data_dict, model.ResourceView, context)
