@@ -1,5 +1,7 @@
-import datetime
+# encoding: utf-8
 
+import datetime
+import pytz
 from babel import numbers
 
 import ckan.lib.i18n as i18n
@@ -68,35 +70,85 @@ _MONTH_FUNCTIONS = [_month_jan, _month_feb, _month_mar, _month_apr,
                    _month_sept, _month_oct, _month_nov, _month_dec]
 
 
-def localised_nice_date(datetime_):
-    ''' Returns a friendly localised unicode representation of a datetime. '''
-    now = datetime.datetime.now()
-    date_diff = now - datetime_
-    days = date_diff.days
-    if days < 1 and now > datetime_:
-        # less than one day
-        seconds = date_diff.seconds
-        if seconds < 3600:
-            # less than one hour
-            if seconds < 60:
-                return _('Just now')
+def localised_nice_date(datetime_, show_date=False, with_hours=False):
+    ''' Returns a friendly localised unicode representation of a datetime.
+
+    :param datetime_: The date to format
+    :type datetime_: datetime
+    :param show_date: Show date not 2 days ago etc
+    :type show_date: bool
+    :param with_hours: should the `hours:mins` be shown for dates
+    :type with_hours: bool
+
+    :rtype: sting
+    '''
+
+    def months_between(date1, date2):
+        if date1 > date2:
+            date1, date2 = date2, date1
+        m1 = date1.year * 12 + date1.month
+        m2 = date2.year * 12 + date2.month
+        months = m2 - m1
+        if date1.day > date2.day:
+            months -= 1
+        elif date1.day == date2.day:
+            seconds1 = date1.hour * 3600 + date1.minute + date1.second
+            seconds2 = date2.hour * 3600 + date2.minute + date2.second
+            if seconds1 > seconds2:
+                months -= 1
+        return months
+
+    if not show_date:
+        now = datetime.datetime.now(pytz.utc)
+        if datetime_.tzinfo is None:
+            datetime_ = datetime_.replace(tzinfo=pytz.utc)
+
+        date_diff = now - datetime_
+        days = date_diff.days
+        if days < 1 and now > datetime_:
+            # less than one day
+            seconds = date_diff.seconds
+            if seconds < 3600:
+                # less than one hour
+                if seconds < 60:
+                    return _('Just now')
+                else:
+                    return ungettext('{mins} minute ago', '{mins} minutes ago',
+                                     seconds / 60).format(mins=seconds / 60)
             else:
-                return ungettext('{mins} minute ago', '{mins} minutes ago',
-                                 seconds / 60).format(mins=seconds / 60)
-        else:
-            return ungettext('{hours} hour ago', '{hours} hours ago',
-                             seconds / 3600).format(hours=seconds / 3600)
-    # more than one day
-    if days < 31:
-        return ungettext('{days} day ago', '{days} days ago',
-                         days).format(days=days)
+                return ungettext('{hours} hour ago', '{hours} hours ago',
+                                 seconds / 3600).format(hours=seconds / 3600)
+        # more than one day
+        months = months_between(datetime_, now)
+
+        if months < 1:
+            return ungettext('{days} day ago', '{days} days ago',
+                             days).format(days=days)
+        if months < 13:
+            return ungettext('{months} month ago', '{months} months ago',
+                             months).format(months=months)
+        return ungettext('over {years} year ago', 'over {years} years ago',
+                         months / 12).format(years=months / 12)
+
     # actual date
-    month = datetime_.month
-    day = datetime_.day
-    year = datetime_.year
-    month_name = _MONTH_FUNCTIONS[month - 1]()
-    return _('{month} {day}, {year}').format(month=month_name, day=day,
-                                             year=year)
+    details = {
+        'min': datetime_.minute,
+        'hour': datetime_.hour,
+        'day': datetime_.day,
+        'year': datetime_.year,
+        'month': _MONTH_FUNCTIONS[datetime_.month - 1](),
+        'timezone': datetime_.tzname(),
+    }
+
+    if with_hours:
+        return (
+            # NOTE: This is for translating dates like `April 24, 2013, 10:45 (Europe/Zurich)`
+            _('{month} {day}, {year}, {hour:02}:{min:02} ({timezone})') \
+            .format(**details))
+    else:
+        return (
+            # NOTE: This is for translating dates like `April 24, 2013`
+            _('{month} {day}, {year}').format(**details))
 
 
 def localised_number(number):
