@@ -1,8 +1,10 @@
+# encoding: utf-8
+
+# TODO: Remove once the old Pylons API controller is deleted
+
 import decorator
 
-from pylons import request, response
-
-from ckan.lib.helpers import json
+from ckan.common import json, request, response
 
 
 def to_jsonp(data):
@@ -20,7 +22,7 @@ def to_jsonp(data):
 def jsonpify(func, *args, **kwargs):
     """A decorator that reformats the output as JSON; or, if the
     *callback* parameter is specified (in the HTTP request), as JSONP.
-    
+
     Very much modelled after pylons.decorators.jsonify .
     """
     data = func(*args, **kwargs)
