@@ -1,38 +1,48 @@
+# encoding: utf-8
+
+import codecs
+import os
 import smtplib
+import socket
 import logging
 import uuid
 from time import time
 from email.mime.text import MIMEText
 from email.header import Header
 from email import Utils
-from urlparse import urljoin
 
-from pylons import config
+from ckan.common import config
 import paste.deploy.converters
+from six import text_type
 
 import ckan
 import ckan.model as model
 import ckan.lib.helpers as h
+from ckan.lib.base import render_jinja2
 
-from ckan.common import _, g
+from ckan.common import _
 
 log = logging.getLogger(__name__)
 
+
 class MailerException(Exception):
     pass
 
-def add_msg_niceties(recipient_name, body, sender_name, sender_url):
-    return _(u"Dear %s,") % recipient_name \
-           + u"\r\n\r\n%s\r\n\r\n" % body \
-           + u"--\r\n%s (%s)" % (sender_name, sender_url)
 
 def _mail_recipient(recipient_name, recipient_email,
-        sender_name, sender_url, subject,
-        body, headers={}):
+                    sender_name, sender_url, subject,
+                    body, headers=None):
+
+    if not headers:
+        headers = {}
+
     mail_from = config.get('smtp.mail_from')
-    body = add_msg_niceties(recipient_name, body, sender_name, sender_url)
     msg = MIMEText(body.encode('utf-8'), 'plain', 'utf-8')
-    for k, v in headers.items(): msg[k] = v
+    for k, v in headers.items():
+        if k in msg.keys():
+            msg.replace_header(k, v)
+        else:
+            msg.add_header(k, v)
     subject = Header(subject.encode('utf-8'), 'utf-8')
     msg['Subject'] = subject
     msg['From'] = _("%s <%s>") % (sender_name, mail_from)
@@ -53,13 +63,17 @@ def _mail_recipient(recipient_name, recipient_email,
     else:
         smtp_server = config.get('smtp.server', 'localhost')
         smtp_starttls = paste.deploy.converters.asbool(
-                config.get('smtp.starttls'))
+            config.get('smtp.starttls'))
         smtp_user = config.get('smtp.user')
         smtp_password = config.get('smtp.password')
-    smtp_connection.connect(smtp_server)
-    try:
-        #smtp_connection.set_debuglevel(True)
 
+    try:
+        smtp_connection.connect(smtp_server)
+    except socket.error as e:
+        log.exception(e)
+        raise MailerException('SMTP server could not be connected to: "%s" %s'
+                              % (smtp_server, e))
+    try:
         # Identify ourselves and prompt the server for supported features.
         smtp_connection.ehlo()
 
@@ -76,87 +90,112 @@ def _mail_recipient(recipient_name, recipient_email,
         # If 'smtp.user' is in CKAN config, try to login to SMTP server.
         if smtp_user:
             assert smtp_password, ("If smtp.user is configured then "
-                    "smtp.password must be configured as well.")
+                                   "smtp.password must be configured as well.")
             smtp_connection.login(smtp_user, smtp_password)
 
         smtp_connection.sendmail(mail_from, [recipient_email], msg.as_string())
         log.info("Sent email to {0}".format(recipient_email))
 
-    except smtplib.SMTPException, e:
+    except smtplib.SMTPException as e:
         msg = '%r' % e
         log.exception(msg)
         raise MailerException(msg)
     finally:
         smtp_connection.quit()
 
+
 def mail_recipient(recipient_name, recipient_email, subject,
-        body, headers={}):
+                   body, headers={}):
+    site_title = config.get('ckan.site_title')
+    site_url = config.get('ckan.site_url')
     return _mail_recipient(recipient_name, recipient_email,
-            g.site_title, g.site_url, subject, body, headers=headers)
+                           site_title, site_url, subject, body,
+                           headers=headers)
+
 
 def mail_user(recipient, subject, body, headers={}):
     if (recipient.email is None) or not len(recipient.email):
         raise MailerException(_("No recipient email address available!"))
     mail_recipient(recipient.display_name, recipient.email, subject,
-            body, headers=headers)
+                   body, headers=headers)
+
 
 def get_reset_link_body(user):
-    reset_link_message = _(
-    '''You have requested your password on %(site_title)s to be reset.
-
-    Please click the following link to confirm this request:
-
-       %(reset_link)s
-    ''')
-
-    d = {
+    extra_vars = {
         'reset_link': get_reset_link(user),
-        'site_title': g.site_title
-        }
-    return reset_link_message % d
-
-def get_invite_body(user):
-    invite_message = _(
-    '''You have been invited to %(site_title)s. A user has already been created to
-    you with the username %(user_name)s. You can change it later.
-
-    To accept this invite, please reset your password at:
-
-       %(reset_link)s
-    ''')
-
-    d = {
-        'reset_link': get_reset_link(user),
-        'site_title': g.site_title,
+        'site_title': config.get('ckan.site_title'),
+        'site_url': config.get('ckan.site_url'),
         'user_name': user.name,
         }
-    return invite_message % d
+    # NOTE: This template is translated
+    return render_jinja2('emails/reset_password.txt', extra_vars)
+
+
+def get_invite_body(user, group_dict=None, role=None):
+    if group_dict:
+        group_type = (_('organization') if group_dict['is_organization']
+                      else _('group'))
+
+    extra_vars = {
+        'reset_link': get_reset_link(user),
+        'site_title': config.get('ckan.site_title'),
+        'site_url': config.get('ckan.site_url'),
+        'user_name': user.name,
+        }
+    if role:
+        extra_vars['role_name'] = h.roles_translated().get(role, _(role))
+    if group_dict:
+        extra_vars['group_type'] = group_type
+        extra_vars['group_title'] = group_dict.get('title')
+
+    # NOTE: This template is translated
+    return render_jinja2('emails/invite_user.txt', extra_vars)
+
 
 def get_reset_link(user):
-    return urljoin(g.site_url,
-                   h.url_for(controller='user',
-                           action='perform_reset',
-                           id=user.id,
-                           key=user.reset_key))
+    return h.url_for(controller='user',
+                     action='perform_reset',
+                     id=user.id,
+                     key=user.reset_key,
+                     qualified=True)
+
 
 def send_reset_link(user):
     create_reset_key(user)
     body = get_reset_link_body(user)
-    subject = _('Reset your password')
+    extra_vars = {
+        'site_title': config.get('ckan.site_title')
+    }
+    subject = render_jinja2('emails/reset_password_subject.txt', extra_vars)
+
+    # Make sure we only use the first line
+    subject = subject.split('\n')[0]
+
     mail_user(user, subject, body)
 
-def send_invite(user):
+
+def send_invite(user, group_dict=None, role=None):
     create_reset_key(user)
-    body = get_invite_body(user)
-    subject = _('Invite for {site_title}'.format(site_title=g.site_title))
+    body = get_invite_body(user, group_dict, role)
+    extra_vars = {
+        'site_title': config.get('ckan.site_title')
+    }
+    subject = render_jinja2('emails/invite_user_subject.txt', extra_vars)
+
+    # Make sure we only use the first line
+    subject = subject.split('\n')[0]
+
     mail_user(user, subject, body)
 
+
 def create_reset_key(user):
-    user.reset_key = unicode(make_key())
+    user.reset_key = text_type(make_key())
     model.repo.commit_and_remove()
 
+
 def make_key():
-    return uuid.uuid4().hex[:10]
+    return codecs.encode(os.urandom(16), 'hex')
+
 
 def verify_reset_link(user, key):
     if not key:
@@ -164,6 +203,3 @@ def verify_reset_link(user, key):
     if not user.reset_key or len(user.reset_key) < 5:
         return False
     return key.strip() == user.reset_key
-
-
-
