@@ -1,12 +1,16 @@
-import logging
+# encoding: utf-8
 
-from pylons import c
+import logging
+import os
+import sys
+
+from ckan.common import c
 from ckan.lib import base
-import ckan.lib.maintain as maintain
 from ckan import logic
 import logic.schema
 from ckan import plugins
-import ckan.new_authz
+import ckan.authz
+import ckan.plugins.toolkit as toolkit
 
 log = logging.getLogger(__name__)
 
@@ -15,10 +19,13 @@ _package_plugins = {}
 # The fallback behaviour
 _default_package_plugin = None
 
-# Mapping from group-type strings to IDatasetForm instances
+# Mapping from group-type strings to IGroupForm instances
 _group_plugins = {}
 # The fallback behaviour
 _default_group_plugin = None
+_default_organization_plugin = None
+# Mapping from group-type strings to controllers
+_group_controllers = {}
 
 
 def reset_package_plugins():
@@ -28,8 +35,12 @@ def reset_package_plugins():
     _package_plugins = {}
     global _default_group_plugin
     _default_group_plugin = None
+    global _default_organization_plugin
+    _default_organization_plugin = None
     global _group_plugins
     _group_plugins = {}
+    global _group_controllers
+    _group_controllers = {}
 
 
 def lookup_package_plugin(package_type=None):
@@ -46,7 +57,7 @@ def lookup_package_plugin(package_type=None):
 
 def lookup_group_plugin(group_type=None):
     """
-    Returns the plugin controller associoated with the given group type.
+    Returns the form plugin associated with the given group type.
 
     If the group type is None or cannot be found in the mapping, then the
     fallback behaviour is used.
@@ -57,6 +68,14 @@ def lookup_group_plugin(group_type=None):
         if group_type == 'organization' else _default_group_plugin)
 
 
+def lookup_group_controller(group_type=None):
+    """
+    Returns the group controller associated with the given group type. The
+    controller is expressed as a string that you'd pass to url_to(controller=x)
+    """
+    return _group_controllers.get(group_type)
+
+
 def register_package_plugins(map):
     """
     Register the various IDatasetForm instances.
@@ -77,8 +96,8 @@ def register_package_plugins(map):
     for plugin in plugins.PluginImplementations(plugins.IDatasetForm):
         if plugin.is_fallback():
             if _default_package_plugin is not None:
-                raise ValueError, "More than one fallback "\
-                                  "IDatasetForm has been registered"
+                raise ValueError("More than one fallback "
+                                 "IDatasetForm has been registered")
             _default_package_plugin = plugin
 
         for package_type in plugin.package_types():
@@ -95,15 +114,14 @@ def register_package_plugins(map):
 
             for action in ['edit', 'authz', 'history']:
                 map.connect('%s_%s' % (package_type, action),
-                        '/%s/%s/{id}' % (package_type, action),
-                        controller='package',
-                        action=action
-                )
+                            '/%s/%s/{id}' % (package_type, action),
+                            controller='package',
+                            action=action)
 
             if package_type in _package_plugins:
-                raise ValueError, "An existing IDatasetForm is "\
-                                  "already associated with the package type "\
-                                  "'%s'" % package_type
+                raise ValueError("An existing IDatasetForm is "
+                                 "already associated with the package type "
+                                 "'%s'" % package_type)
             _package_plugins[package_type] = plugin
 
     # Setup the fallback behaviour if one hasn't been defined.
@@ -118,22 +136,46 @@ def register_group_plugins(map):
     This method will setup the mappings between group types and the
     registered IGroupForm instances. If it's called more than once an
     exception will be raised.
+
+    It will register IGroupForm instances for both groups and organizations
     """
     global _default_group_plugin
+    global _default_organization_plugin
 
     # This function should have not effect if called more than once.
     # This should not occur in normal deployment, but it may happen when
     # running unit tests.
-    if _default_group_plugin is not None:
+    if (_default_group_plugin is not None or
+            _default_organization_plugin is not None):
         return
 
     # Create the mappings and register the fallback behaviour if one is found.
     for plugin in plugins.PluginImplementations(plugins.IGroupForm):
+
+        # Get group_controller from plugin if there is one,
+        # otherwise use 'group'
+        try:
+            group_controller = plugin.group_controller()
+        except AttributeError:
+            group_controller = 'group'
+
         if plugin.is_fallback():
-            if _default_group_plugin is not None:
-                raise ValueError, "More than one fallback "\
-                                  "IGroupForm has been registered"
-            _default_group_plugin = plugin
+            if hasattr(plugin, 'is_organization'):
+                is_organization = plugin.is_organization
+            else:
+                is_organization = group_controller == 'organization'
+
+            if is_organization:
+                if _default_organization_plugin is not None:
+                    raise ValueError("More than one fallback IGroupForm for "
+                                     "organizations has been registered")
+                _default_organization_plugin = plugin
+
+            else:
+                if _default_group_plugin is not None:
+                    raise ValueError("More than one fallback IGroupForm for "
+                                     "groups has been registered")
+                _default_group_plugin = plugin
 
         for group_type in plugin.group_types():
             # Create the routes based on group_type here, this will
@@ -148,25 +190,93 @@ def register_group_plugins(map):
             # routing setup
 
             map.connect('%s_index' % group_type, '/%s' % group_type,
-                        controller='group', action='index')
+                        controller=group_controller, action='index')
             map.connect('%s_new' % group_type, '/%s/new' % group_type,
-                        controller='group', action='new')
+                        controller=group_controller, action='new')
             map.connect('%s_read' % group_type, '/%s/{id}' % group_type,
-                        controller='group', action='read')
+                        controller=group_controller, action='read')
             map.connect('%s_action' % group_type,
-                        '/%s/{action}/{id}' % group_type, controller='group',
-                requirements=dict(action='|'.join(['edit', 'authz', 'history']))
-            )
+                        '/%s/{action}/{id}' % group_type,
+                        controller=group_controller,
+                        requirements=dict(action='|'.join(
+                            ['edit', 'authz', 'delete', 'history', 'member_new',
+                             'member_delete', 'followers', 'follow',
+                             'unfollow', 'admins', 'activity'])))
+            map.connect('%s_edit' % group_type, '/%s/edit/{id}' % group_type,
+                        controller=group_controller, action='edit',
+                        ckan_icon='pencil-square-o')
+            map.connect('%s_members' % group_type,
+                        '/%s/members/{id}' % group_type,
+                        controller=group_controller,
+                        action='members',
+                        ckan_icon='users')
+            map.connect('%s_member_new' % group_type,
+                        '/%s/member_new/{id}' % group_type,
+                        controller=group_controller,
+                        action='member_new')
+            map.connect('%s_member_delete' % group_type,
+                        '/%s/member_delete/{id}' % group_type,
+                        controller=group_controller,
+                        action='member_delete')
+            map.connect('%s_activity' % group_type,
+                        '/%s/activity/{id}/{offset}' % group_type,
+                        controller=group_controller,
+                        action='activity', ckan_icon='clock-o'),
+            map.connect('%s_about' % group_type, '/%s/about/{id}' % group_type,
+                        controller=group_controller,
+                        action='about', ckan_icon='info-circle')
+            map.connect('%s_bulk_process' % group_type,
+                        '/%s/bulk_process/{id}' % group_type,
+                        controller=group_controller,
+                        action='bulk_process', ckan_icon='sitemap')
 
             if group_type in _group_plugins:
-                raise ValueError, "An existing IGroupForm is "\
-                                  "already associated with the group type "\
-                                  "'%s'" % group_type
+                raise ValueError("An existing IGroupForm is "
+                                 "already associated with the group type "
+                                 "'%s'" % group_type)
             _group_plugins[group_type] = plugin
+            _group_controllers[group_type] = group_controller
+
+            controller_obj = None
+            # If using one of the default controllers, tell it that it is allowed
+            # to handle other group_types.
+            # Import them here to avoid circular imports.
+            if group_controller == 'group':
+                from ckan.controllers.group import GroupController as controller_obj
+            elif group_controller == 'organization':
+                from ckan.controllers.organization import OrganizationController as controller_obj
+            if controller_obj is not None:
+                controller_obj.add_group_type(group_type)
 
     # Setup the fallback behaviour if one hasn't been defined.
     if _default_group_plugin is None:
         _default_group_plugin = DefaultGroupForm()
+    if _default_organization_plugin is None:
+        _default_organization_plugin = DefaultOrganizationForm()
+    if 'group' not in _group_controllers:
+        _group_controllers['group'] = 'group'
+    if 'organization' not in _group_controllers:
+        _group_controllers['organization'] = 'organization'
+
+
+def plugin_validate(plugin, context, data_dict, schema, action):
+    """
+    Backwards compatibility with 2.x dataset group and org plugins:
+    return a default validate method if one has not been provided.
+    """
+    if hasattr(plugin, 'validate'):
+        result = plugin.validate(context, data_dict, schema, action)
+        if result is not None:
+            return result
+
+    return toolkit.navl_validate(data_dict, schema, context)
+
+
+def get_permission_labels():
+    '''Return the permission label plugin (or default implementation)'''
+    for plugin in plugins.PluginImplementations(plugins.IPermissionLabels):
+        return plugin
+    return DefaultPermissionLabels()
 
 
 class DefaultDatasetForm(object):
@@ -209,13 +319,16 @@ class DefaultDatasetForm(object):
         c.groups_available = authz_fn(context, data_dict)
 
         c.licenses = [('', '')] + base.model.Package.get_license_options()
-        # CS: bad_spelling ignore 2 lines
-        c.licences = c.licenses
-        maintain.deprecate_context_item('licences', 'Use `c.licenses` instead')
-        c.is_sysadmin = ckan.new_authz.is_sysadmin(c.user)
+        c.is_sysadmin = ckan.authz.is_sysadmin(c.user)
 
-        if c.pkg:
-            c.related_count = c.pkg.related_count
+        if context.get('revision_id') or context.get('revision_date'):
+            if context.get('revision_id'):
+                rev = base.model.Session.query(base.model.Revision) \
+                                .filter_by(id=context['revision_id']) \
+                                .first()
+                c.revision_date = rev.timestamp if rev else '?'
+            else:
+                c.revision_date = context.get('revision_date')
 
         ## This is messy as auths take domain object not data_dict
         context_pkg = context.get('package', None)
@@ -244,9 +357,15 @@ class DefaultDatasetForm(object):
     def history_template(self):
         return 'package/history.html'
 
+    def resource_template(self):
+        return 'package/resource_read.html'
+
     def package_form(self):
         return 'package/new_package_form.html'
 
+    def resource_form(self):
+        return 'package/snippets/resource_form.html'
+
 
 class DefaultGroupForm(object):
     """
@@ -264,6 +383,9 @@ class DefaultGroupForm(object):
     Note - this isn't a plugin implementation. This is deliberate, as we
            don't want this being registered.
     """
+    def group_controller(self):
+        return 'group'
+
     def new_template(self):
         """
         Returns a string representing the location of the template to be
@@ -327,14 +449,6 @@ class DefaultGroupForm(object):
         """
         return 'group/bulk_process.html'
 
-    def about_template(self):
-        '''Return the path to the template for the group's 'about' page.
-
-        :rtype: string
-
-        '''
-        return 'group/about.html'
-
     def group_form(self):
         return 'group/new_group_form.html'
 
@@ -372,7 +486,7 @@ class DefaultGroupForm(object):
         into a format suitable for the form (optional)'''
 
     def db_to_form_schema_options(self, options):
-        '''This allows the selectino of different schemas for different
+        '''This allows the selection of different schemas for different
         purposes.  It is optional and if not available, ``db_to_form_schema``
         should be used.
         If a context is provided, and it contains a schema, it will be
@@ -406,7 +520,7 @@ class DefaultGroupForm(object):
         pass
 
     def setup_template_variables(self, context, data_dict):
-        c.is_sysadmin = ckan.new_authz.is_sysadmin(c.user)
+        c.is_sysadmin = ckan.authz.is_sysadmin(c.user)
 
         ## This is messy as auths take domain object not data_dict
         context_group = context.get('group', None)
@@ -422,6 +536,9 @@ class DefaultGroupForm(object):
 
 
 class DefaultOrganizationForm(DefaultGroupForm):
+    def group_controller(self):
+        return 'organization'
+
     def group_form(self):
         return 'organization/new_organization_form.html'
 
@@ -454,4 +571,66 @@ class DefaultOrganizationForm(DefaultGroupForm):
     def activity_template(self):
         return 'organization/activity_stream.html'
 
-_default_organization_plugin = DefaultOrganizationForm()
+
+class DefaultTranslation(object):
+    def i18n_directory(self):
+        '''Change the directory of the *.mo translation files
+
+        The default implementation assumes the plugin is
+        ckanext/myplugin/plugin.py and the translations are stored in
+        i18n/
+        '''
+        # assume plugin is called ckanext.<myplugin>.<...>.PluginClass
+        extension_module_name = '.'.join(self.__module__.split('.')[:3])
+        module = sys.modules[extension_module_name]
+        return os.path.join(os.path.dirname(module.__file__), 'i18n')
+
+    def i18n_locales(self):
+        '''Change the list of locales that this plugin handles
+
+        By default the will assume any directory in subdirectory in the
+        directory defined by self.directory() is a locale handled by this
+        plugin
+        '''
+        directory = self.i18n_directory()
+        return [ d for
+                 d in os.listdir(directory)
+                 if os.path.isdir(os.path.join(directory, d))
+        ]
+
+    def i18n_domain(self):
+        '''Change the gettext domain handled by this plugin
+
+        This implementation assumes the gettext domain is
+        ckanext-{extension name}, hence your pot, po and mo files should be
+        named ckanext-{extension name}.mo'''
+        return 'ckanext-{name}'.format(name=self.name)
+
+
+class DefaultPermissionLabels(object):
+    u'''
+    Default permissions for package_search/package_show:
+    - everyone can read public datasets "public"
+    - users can read their own drafts "creator-(user id)"
+    - users can read datasets belonging to their orgs "member-(org id)"
+    '''
+    def get_dataset_labels(self, dataset_obj):
+        if dataset_obj.state == u'active' and not dataset_obj.private:
+            return [u'public']
+
+        if dataset_obj.owner_org:
+            return [u'member-%s' % dataset_obj.owner_org]
+
+        return [u'creator-%s' % dataset_obj.creator_user_id]
+
+    def get_user_dataset_labels(self, user_obj):
+        labels = [u'public']
+        if not user_obj:
+            return labels
+
+        labels.append(u'creator-%s' % user_obj.id)
+
+        orgs = logic.get_action(u'organization_list_for_user')(
+            {u'user': user_obj.id}, {u'permission': u'read'})
+        labels.extend(u'member-%s' % o[u'id'] for o in orgs)
+        return labels
