@@ -1,3 +1,5 @@
+# encoding: utf-8
+
 import socket
 import string
 import logging
@@ -8,10 +10,10 @@ from dateutil.parser import parse
 
 import re
 
-import solr
-
-from pylons import config
+import pysolr
+from ckan.common import config
 from paste.deploy.converters import asbool
+from six import text_type
 
 from common import SearchIndexError, make_connection
 from ckan.model import PackageRelationship
@@ -24,14 +26,12 @@ import ckan.lib.navl.dictization_functions
 
 log = logging.getLogger(__name__)
 
-_validate = ckan.lib.navl.dictization_functions.validate
-
 TYPE_FIELD = "entity_type"
 PACKAGE_TYPE = "package"
 KEY_CHARS = string.digits + string.letters + "_-"
 SOLR_FIELDS = [TYPE_FIELD, "res_url", "text", "urls", "indexed_ts", "site_id"]
-RESERVED_FIELDS = SOLR_FIELDS + ["tags", "groups", "res_description",
-                                 "res_format", "res_url"]
+RESERVED_FIELDS = SOLR_FIELDS + ["tags", "groups", "res_name", "res_description",
+                                 "res_format", "res_url", "res_type"]
 RELATIONSHIP_TYPES = PackageRelationship.types
 
 # Regular expression used to strip invalid XML characters
@@ -47,22 +47,19 @@ def escape_xml_illegal_chars(val, replacement=''):
 
 
 def clear_index():
-    import solr.core
     conn = make_connection()
     query = "+site_id:\"%s\"" % (config.get('ckan.site_id'))
     try:
-        conn.delete_query(query)
-        conn.commit()
-    except socket.error, e:
+        conn.delete(q=query)
+    except socket.error as e:
         err = 'Could not connect to SOLR %r: %r' % (conn.url, e)
         log.error(err)
         raise SearchIndexError(err)
-    except solr.core.SolrException, e:
+    except pysolr.SolrError as e:
         err = 'SOLR %r exception: %r' % (conn.url, e)
         log.error(err)
         raise SearchIndexError(err)
-    finally:
-        conn.close()
+
 
 class SearchIndex(object):
     """
@@ -108,24 +105,33 @@ class PackageSearchIndex(SearchIndex):
         if pkg_dict is None:
             return
 
+        # tracking summary values will be stale, never store them
+        tracking_summary = pkg_dict.pop('tracking_summary', None)
+        for r in pkg_dict.get('resources', []):
+            r.pop('tracking_summary', None)
+
+        data_dict_json = json.dumps(pkg_dict)
+
         if config.get('ckan.cache_validated_datasets', True):
             package_plugin = lib_plugins.lookup_package_plugin(
                 pkg_dict.get('type'))
 
             schema = package_plugin.show_package_schema()
-            validated_pkg_dict, errors = _validate(pkg_dict, schema, {
-                'model': model, 'session': model.Session})
+            validated_pkg_dict, errors = lib_plugins.plugin_validate(
+                package_plugin, {'model': model, 'session': model.Session},
+                pkg_dict, schema, 'package_show')
             pkg_dict['validated_data_dict'] = json.dumps(validated_pkg_dict,
                 cls=ckan.lib.navl.dictization_functions.MissingNullEncoder)
 
-        pkg_dict['data_dict'] = json.dumps(pkg_dict)
+        pkg_dict['data_dict'] = data_dict_json
 
         # add to string field for sorting
         title = pkg_dict.get('title')
         if title:
             pkg_dict['title_string'] = title
 
-        if (not pkg_dict.get('state')) or ('active' not in pkg_dict.get('state')):
+        # delete the package if there is no state, or the state is `deleted`
+        if (not pkg_dict.get('state') or 'deleted' in pkg_dict.get('state')):
             return self.delete_package(pkg_dict)
 
         index_fields = RESERVED_FIELDS + pkg_dict.keys()
@@ -135,7 +141,7 @@ class PackageSearchIndex(SearchIndex):
         for extra in extras:
             key, value = extra['key'], extra['value']
             if isinstance(value, (tuple, list)):
-                value = " ".join(map(unicode, value))
+                value = " ".join(map(text_type, value))
             key = ''.join([c for c in key if c in KEY_CHARS])
             pkg_dict['extras_' + key] = value
             if key not in index_fields:
@@ -181,16 +187,22 @@ class PackageSearchIndex(SearchIndex):
            pkg_dict['organization'] = None
 
         # tracking
-        tracking_summary = pkg_dict.pop('tracking_summary', None)
-        if tracking_summary:
-            pkg_dict['views_total'] = tracking_summary['total']
-            pkg_dict['views_recent'] = tracking_summary['recent']
+        if not tracking_summary:
+            tracking_summary = model.TrackingSummary.get_for_package(
+                pkg_dict['id'])
+        pkg_dict['views_total'] = tracking_summary['total']
+        pkg_dict['views_recent'] = tracking_summary['recent']
 
+        resource_fields = [('name', 'res_name'),
+                           ('description', 'res_description'),
+                           ('format', 'res_format'),
+                           ('url', 'res_url'),
+                           ('resource_type', 'res_type')]
+        resource_extras = [(e, 'res_extras_' + e) for e
+                            in model.Resource.get_extra_columns()]
         # flatten the structure for indexing:
         for resource in pkg_dict.get('resources', []):
-            for (okey, nkey) in [('description', 'res_description'),
-                                 ('format', 'res_format'),
-                                 ('url', 'res_url')]:
+            for (okey, nkey) in resource_fields + resource_extras:
                 pkg_dict[nkey] = pkg_dict.get(nkey, []) + [resource.get(okey, u'')]
         pkg_dict.pop('resources', None)
 
@@ -268,50 +280,48 @@ class PackageSearchIndex(SearchIndex):
 
         assert pkg_dict, 'Plugin must return non empty package dict on index'
 
+        # permission labels determine visibility in search, can't be set
+        # in original dataset or before_index plugins
+        labels = lib_plugins.get_permission_labels()
+        dataset = model.Package.get(pkg_dict['id'])
+        pkg_dict['permission_labels'] = labels.get_dataset_labels(
+            dataset) if dataset else [] # TestPackageSearchIndex-workaround
+
         # send to solr:
         try:
             conn = make_connection()
             commit = not defer_commit
             if not asbool(config.get('ckan.search.solr_commit', 'true')):
                 commit = False
-            conn.add_many([pkg_dict], _commit=commit)
-        except solr.core.SolrException, e:
-            msg = 'Solr returned an error: {0} {1} - {2}'.format(
-                e.httpcode, e.reason, e.body[:1000] # limit huge responses
+            conn.add(docs=[pkg_dict], commit=commit)
+        except pysolr.SolrError as e:
+            msg = 'Solr returned an error: {0}'.format(
+                e[:1000] # limit huge responses
             )
             raise SearchIndexError(msg)
-        except socket.error, e:
+        except socket.error as e:
             err = 'Could not connect to Solr using {0}: {1}'.format(conn.url, str(e))
             log.error(err)
             raise SearchIndexError(err)
-        finally:
-            conn.close()
 
-        commit_debug_msg = 'Not commited yet' if defer_commit else 'Commited'
+        commit_debug_msg = 'Not committed yet' if defer_commit else 'Committed'
         log.debug('Updated index for %s [%s]' % (pkg_dict.get('name'), commit_debug_msg))
 
     def commit(self):
         try:
             conn = make_connection()
-            conn.commit(wait_searcher=False)
-        except Exception, e:
+            conn.commit(waitSearcher=False)
+        except Exception as e:
             log.exception(e)
             raise SearchIndexError(e)
-        finally:
-            conn.close()
-
 
     def delete_package(self, pkg_dict):
         conn = make_connection()
-        query = "+%s:%s (+id:\"%s\" OR +name:\"%s\") +site_id:\"%s\"" % (TYPE_FIELD, PACKAGE_TYPE,
-                                                       pkg_dict.get('id'), pkg_dict.get('id'),
-                                                       config.get('ckan.site_id'))
+        query = "+%s:%s AND +(id:\"%s\" OR name:\"%s\") AND +site_id:\"%s\"" % \
+                (TYPE_FIELD, PACKAGE_TYPE, pkg_dict.get('id'), pkg_dict.get('id'), config.get('ckan.site_id'))
         try:
-            conn.delete_query(query)
-            if asbool(config.get('ckan.search.solr_commit', 'true')):
-                conn.commit()
-        except Exception, e:
+            commit = asbool(config.get('ckan.search.solr_commit', 'true'))
+            conn.delete(q=query, commit=commit)
+        except Exception as e:
             log.exception(e)
             raise SearchIndexError(e)
-        finally:
-            conn.close()
