@@ -1,13 +1,19 @@
+# encoding: utf-8
+
 '''API functions for updating existing data in CKAN.'''
 
 import logging
 import datetime
+import time
 import json
+import mimetypes
+import os
 
-from pylons import config
-from vdm.sqlalchemy.base import SQLAlchemySession
+from ckan.common import config
 import paste.deploy.converters as converters
+from six import text_type
 
+import ckan.lib.helpers as h
 import ckan.plugins as plugins
 import ckan.logic as logic
 import ckan.logic.schema as schema_
@@ -20,6 +26,9 @@ import ckan.lib.plugins as lib_plugins
 import ckan.lib.email_notifications as email_notifications
 import ckan.lib.search as search
 import ckan.lib.uploader as uploader
+import ckan.lib.datapreview
+import ckan.lib.app_globals as app_globals
+
 
 from ckan.common import _, request
 
@@ -35,151 +44,6 @@ NotFound = logic.NotFound
 ValidationError = logic.ValidationError
 _get_or_bust = logic.get_or_bust
 
-def _make_latest_rev_active(context, q):
-
-    session = context['model'].Session
-
-    old_current = q.filter_by(current=True).first()
-    if old_current:
-        old_current.current = False
-        session.add(old_current)
-
-    latest_rev = q.filter_by(expired_timestamp=datetime.datetime(9999, 12, 31)).one()
-    latest_rev.current = True
-    if latest_rev.state in ('pending-deleted', 'deleted'):
-        latest_rev.state = 'deleted'
-        latest_rev.continuity.state = 'deleted'
-    else:
-        latest_rev.continuity.state = 'active'
-        latest_rev.state = 'active'
-
-    session.add(latest_rev)
-
-    ##this is just a way to get the latest revision that changed
-    ##in order to timestamp
-    old_latest = context.get('latest_revision_date')
-    if old_latest:
-        if latest_rev.revision_timestamp > old_latest:
-            context['latest_revision_date'] = latest_rev.revision_timestamp
-            context['latest_revision'] = latest_rev.revision_id
-    else:
-        context['latest_revision_date'] = latest_rev.revision_timestamp
-        context['latest_revision'] = latest_rev.revision_id
-
-def make_latest_pending_package_active(context, data_dict):
-    '''TODO: What does this function do?
-
-    You must be authorized to update the dataset.
-
-    :param id: the name or id of the dataset, e.g. ``'warandpeace'``
-    :type id: string
-
-    '''
-    model = context['model']
-    session = model.Session
-    SQLAlchemySession.setattr(session, 'revisioning_disabled', True)
-    id = _get_or_bust(data_dict, "id")
-    pkg = model.Package.get(id)
-
-    _check_access('make_latest_pending_package_active', context, data_dict)
-
-    #packages
-    q = session.query(model.PackageRevision).filter_by(id=pkg.id)
-    _make_latest_rev_active(context, q)
-
-    #resources
-    for resource in pkg.resource_groups_all[0].resources_all:
-        q = session.query(model.ResourceRevision).filter_by(id=resource.id)
-        _make_latest_rev_active(context, q)
-
-    #tags
-    for tag in pkg.package_tag_all:
-        q = session.query(model.PackageTagRevision).filter_by(id=tag.id)
-        _make_latest_rev_active(context, q)
-
-    #extras
-    for extra in pkg.extras_list:
-        q = session.query(model.PackageExtraRevision).filter_by(id=extra.id)
-        _make_latest_rev_active(context, q)
-
-    latest_revision = context.get('latest_revision')
-    if not latest_revision:
-        return
-
-    q = session.query(model.Revision).filter_by(id=latest_revision)
-    revision = q.first()
-    revision.approved_timestamp = datetime.datetime.now()
-    session.add(revision)
-
-    if not context.get('defer_commit'):
-        session.commit()
-
-
-def related_update(context, data_dict):
-    '''Update a related item.
-
-    You must be the owner of a related item to update it.
-
-    For further parameters see ``related_create()``.
-
-    :param id: the id of the related item to update
-    :type id: string
-
-    :returns: the updated related item
-    :rtype: dictionary
-
-    '''
-    model = context['model']
-    id = _get_or_bust(data_dict, "id")
-
-    session = context['session']
-    schema = context.get('schema') or schema_.default_update_related_schema()
-
-    related = model.Related.get(id)
-    context["related"] = related
-
-    if not related:
-        logging.error('Could not find related ' + id)
-        raise NotFound(_('Item was not found.'))
-
-    _check_access('related_update', context, data_dict)
-    data, errors = _validate(data_dict, schema, context)
-    if errors:
-        model.Session.rollback()
-        raise ValidationError(errors)
-
-    related = model_save.related_dict_save(data, context)
-
-    dataset_dict = None
-    if 'package' in context:
-        dataset = context['package']
-        dataset_dict = ckan.lib.dictization.table_dictize(dataset, context)
-
-    related_dict = model_dictize.related_dictize(related, context)
-    activity_dict = {
-        'user_id': context['user'],
-        'object_id': related.id,
-        'activity_type': 'changed related item',
-    }
-    activity_dict['data'] = {
-        'related': related_dict,
-        'dataset': dataset_dict,
-    }
-    activity_create_context = {
-        'model': model,
-        'user': context['user'],
-        'defer_commit': True,
-        'ignore_auth': True,
-        'session': session
-    }
-
-    _get_action('activity_create')(activity_create_context, activity_dict)
-
-    if not context.get('defer_commit'):
-        model.repo.commit()
-    return model_dictize.related_dictize(related, context)
-
-
 
 def resource_update(context, data_dict):
     '''Update a resource.
@@ -187,7 +51,8 @@ def resource_update(context, data_dict):
     To update a resource you must be authorized to update the dataset that the
     resource belongs to.
 
-    For further parameters see ``resource_create()``.
+    For further parameters see
+    :py:func:`~ckan.logic.action.create.resource_create`.
 
     :param id: the id of the resource to update
     :type id: string
@@ -200,42 +65,167 @@ def resource_update(context, data_dict):
     user = context['user']
     id = _get_or_bust(data_dict, "id")
 
+    if not data_dict.get('url'):
+        data_dict['url'] = ''
+
     resource = model.Resource.get(id)
     context["resource"] = resource
+    old_resource_format = resource.format
 
     if not resource:
-        logging.error('Could not find resource ' + id)
+        log.debug('Could not find resource %s', id)
         raise NotFound(_('Resource was not found.'))
 
     _check_access('resource_update', context, data_dict)
     del context["resource"]
 
-    package_id = resource.resource_group.package.id
-    pkg_dict = _get_action('package_show')(context, {'id': package_id})
+    package_id = resource.package.id
+    pkg_dict = _get_action('package_show')(dict(context, return_type='dict'),
+        {'id': package_id})
 
     for n, p in enumerate(pkg_dict['resources']):
         if p['id'] == id:
             break
     else:
-        logging.error('Could not find resource ' + id)
+        log.error('Could not find resource %s after all', id)
         raise NotFound(_('Resource was not found.'))
 
-    upload = uploader.ResourceUpload(data_dict)
+    # Persist the datastore_active extra if already present and not provided
+    if ('datastore_active' in resource.extras and
+            'datastore_active' not in data_dict):
+        data_dict['datastore_active'] = resource.extras['datastore_active']
+
+    for plugin in plugins.PluginImplementations(plugins.IResourceController):
+        plugin.before_update(context, pkg_dict['resources'][n], data_dict)
+
+    upload = uploader.get_resource_uploader(data_dict)
+
+    if 'mimetype' not in data_dict:
+        if hasattr(upload, 'mimetype'):
+            data_dict['mimetype'] = upload.mimetype
+
+    if 'size' not in data_dict and 'url_type' in data_dict:
+        if hasattr(upload, 'filesize'):
+            data_dict['size'] = upload.filesize
 
     pkg_dict['resources'][n] = data_dict
 
     try:
         context['defer_commit'] = True
         context['use_cache'] = False
-        pkg_dict = _get_action('package_update')(context, pkg_dict)
+        updated_pkg_dict = _get_action('package_update')(context, pkg_dict)
         context.pop('defer_commit')
-    except ValidationError, e:
-        errors = e.error_dict['resources'][n]
-        raise ValidationError(errors)
+    except ValidationError as e:
+        try:
+            raise ValidationError(e.error_dict['resources'][n])
+        except (KeyError, IndexError):
+            raise ValidationError(e.error_dict)
 
     upload.upload(id, uploader.get_max_resource_size())
     model.repo.commit()
-    return _get_action('resource_show')(context, {'id': id})
+
+    resource = _get_action('resource_show')(context, {'id': id})
+
+    if old_resource_format != resource['format']:
+        _get_action('resource_create_default_resource_views')(
+            {'model': context['model'], 'user': context['user'],
+             'ignore_auth': True},
+            {'package': updated_pkg_dict,
+             'resource': resource})
+
+    for plugin in plugins.PluginImplementations(plugins.IResourceController):
+        plugin.after_update(context, resource)
+
+    return resource
+
+
+def resource_view_update(context, data_dict):
+    '''Update a resource view.
+
+    To update a resource_view you must be authorized to update the resource
+    that the resource_view belongs to.
+
+    For further parameters see ``resource_view_create()``.
+
+    :param id: the id of the resource_view to update
+    :type id: string
+
+    :returns: the updated resource_view
+    :rtype: string
+
+    '''
+    model = context['model']
+    id = _get_or_bust(data_dict, "id")
+
+    resource_view = model.ResourceView.get(id)
+    if not resource_view:
+        raise NotFound
+
+    view_plugin = ckan.lib.datapreview.get_view_plugin(resource_view.view_type)
+    schema = (context.get('schema') or
+              schema_.default_update_resource_view_schema(view_plugin))
+    plugin_schema = view_plugin.info().get('schema', {})
+    schema.update(plugin_schema)
+
+    data, errors = _validate(data_dict, schema, context)
+    if errors:
+        model.Session.rollback()
+        raise ValidationError(errors)
+
+    context['resource_view'] = resource_view
+    context['resource'] = model.Resource.get(resource_view.resource_id)
+
+    _check_access('resource_view_update', context, data_dict)
+
+    if context.get('preview'):
+        return data
+
+    resource_view = model_save.resource_view_dict_save(data, context)
+    if not context.get('defer_commit'):
+        model.repo.commit()
+    return model_dictize.resource_view_dictize(resource_view, context)
+
+def resource_view_reorder(context, data_dict):
+    '''Reorder resource views.
+
+    :param id: the id of the resource
+    :type id: string
+    :param order: the list of id of the resource to update the order of the views
+    :type order: list of strings
+
+    :returns: the updated order of the view
+    :rtype: dictionary
+    '''
+    model = context['model']
+    id, order = _get_or_bust(data_dict, ["id", "order"])
+    if not isinstance(order, list):
+        raise ValidationError({"order": "Must supply order as a list"})
+    if len(order) != len(set(order)):
+        raise ValidationError({"order": "No duplicates allowed in order"})
+    resource = model.Resource.get(id)
+    context['resource'] = resource
+
+    _check_access('resource_view_reorder', context, data_dict)
+
+    q = model.Session.query(model.ResourceView.id).filter_by(resource_id=id)
+    existing_views = [res[0] for res in
+                      q.order_by(model.ResourceView.order).all()]
+    ordered_views = []
+    for view in order:
+        try:
+            existing_views.remove(view)
+            ordered_views.append(view)
+        except ValueError:
+            raise ValidationError(
+                {"order": "View {view} does not exist".format(view=view)}
+            )
+    new_order = ordered_views + existing_views
+
+    for num, view in enumerate(new_order):
+        model.Session.query(model.ResourceView).\
+            filter_by(id=view).update({"order": num + 1})
+    model.Session.commit()
+    return {'id': id, 'order': new_order}
 
 
 def package_update(context, data_dict):
@@ -244,30 +234,38 @@ def package_update(context, data_dict):
     You must be authorized to edit the dataset and the groups that it belongs
     to.
 
-    Plugins may change the parameters of this function depending on the value
-    of the dataset's ``type`` attribute, see the ``IDatasetForm`` plugin
-    interface.
+    It is recommended to call
+    :py:func:`ckan.logic.action.get.package_show`, make the desired changes to
+    the result, and then call ``package_update()`` with it.
 
-    For further parameters see ``package_create()``.
+    Plugins may change the parameters of this function depending on the value
+    of the dataset's ``type`` attribute, see the
+    :py:class:`~ckan.plugins.interfaces.IDatasetForm` plugin interface.
+
+    For further parameters see
+    :py:func:`~ckan.logic.action.create.package_create`.
 
     :param id: the name or id of the dataset to update
     :type id: string
 
-    :returns: the updated dataset (if 'return_package_dict' is True in the
-              context, which is the default. Otherwise returns just the
+    :returns: the updated dataset (if ``'return_package_dict'`` is ``True`` in
+              the context, which is the default. Otherwise returns just the
               dataset id)
     :rtype: dictionary
 
     '''
     model = context['model']
     user = context['user']
-    name_or_id = data_dict.get("id") or data_dict['name']
+    name_or_id = data_dict.get('id') or data_dict.get('name')
+    if name_or_id is None:
+        raise ValidationError({'id': _('Missing value')})
 
     pkg = model.Package.get(name_or_id)
     if pkg is None:
         raise NotFound(_('Package was not found.'))
     context["package"] = pkg
     data_dict["id"] = pkg.id
+    data_dict['type'] = pkg.type
 
     _check_access('package_update', context, data_dict)
 
@@ -291,7 +289,8 @@ def package_update(context, data_dict):
                 # to ensure they still work.
                 package_plugin.check_data_dict(data_dict)
 
-    data, errors = _validate(data_dict, schema, context)
+    data, errors = lib_plugins.plugin_validate(
+        package_plugin, context, data_dict, schema, 'package_update')
     log.debug('package_update validate_errs=%r user=%s package=%s data=%r',
               errors, context.get('user'),
               context.get('package').name if context.get('package') else '',
@@ -318,10 +317,17 @@ def package_update(context, data_dict):
     context_org_update = context.copy()
     context_org_update['ignore_auth'] = True
     context_org_update['defer_commit'] = True
+    context_org_update['add_revision'] = False
     _get_action('package_owner_org_update')(context_org_update,
                                             {'id': pkg.id,
                                              'organization_id': pkg.owner_org})
 
+    # Needed to let extensions know the new resources ids
+    model.Session.flush()
+    if data.get('resources'):
+        for index, resource in enumerate(data['resources']):
+            resource['id'] = pkg.resources[index].id
+
     for item in plugins.PluginImplementations(plugins.IPackageController):
         item.edit(pkg)
 
@@ -352,7 +358,7 @@ def package_resource_reorder(context, data_dict):
     :param id: the id or name of the package to update
     :type id: string
     :param order: a list of resource ids in the order needed
-    :type list: list
+    :type order: list
     '''
 
     id = _get_or_bust(data_dict, "id")
@@ -405,22 +411,24 @@ def _update_package_relationship(relationship, comment, context):
                                     ref_package_by=ref_package_by)
     return rel_dict
 
+
 def package_relationship_update(context, data_dict):
     '''Update a relationship between two datasets (packages).
 
+    The subject, object and type parameters are required to identify the
+    relationship. Only the comment can be updated.
+
     You must be authorized to edit both the subject and the object datasets.
 
-    :param id: the id of the package relationship to update
-    :type id: string
     :param subject: the name or id of the dataset that is the subject of the
-        relationship (optional)
+        relationship
     :type subject: string
     :param object: the name or id of the dataset that is the object of the
-        relationship (optional)
+        relationship
+    :type object: string
     :param type: the type of the relationship, one of ``'depends_on'``,
         ``'dependency_of'``, ``'derives_from'``, ``'has_derivation'``,
         ``'links_to'``, ``'linked_from'``, ``'child_of'`` or ``'parent_of'``
-        (optional)
     :type type: string
     :param comment: a comment about the relationship (optional)
     :type comment: string
@@ -430,7 +438,8 @@ def package_relationship_update(context, data_dict):
 
     '''
     model = context['model']
-    schema = context.get('schema') or schema_.default_update_relationship_schema()
+    schema = context.get('schema') \
+        or schema_.default_update_relationship_schema()
 
     id, id2, rel = _get_or_bust(data_dict, ['subject', 'object', 'type'])
 
@@ -456,6 +465,7 @@ def package_relationship_update(context, data_dict):
     context['relationship'] = entity
     return _update_package_relationship(entity, comment, context)
 
+
 def _group_or_org_update(context, data_dict, is_org=False):
     model = context['model']
     user = context['user']
@@ -467,18 +477,20 @@ def _group_or_org_update(context, data_dict, is_org=False):
     if group is None:
         raise NotFound('Group was not found.')
 
+    data_dict['type'] = group.type
+
     # get the schema
     group_plugin = lib_plugins.lookup_group_plugin(group.type)
     try:
-        schema = group_plugin.form_to_db_schema_options({'type':'update',
-                                               'api':'api_version' in context,
+        schema = group_plugin.form_to_db_schema_options({'type': 'update',
+                                               'api': 'api_version' in context,
                                                'context': context})
     except AttributeError:
         schema = group_plugin.form_to_db_schema()
 
-    upload = uploader.Upload('group', group.image_url)
+    upload = uploader.get_uploader('group', group.image_url)
     upload.update_data_dict(data_dict, 'image_url',
-                           'image_upload', 'clear_upload')
+                            'image_upload', 'clear_upload')
 
     if is_org:
         _check_access('organization_update', context, data_dict)
@@ -493,7 +505,9 @@ def _group_or_org_update(context, data_dict, is_org=False):
         except TypeError:
             group_plugin.check_data_dict(data_dict)
 
-    data, errors = _validate(data_dict, schema, context)
+    data, errors = lib_plugins.plugin_validate(
+        group_plugin, context, data_dict, schema,
+        'organization_update' if is_org else 'group_update')
     log.debug('group_update validate_errs=%r user=%s group=%s data_dict=%r',
               errors, context.get('user'),
               context.get('group').name if context.get('group') else '',
@@ -511,14 +525,12 @@ def _group_or_org_update(context, data_dict, is_org=False):
     else:
         rev.message = _(u'REST API: Update object %s') % data.get("name")
 
-    # when editing an org we do not want to update the packages if using the
-    # new templates.
-    if ((not is_org)
-            and not converters.asbool(
-                config.get('ckan.legacy_templates', False))
-            and 'api_version' not in context):
-        context['prevent_packages_update'] = True
-    group = model_save.group_dict_save(data, context)
+    contains_packages = 'packages' in data_dict
+
+    group = model_save.group_dict_save(
+        data, context,
+        prevent_packages_update=is_org or not contains_packages
+    )
 
     if is_org:
         plugin_type = plugins.IOrganizationController
@@ -568,21 +580,24 @@ def _group_or_org_update(context, data_dict, is_org=False):
         # in the group.
 
     upload.upload(uploader.get_max_image_size())
+
     if not context.get('defer_commit'):
         model.repo.commit()
 
-
     return model_dictize.group_dictize(group, context)
 
+
 def group_update(context, data_dict):
     '''Update a group.
 
     You must be authorized to edit the group.
 
     Plugins may change the parameters of this function depending on the value
-    of the group's ``type`` attribute, see the ``IGroupForm`` plugin interface.
+    of the group's ``type`` attribute, see the
+    :py:class:`~ckan.plugins.interfaces.IGroupForm` plugin interface.
 
-    For further parameters see ``group_create()``.
+    For further parameters see
+    :py:func:`~ckan.logic.action.create.group_create`.
 
     :param id: the name or id of the group to update
     :type id: string
@@ -591,6 +606,9 @@ def group_update(context, data_dict):
     :rtype: dictionary
 
     '''
+    # Callers that set context['allow_partial_update'] = True can choose to not
+    # specify particular keys and they will be left at their existing
+    # values. This includes: packages, users, groups, tags, extras
     return _group_or_org_update(context, data_dict)
 
 def organization_update(context, data_dict):
@@ -598,24 +616,32 @@ def organization_update(context, data_dict):
 
     You must be authorized to edit the organization.
 
-    For further parameters see ``organization_create()``.
+    For further parameters see
+    :py:func:`~ckan.logic.action.create.organization_create`.
 
     :param id: the name or id of the organization to update
     :type id: string
+    :param packages: ignored. use
+        :py:func:`~ckan.logic.action.update.package_owner_org_update`
+        to change package ownership
 
     :returns: the updated organization
     :rtype: dictionary
 
     '''
+    # Callers that set context['allow_partial_update'] = True can choose to not
+    # specify particular keys and they will be left at their existing
+    # values. This includes: users, groups, tags, extras
     return _group_or_org_update(context, data_dict, is_org=True)
 
 def user_update(context, data_dict):
     '''Update a user account.
 
     Normal users can only update their own user accounts. Sysadmins can update
-    any user account.
+    any user account. Can not modify exisiting user's name.
 
-    For further parameters see ``user_create()``.
+    For further parameters see
+    :py:func:`~ckan.logic.action.create.user_create`.
 
     :param id: the name or id of the user to update
     :type id: string
@@ -642,6 +668,10 @@ def user_update(context, data_dict):
         session.rollback()
         raise ValidationError(errors)
 
+    # user schema prevents non-sysadmins from providing password_hash
+    if 'password_hash' in data:
+        data['_password'] = data.pop('password_hash')
+
     user = model_save.user_dict_save(data, context)
 
     activity_dict = {
@@ -664,6 +694,40 @@ def user_update(context, data_dict):
         model.repo.commit()
     return model_dictize.user_dictize(user, context)
 
+
+def user_generate_apikey(context, data_dict):
+    '''Cycle a user's API key
+
+    :param id: the name or id of the user whose key needs to be updated
+    :type id: string
+
+    :returns: the updated user
+    :rtype: dictionary
+    '''
+    model = context['model']
+    user = context['user']
+    session = context['session']
+    schema = context.get('schema') or schema_.default_generate_apikey_user_schema()
+    context['schema'] = schema
+    # check if user id in data_dict
+    id = _get_or_bust(data_dict, 'id')
+
+    # check if user exists
+    user_obj = model.User.get(id)
+    context['user_obj'] = user_obj
+    if user_obj is None:
+        raise NotFound('User was not found.')
+
+    # check permission
+    _check_access('user_generate_apikey', context, data_dict)
+
+    # change key
+    old_data = _get_action('user_show')(context, data_dict)
+    old_data['apikey'] = model.types.make_uuid()
+    data_dict = old_data
+    return _get_action('user_update')(context, data_dict)
+
+
 def task_status_update(context, data_dict):
     '''Update a task status.
 
@@ -722,7 +786,8 @@ def task_status_update_many(context, data_dict):
     '''Update many task statuses at once.
 
     :param data: the task_status dictionaries to update, for the format of task
-        status dictionaries see ``task_status_update()``
+        status dictionaries see
+        :py:func:`~task_status_update`
     :type data: list of dictionaries
 
     :returns: the updated task statuses
@@ -763,9 +828,9 @@ def term_translation_update(context, data_dict):
 
     _check_access('term_translation_update', context, data_dict)
 
-    schema = {'term': [validators.not_empty, unicode],
-              'term_translation': [validators.not_empty, unicode],
-              'lang_code': [validators.not_empty, unicode]}
+    schema = {'term': [validators.not_empty, text_type],
+              'term_translation': [validators.not_empty, text_type],
+              'lang_code': [validators.not_empty, text_type]}
 
     data, errors = _validate(data_dict, schema, context)
     if errors:
@@ -796,7 +861,7 @@ def term_translation_update_many(context, data_dict):
 
     :param data: the term translation dictionaries to create or update,
         for the format of term translation dictionaries see
-        ``term_translation_update()``
+        :py:func:`~term_translation_update`
     :type data: list of dictionaries
 
     :returns: a dictionary with key ``'success'`` whose value is a string
@@ -823,64 +888,13 @@ def term_translation_update_many(context, data_dict):
     return {'success': '%s rows updated' % (num + 1)}
 
 
-## Modifications for rest api
-
-def package_update_rest(context, data_dict):
-
-    model = context['model']
-    id = data_dict.get("id")
-    request_id = context['id']
-    pkg = model.Package.get(request_id)
-
-    if not pkg:
-        raise NotFound
-
-    if id and id != pkg.id:
-        pkg_from_data = model.Package.get(id)
-        if pkg_from_data != pkg:
-            error_dict = {id:('Cannot change value of key from %s to %s. '
-                'This key is read-only') % (pkg.id, id)}
-            raise ValidationError(error_dict)
-
-    context["package"] = pkg
-    context["allow_partial_update"] = False
-    dictized_package = model_save.package_api_to_dict(data_dict, context)
-
-    _check_access('package_update_rest', context, dictized_package)
-
-    dictized_after = _get_action('package_update')(context, dictized_package)
-
-    pkg = context['package']
-
-    package_dict = model_dictize.package_to_api(pkg, context)
-
-    return package_dict
-
-def group_update_rest(context, data_dict):
-
-    model = context['model']
-    id = _get_or_bust(data_dict, "id")
-    group = model.Group.get(id)
-    context["group"] = group
-    context["allow_partial_update"] = True
-    dictized_group = model_save.group_api_to_dict(data_dict, context)
-
-    _check_access('group_update_rest', context, dictized_group)
-
-    dictized_after = _get_action('group_update')(context, dictized_group)
-
-    group = context['group']
-
-    group_dict = model_dictize.group_to_api(group, context)
-
-    return group_dict
-
 def vocabulary_update(context, data_dict):
     '''Update a tag vocabulary.
 
     You must be a sysadmin to update vocabularies.
 
-    For further parameters see ``vocabulary_create()``.
+    For further parameters see
+    :py:func:`~ckan.logic.action.create.vocabulary_create`.
 
     :param id: the id of the vocabulary to update
     :type id: string
@@ -919,122 +933,12 @@ def vocabulary_update(context, data_dict):
 
     return model_dictize.vocabulary_dictize(updated_vocab, context)
 
-def package_relationship_update_rest(context, data_dict):
-
-    # rename keys
-    key_map = {'id': 'subject',
-               'id2': 'object',
-               'rel': 'type'}
-
-    # We want 'destructive', so that the value of the subject,
-    # object and rel in the URI overwrite any values for these
-    # in params. This is because you are not allowed to change
-    # these values.
-    data_dict = logic.action.rename_keys(data_dict, key_map, destructive=True)
-
-    relationship_dict = _get_action('package_relationship_update')(context, data_dict)
-
-    return relationship_dict
-
-def user_role_update(context, data_dict):
-    '''Update a user or authorization group's roles for a domain object.
-
-    The ``user`` parameter must be given.
-
-    You must be authorized to update the domain object.
-
-    To delete all of a user or authorization group's roles for domain object,
-    pass an empty list ``[]`` to the ``roles`` parameter.
-
-    :param user: the name or id of the user
-    :type user: string
-    :param domain_object: the name or id of the domain object (e.g. a package,
-        group or authorization group)
-    :type domain_object: string
-    :param roles: the new roles, e.g. ``['editor']``
-    :type roles: list of strings
-
-    :returns: the updated roles of all users for the
-        domain object
-    :rtype: dictionary
-
-    '''
-    model = context['model']
-
-    new_user_ref = data_dict.get('user') # the user who is being given the new role
-    if not bool(new_user_ref):
-        raise ValidationError('You must provide the "user" parameter.')
-    domain_object_ref = _get_or_bust(data_dict, 'domain_object')
-    if not isinstance(data_dict['roles'], (list, tuple)):
-        raise ValidationError('Parameter "%s" must be of type: "%s"' % ('role', 'list'))
-    desired_roles = set(data_dict['roles'])
-
-    if new_user_ref:
-        user_object = model.User.get(new_user_ref)
-        if not user_object:
-            raise NotFound('Cannot find user %r' % new_user_ref)
-        data_dict['user'] = user_object.id
-        add_user_to_role_func = model.add_user_to_role
-        remove_user_from_role_func = model.remove_user_from_role
-
-    domain_object = logic.action.get_domain_object(model, domain_object_ref)
-    data_dict['id'] = domain_object.id
-
-    # current_uors: in order to avoid either creating a role twice or
-    # deleting one which is non-existent, we need to get the users\'
-    # current roles (if any)
-    current_role_dicts = _get_action('roles_show')(context, data_dict)['roles']
-    current_roles = set([role_dict['role'] for role_dict in current_role_dicts])
-
-    # Whenever our desired state is different from our current state,
-    # change it.
-    for role in (desired_roles - current_roles):
-        add_user_to_role_func(user_object, role, domain_object)
-    for role in (current_roles - desired_roles):
-        remove_user_from_role_func(user_object, role, domain_object)
-
-    # and finally commit all these changes to the database
-    if not (current_roles == desired_roles):
-        model.repo.commit_and_remove()
-
-    return _get_action('roles_show')(context, data_dict)
-
-def user_role_bulk_update(context, data_dict):
-    '''Update the roles of many users or authorization groups for an object.
-
-    You must be authorized to update the domain object.
-
-    :param user_roles: the updated user roles, for the format of user role
-        dictionaries see ``user_role_update()``
-    :type user_roles: list of dictionaries
-
-    :returns: the updated roles of all users and authorization groups for the
-        domain object
-    :rtype: dictionary
-
-    '''
-    # Collate all the roles for each user
-    roles_by_user = {} # user:roles
-    for user_role_dict in data_dict['user_roles']:
-        user = user_role_dict.get('user')
-        if user:
-            roles = user_role_dict['roles']
-            if user not in roles_by_user:
-                roles_by_user[user] = []
-            roles_by_user[user].extend(roles)
-    # For each user, update its roles
-    for user in roles_by_user:
-        uro_data_dict = {'user': user,
-                         'roles': roles_by_user[user],
-                         'domain_object': data_dict['domain_object']}
-        user_role_update(context, uro_data_dict)
-    return _get_action('roles_show')(context, data_dict)
-
 
 def dashboard_mark_activities_old(context, data_dict):
     '''Mark all the authorized user's new dashboard activities as old.
 
-    This will reset dashboard_new_activities_count to 0.
+    This will reset
+    :py:func:`~ckan.logic.action.get.dashboard_new_activities_count` to 0.
 
     '''
     _check_access('dashboard_mark_activities_old', context,
@@ -1042,7 +946,7 @@ def dashboard_mark_activities_old(context, data_dict):
     model = context['model']
     user_id = model.User.get(context['user']).id
     model.Dashboard.get(user_id).activity_stream_last_viewed = (
-            datetime.datetime.now())
+            datetime.datetime.utcnow())
     if not context.get('defer_commit'):
         model.repo.commit()
 
@@ -1077,9 +981,10 @@ def package_owner_org_update(context, data_dict):
     :type id: string
 
     :param organization_id: the name or id of the owning organization
-    :type id: string
+    :type organization_id: string
     '''
     model = context['model']
+    user = context['user']
     name_or_id = data_dict.get('id')
     organization_id = data_dict.get('organization_id')
 
@@ -1099,10 +1004,17 @@ def package_owner_org_update(context, data_dict):
         org = None
         pkg.owner_org = None
 
+    if context.get('add_revision', True):
+        rev = model.repo.new_revision()
+        rev.author = user
+        if 'message' in context:
+            rev.message = context['message']
+        else:
+            rev.message = _(u'REST API: Update object %s') % pkg.get("name")
 
     members = model.Session.query(model.Member) \
-            .filter(model.Member.table_id == pkg.id) \
-            .filter(model.Member.capacity == 'organization')
+        .filter(model.Member.table_id == pkg.id) \
+        .filter(model.Member.capacity == 'organization')
 
     need_update = True
     for member_obj in members:
@@ -1133,7 +1045,6 @@ def _bulk_update_dataset(context, data_dict, update_dict):
     org_id = data_dict.get('org_id')
 
     model = context['model']
-
     model.Session.query(model.package_table) \
         .filter(model.Package.id.in_(datasets)) \
         .filter(model.Package.owner_org == org_id) \
@@ -1143,7 +1054,7 @@ def _bulk_update_dataset(context, data_dict, update_dict):
     model.Session.query(model.package_revision_table) \
         .filter(model.PackageRevision.id.in_(datasets)) \
         .filter(model.PackageRevision.owner_org == org_id) \
-        .filter(model.PackageRevision.current == True) \
+        .filter(model.PackageRevision.current is True) \
         .update(update_dict, synchronize_session=False)
 
     model.Session.commit()
@@ -1223,3 +1134,104 @@ def bulk_update_delete(context, data_dict):
 
     _check_access('bulk_update_delete', context, data_dict)
     _bulk_update_dataset(context, data_dict, {'state': 'deleted'})
+
+
+def config_option_update(context, data_dict):
+    '''
+
+    .. versionadded:: 2.4
+
+    Allows to modify some CKAN runtime-editable config options
+
+    It takes arbitrary key, value pairs and checks the keys against the
+    config options update schema. If some of the provided keys are not present
+    in the schema a :py:class:`~ckan.plugins.logic.ValidationError` is raised.
+    The values are then validated against the schema, and if validation is
+    passed, for each key, value config option:
+
+    * It is stored on the ``system_info`` database table
+    * The Pylons ``config`` object is updated.
+    * The ``app_globals`` (``g``) object is updated (this only happens for
+      options explicitly defined in the ``app_globals`` module.
+
+    The following lists a ``key`` parameter, but this should be replaced by
+    whichever config options want to be updated, eg::
+
+        get_action('config_option_update)({}, {
+            'ckan.site_title': 'My Open Data site',
+            'ckan.homepage_layout': 2,
+        })
+
+    :param key: a configuration option key (eg ``ckan.site_title``). It must
+        be present on the ``update_configuration_schema``
+    :type key: string
+
+    :returns: a dictionary with the options set
+    :rtype: dictionary
+
+    .. note:: You can see all available runtime-editable configuration options
+        calling
+        the :py:func:`~ckan.logic.action.get.config_option_list` action
+
+    .. note:: Extensions can modify which configuration options are
+        runtime-editable.
+        For details, check :doc:`/extensions/remote-config-update`.
+
+    .. warning:: You should only add config options that you are comfortable
+        they can be edited during runtime, such as ones you've added in your
+        own extension, or have reviewed the use of in core CKAN.
+
+    '''
+    model = context['model']
+
+    _check_access('config_option_update', context, data_dict)
+
+    schema = schema_.update_configuration_schema()
+
+    available_options = schema.keys()
+
+    provided_options = data_dict.keys()
+
+    unsupported_options = set(provided_options) - set(available_options)
+    if unsupported_options:
+        msg = 'Configuration option(s) \'{0}\' can not be updated'.format(
+              ' '.join(list(unsupported_options)))
+
+        raise ValidationError(msg, error_summary={'message': msg})
+
+    upload = uploader.get_uploader('admin')
+    upload.update_data_dict(data_dict, 'ckan.site_logo',
+                            'logo_upload', 'clear_logo_upload')
+    upload.upload(uploader.get_max_image_size())
+    data, errors = _validate(data_dict, schema, context)
+    if errors:
+        model.Session.rollback()
+        raise ValidationError(errors)
+
+    for key, value in data.iteritems():
+
+        # Set full Logo url
+        if key == 'ckan.site_logo' and value and not value.startswith('http')\
+                and not value.startswith('/'):
+            image_path = 'uploads/admin/'
+
+            value = h.url_for_static('{0}{1}'.format(image_path, value))
+
+        # Save value in database
+        model.set_system_info(key, value)
+
+        # Update CKAN's `config` object
+        config[key] = value
+
+        # Only add it to the app_globals (`g`) object if explicitly defined
+        # there
+        globals_keys = app_globals.app_globals_from_config_details.keys()
+        if key in globals_keys:
+            app_globals.set_app_global(key, value)
+
+    # Update the config update timestamp
+    model.set_system_info('ckan.config_update', str(time.time()))
+
+    log.info('Updated config options: {0}'.format(data))
+
+    return data
