@@ -1,10 +1,10 @@
+# encoding: utf-8
+
 import ckan.logic as logic
-import ckan.new_authz as new_authz
-from ckan.logic.auth import get_group_object, get_related_object
+import ckan.authz as authz
+from ckan.logic.auth import get_group_object
 from ckan.logic.auth import get_resource_object
-import ckan.logic.auth.create as _auth_create
-import ckan.logic.auth.update as _auth_update
-from ckan.lib.base import _
+from ckan.common import _
 
 
 def user_delete(context, data_dict):
@@ -13,9 +13,15 @@ def user_delete(context, data_dict):
 
 
 def package_delete(context, data_dict):
-    # Defer auhtorization for package_delete to package_update, as deletions
+    # Defer authorization for package_delete to package_update, as deletions
     # are essentially changing the state field
-    return _auth_update.package_update(context, data_dict)
+    return authz.is_authorized('package_update', context, data_dict)
+
+
+def dataset_purge(context, data_dict):
+    # Only sysadmins are authorized to purge datasets
+    return {'success': False}
+
 
 def resource_delete(context, data_dict):
     model = context['model']
@@ -23,16 +29,12 @@ def resource_delete(context, data_dict):
     resource = get_resource_object(context, data_dict)
 
     # check authentication against package
-    query = model.Session.query(model.Package)\
-        .join(model.ResourceGroup)\
-        .join(model.Resource)\
-        .filter(model.ResourceGroup.id == resource.resource_group_id)
-    pkg = query.first()
+    pkg = model.Package.get(resource.package_id)
     if not pkg:
         raise logic.NotFound(_('No package found for this resource, cannot check auth.'))
 
     pkg_dict = {'id': pkg.id}
-    authorized = package_delete(context, pkg_dict).get('success')
+    authorized = authz.is_authorized('package_delete', context, pkg_dict).get('success')
 
     if not authorized:
         return {'success': False, 'msg': _('User %s not authorized to delete resource %s') % (user, resource.id)}
@@ -40,35 +42,33 @@ def resource_delete(context, data_dict):
         return {'success': True}
 
 
-def related_delete(context, data_dict):
-    model = context['model']
-    user = context['user']
-    if not user:
-        return {'success': False, 'msg': _('Only the owner can delete a related item')}
+def resource_view_delete(context, data_dict):
 
-    related = get_related_object(context, data_dict)
-    userobj = model.User.get( user )
+    if context.get('resource'):
+        return authz.is_authorized('resource_delete', context, {})
+    if context.get('resource_view'):
+        return authz.is_authorized('resource_delete', context, {'id': context['resource_view'].resource_id})
 
-    if related.datasets:
-        package = related.datasets[0]
+    resource_id = data_dict.get('resource_id')
+    if not resource_id:
+        resource_view = context['model'].ResourceView.get(data_dict['id'])
+        if not resource_view:
+            raise logic.NotFound(_('Resource view not found, cannot check auth.'))
+        resource_id = resource_view.resource_id
 
-        pkg_dict = { 'id': package.id }
-        authorized = package_delete(context, pkg_dict).get('success')
-        if authorized:
-            return {'success': True}
+    return authz.is_authorized('resource_delete', context, {'id': resource_id})
 
-    if not userobj or userobj.id != related.owner_id:
-        return {'success': False, 'msg': _('Only the owner can delete a related item')}
-
-    return {'success': True}
 
+def resource_view_clear(context, data_dict):
+    # sysadmins only
+    return {'success': False}
 
 def package_relationship_delete(context, data_dict):
     user = context['user']
     relationship = context['relationship']
 
     # If you can create this relationship the you can also delete it
-    authorized = new_authz.is_authorized_boolean('package_relationship_create', context, data_dict)
+    authorized = authz.is_authorized_boolean('package_relationship_create', context, data_dict)
     if not authorized:
         return {'success': False, 'msg': _('User %s not authorized to delete relationship %s') % (user ,relationship.id)}
     else:
@@ -77,10 +77,10 @@ def package_relationship_delete(context, data_dict):
 def group_delete(context, data_dict):
     group = get_group_object(context, data_dict)
     user = context['user']
-    if not new_authz.check_config_permission('user_delete_groups'):
+    if not authz.check_config_permission('user_delete_groups'):
         return {'success': False,
             'msg': _('User %s not authorized to delete groups') % user}
-    authorized = new_authz.has_user_permission_for_group_or_org(
+    authorized = authz.has_user_permission_for_group_or_org(
         group.id, user, 'delete')
     if not authorized:
         return {'success': False, 'msg': _('User %s not authorized to delete group %s') % (user ,group.id)}
@@ -98,10 +98,10 @@ def organization_purge(context, data_dict):
 def organization_delete(context, data_dict):
     group = get_group_object(context, data_dict)
     user = context['user']
-    if not new_authz.check_config_permission('user_delete_organizations'):
+    if not authz.check_config_permission('user_delete_organizations'):
         return {'success': False,
             'msg': _('User %s not authorized to delete organizations') % user}
-    authorized = new_authz.has_user_permission_for_group_or_org(
+    authorized = authz.has_user_permission_for_group_or_org(
         group.id, user, 'delete')
     if not authorized:
         return {'success': False, 'msg': _('User %s not authorized to delete organization %s') % (user ,group.id)}
@@ -136,4 +136,14 @@ def organization_member_delete(context, data_dict):
     return {'success': True}
 
 def member_delete(context, data_dict):
-    return _auth_create.member_create(context, data_dict)
+    return authz.is_authorized('member_create', context, data_dict)
+
+
+def job_clear(context, data_dict):
+    '''Clear background jobs. Only sysadmins.'''
+    return {'success': False}
+
+
+def job_cancel(context, data_dict):
+    '''Cancel a background job. Only sysadmins.'''
+    return {'success': False}
