@@ -1,5 +1,7 @@
+# encoding: utf-8
+
 import ckan.logic as logic
-import ckan.new_authz as new_authz
+import ckan.authz as authz
 import ckan.logic.auth as logic_auth
 from ckan.common import _
 
@@ -7,9 +9,6 @@ from ckan.common import _
 from ckan.logic.auth.create import _check_group_auth
 
 
-def make_latest_pending_package_active(context, data_dict):
-    return new_authz.is_authorized('package_update', context, data_dict)
-
 @logic.auth_allow_anonymous_access
 def package_update(context, data_dict):
     user = context.get('user')
@@ -18,22 +17,22 @@ def package_update(context, data_dict):
     if package.owner_org:
         # if there is an owner org then we must have update_dataset
         # permission for that organization
-        check1 = new_authz.has_user_permission_for_group_or_org(
+        check1 = authz.has_user_permission_for_group_or_org(
             package.owner_org, user, 'update_dataset'
         )
     else:
         # If dataset is not owned then we can edit if config permissions allow
-        if new_authz.auth_is_anon_user(context):
-            check1 = all(new_authz.check_config_permission(p) for p in (
+        if authz.auth_is_anon_user(context):
+            check1 = all(authz.check_config_permission(p) for p in (
                 'anon_create_dataset',
                 'create_dataset_if_not_in_organization',
                 'create_unowned_dataset',
                 ))
         else:
-            check1 = all(new_authz.check_config_permission(p) for p in (
+            check1 = all(authz.check_config_permission(p) for p in (
                 'create_dataset_if_not_in_organization',
                 'create_unowned_dataset',
-                )) or new_authz.has_user_permission_for_some_org(
+                )) or authz.has_user_permission_for_some_org(
                 user, 'create_dataset')
     if not check1:
         return {'success': False,
@@ -58,18 +57,14 @@ def resource_update(context, data_dict):
     resource = logic_auth.get_resource_object(context, data_dict)
 
     # check authentication against package
-    query = model.Session.query(model.Package)\
-        .join(model.ResourceGroup)\
-        .join(model.Resource)\
-        .filter(model.ResourceGroup.id == resource.resource_group_id)
-    pkg = query.first()
+    pkg = model.Package.get(resource.package_id)
     if not pkg:
         raise logic.NotFound(
             _('No package found for this resource, cannot check auth.')
         )
 
     pkg_dict = {'id': pkg.id}
-    authorized = new_authz.is_authorized('package_update', context, pkg_dict).get('success')
+    authorized = authz.is_authorized('package_update', context, pkg_dict).get('success')
 
     if not authorized:
         return {'success': False,
@@ -79,8 +74,14 @@ def resource_update(context, data_dict):
         return {'success': True}
 
 
+def resource_view_update(context, data_dict):
+    return authz.is_authorized('resource_update', context, {'id': data_dict['resource_id']})
+
+def resource_view_reorder(context, data_dict):
+    return authz.is_authorized('resource_update', context, {'id': data_dict['id']})
+
 def package_relationship_update(context, data_dict):
-    return new_authz.is_authorized('package_relationship_create',
+    return authz.is_authorized('package_relationship_create',
                                    context,
                                    data_dict)
 
@@ -90,7 +91,7 @@ def package_change_state(context, data_dict):
     package = logic_auth.get_package_object(context, data_dict)
 
     # use the logic for package_update
-    authorized = new_authz.is_authorized_boolean('package_update',
+    authorized = authz.is_authorized_boolean('package_update',
                                                  context,
                                                  data_dict)
     if not authorized:
@@ -106,7 +107,7 @@ def package_change_state(context, data_dict):
 def group_update(context, data_dict):
     group = logic_auth.get_group_object(context, data_dict)
     user = context['user']
-    authorized = new_authz.has_user_permission_for_group_or_org(group.id,
+    authorized = authz.has_user_permission_for_group_or_org(group.id,
                                                                 user,
                                                                 'update')
     if not authorized:
@@ -120,7 +121,7 @@ def group_update(context, data_dict):
 def organization_update(context, data_dict):
     group = logic_auth.get_group_object(context, data_dict)
     user = context['user']
-    authorized = new_authz.has_user_permission_for_group_or_org(
+    authorized = authz.has_user_permission_for_group_or_org(
         group.id, user, 'update')
     if not authorized:
         return {'success': False,
@@ -130,34 +131,12 @@ def organization_update(context, data_dict):
         return {'success': True}
 
 
-def related_update(context, data_dict):
-    model = context['model']
-    user = context['user']
-    if not user:
-        return {'success': False,
-                'msg': _('Only the owner can update a related item')}
-
-    related = logic_auth.get_related_object(context, data_dict)
-    userobj = model.User.get(user)
-    if not userobj or userobj.id != related.owner_id:
-        return {'success': False,
-                'msg': _('Only the owner can update a related item')}
-
-    # Only sysadmins can change the featured field.
-    if ('featured' in data_dict and data_dict['featured'] != related.featured):
-        return {'success': False,
-                'msg': _('You must be a sysadmin to change a related item\'s '
-                         'featured field.')}
-
-    return {'success': True}
-
-
 def group_change_state(context, data_dict):
     user = context['user']
     group = logic_auth.get_group_object(context, data_dict)
 
     # use logic for group_update
-    authorized = new_authz.is_authorized_boolean('group_update',
+    authorized = authz.is_authorized_boolean('group_update',
                                                  context,
                                                  data_dict)
     if not authorized:
@@ -174,14 +153,15 @@ def group_edit_permissions(context, data_dict):
     user = context['user']
     group = logic_auth.get_group_object(context, data_dict)
 
-    authorized = new_authz.has_user_permission_for_group_or_org(group.id,
-                                                                user,
-                                                                'update')
+    authorized = authz.has_user_permission_for_group_or_org(
+        group.id, user, 'update')
 
     if not authorized:
-        return {'success': False,
-                'msg': _('User %s not authorized to edit permissions of group %s') %
-                        (str(user), group.id)}
+        return {
+            'success': False,
+            'msg': _('User %s not authorized to'
+                     ' edit permissions of group %s') %
+            (str(user), group.id)}
     else:
         return {'success': True}
 
@@ -219,6 +199,16 @@ def user_update(context, data_dict):
                         (user, user_obj.id)}
 
 
+def user_generate_apikey(context, data_dict):
+    user = context['user']
+    user_obj = logic_auth.get_user_object(context, data_dict)
+    if user == user_obj.name:
+        # Allow users to update only their own user accounts.
+        return {'success': True}
+    return {'success': False, 'msg': _('User {0} not authorized to update user'
+            ' {1}'.format(user, user_obj.id))}
+
+
 def revision_change_state(context, data_dict):
     # FIXME currently only sysadmins can change state
     user = context['user']
@@ -252,7 +242,7 @@ def term_translation_update(context, data_dict):
 
 
 def dashboard_mark_activities_old(context, data_dict):
-    return new_authz.is_authorized('dashboard_activity_list',
+    return authz.is_authorized('dashboard_activity_list',
                                    context,
                                    data_dict)
 
@@ -262,28 +252,6 @@ def send_email_notifications(context, data_dict):
     return {'success': False}
 
 
-## Modifications for rest api
-
-def package_update_rest(context, data_dict):
-    model = context['model']
-    user = context['user']
-    if user in (model.PSEUDO_USER__VISITOR, ''):
-        return {'success': False,
-                'msg': _('Valid API key needed to edit a package')}
-
-    return new_authz.is_authorized('package_update', context, data_dict)
-
-
-def group_update_rest(context, data_dict):
-    model = context['model']
-    user = context['user']
-    if user in (model.PSEUDO_USER__VISITOR, ''):
-        return {'success': False,
-                'msg': _('Valid API key needed to edit a group')}
-
-    return group_update(context, data_dict)
-
-
 def package_owner_org_update(context, data_dict):
     # sysadmins only
     return {'success': False}
@@ -292,7 +260,7 @@ def package_owner_org_update(context, data_dict):
 def bulk_update_private(context, data_dict):
     org_id = data_dict.get('org_id')
     user = context['user']
-    authorized = new_authz.has_user_permission_for_group_or_org(
+    authorized = authz.has_user_permission_for_group_or_org(
         org_id, user, 'update')
     if not authorized:
         return {'success': False}
@@ -302,7 +270,7 @@ def bulk_update_private(context, data_dict):
 def bulk_update_public(context, data_dict):
     org_id = data_dict.get('org_id')
     user = context['user']
-    authorized = new_authz.has_user_permission_for_group_or_org(
+    authorized = authz.has_user_permission_for_group_or_org(
         org_id, user, 'update')
     if not authorized:
         return {'success': False}
@@ -312,8 +280,16 @@ def bulk_update_public(context, data_dict):
 def bulk_update_delete(context, data_dict):
     org_id = data_dict.get('org_id')
     user = context['user']
-    authorized = new_authz.has_user_permission_for_group_or_org(
+    authorized = authz.has_user_permission_for_group_or_org(
         org_id, user, 'update')
     if not authorized:
         return {'success': False}
     return {'success': True}
+
+
+def config_option_update(context, data_dict):
+    '''Update the runtime-editable configuration options
+
+       Only sysdmins can do it
+    '''
+    return {'success': False}
