@@ -1,6 +1,14 @@
+# encoding: utf-8
+
+import collections
 import datetime
 from itertools import count
 import re
+import mimetypes
+import json
+
+from six import string_types, iteritems
+from six.moves.urllib.parse import urlparse
 
 import ckan.lib.navl.dictization_functions as df
 import ckan.logic as logic
@@ -10,7 +18,8 @@ from ckan.model import (MAX_TAG_LENGTH, MIN_TAG_LENGTH,
                         PACKAGE_VERSION_MAX_LENGTH,
                         VOCABULARY_NAME_MAX_LENGTH,
                         VOCABULARY_NAME_MIN_LENGTH)
-import ckan.new_authz as new_authz
+import ckan.authz as authz
+from ckan.model.core import State
 
 from ckan.common import _
 
@@ -24,29 +33,25 @@ def owner_org_validator(key, data, errors, context):
     value = data.get(key)
 
     if value is missing or value is None:
-        if not new_authz.check_config_permission('create_unowned_dataset'):
-            raise Invalid(_('A organization must be supplied'))
+        if not authz.check_config_permission('create_unowned_dataset'):
+            raise Invalid(_('An organization must be provided'))
         data.pop(key, None)
         raise df.StopOnError
 
     model = context['model']
     user = context['user']
     user = model.User.get(user)
-    if value == '' :
-        if not new_authz.check_config_permission('create_unowned_dataset'):
-            raise Invalid(_('A organization must be supplied'))
-        package = context.get('package')
-        # only sysadmins can remove datasets from org
-        if package and package.owner_org and not user.sysadmin:
-            raise Invalid(_('You cannot remove a dataset from an existing organization'))
+    if value == '':
+        if not authz.check_config_permission('create_unowned_dataset'):
+            raise Invalid(_('An organization must be provided'))
         return
 
     group = model.Group.get(value)
     if not group:
         raise Invalid(_('Organization does not exist'))
     group_id = group.id
-    if not(user.sysadmin or
-           new_authz.has_user_permission_for_group_or_org(
+    if not context.get(u'ignore_auth', False) and not(user.sysadmin or
+           authz.has_user_permission_for_group_or_org(
                group_id, user.name, 'create_dataset')):
         raise Invalid(_('You cannot add a dataset to this organization'))
     data[key] = group_id
@@ -61,14 +66,34 @@ def package_id_not_changed(value, context):
     return value
 
 def int_validator(value, context):
-    if isinstance(value, int):
-        return value
+    '''
+    Return an integer for value, which may be a string in base 10 or
+    a numeric type (e.g. int, long, float, Decimal, Fraction). Return
+    None for None or empty/all-whitespace string values.
+
+    :raises: ckan.lib.navl.dictization_functions.Invalid for other
+        inputs or non-whole values
+    '''
+    if value is None:
+        return None
+    if hasattr(value, 'strip') and not value.strip():
+        return None
+
     try:
-        if value.strip() == '':
-            return None
-        return int(value)
-    except (AttributeError, ValueError), e:
-        raise Invalid(_('Invalid integer'))
+        whole, part = divmod(value, 1)
+    except TypeError:
+        try:
+            return int(value)
+        except ValueError:
+            pass
+    else:
+        if not part:
+            try:
+                return int(whole)
+            except TypeError:
+                pass  # complex number: fail like int(complex) does
+
+    raise Invalid(_('Invalid integer'))
 
 def natural_number_validator(value, context):
     value = int_validator(value, context)
@@ -83,6 +108,15 @@ def is_positive_integer(value, context):
     return value
 
 def boolean_validator(value, context):
+    '''
+    Return a boolean for value.
+    Return value when value is a python bool type.
+    Return True for strings 'true', 'yes', 't', 'y', and '1'.
+    Return False in all other cases, including when value is an empty string or
+    None
+    '''
+    if value is missing or value is None:
+        return False
     if isinstance(value, bool):
         return value
     if value.lower() in ['true', 'yes', 't', 'y', '1']:
@@ -96,7 +130,7 @@ def isodate(value, context):
         return None
     try:
         date = h.date_str_to_datetime(value)
-    except (TypeError, ValueError), e:
+    except (TypeError, ValueError) as e:
         raise Invalid(_('Date format incorrect'))
     return date
 
@@ -119,6 +153,16 @@ def package_id_exists(value, context):
         raise Invalid('%s: %s' % (_('Not found'), _('Dataset')))
     return value
 
+def package_id_does_not_exist(value, context):
+
+    model = context['model']
+    session = context['session']
+
+    result = session.query(model.Package).get(value)
+    if result:
+        raise Invalid(_('Dataset id already exists'))
+    return value
+
 def package_name_exists(value, context):
 
     model = context['model']
@@ -152,11 +196,20 @@ def package_id_or_name_exists(package_id_or_name, context):
 
     return package_id_or_name
 
+
+def resource_id_exists(value, context):
+    model = context['model']
+    session = context['session']
+    if not session.query(model.Resource).get(value):
+        raise Invalid('%s: %s' % (_('Not found'), _('Resource')))
+    return value
+
+
 def user_id_exists(user_id, context):
-    """Raises Invalid if the given user_id does not exist in the model given
+    '''Raises Invalid if the given user_id does not exist in the model given
     in the context, otherwise returns the given user_id.
 
-    """
+    '''
     model = context['model']
     session = context['session']
 
@@ -183,10 +236,10 @@ def user_id_or_name_exists(user_id_or_name, context):
     return user_id_or_name
 
 def group_id_exists(group_id, context):
-    """Raises Invalid if the given group_id does not exist in the model given
+    '''Raises Invalid if the given group_id does not exist in the model given
     in the context, otherwise returns the given group_id.
 
-    """
+    '''
     model = context['model']
     session = context['session']
 
@@ -195,24 +248,10 @@ def group_id_exists(group_id, context):
         raise Invalid('%s: %s' % (_('Not found'), _('Group')))
     return group_id
 
-
-def related_id_exists(related_id, context):
-    """Raises Invalid if the given related_id does not exist in the model
-    given in the context, otherwise returns the given related_id.
-
-    """
-    model = context['model']
-    session = context['session']
-
-    result = session.query(model.Related).get(related_id)
-    if not result:
-        raise Invalid('%s: %s' % (_('Not found'), _('Related')))
-    return related_id
-
 def group_id_or_name_exists(reference, context):
-    """
+    '''
     Raises Invalid if a group identified by the name or id cannot be found.
-    """
+    '''
     model = context['model']
     result = model.Group.get(reference)
     if not result:
@@ -220,27 +259,18 @@ def group_id_or_name_exists(reference, context):
     return reference
 
 def activity_type_exists(activity_type):
-    """Raises Invalid if there is no registered activity renderer for the
+    '''Raises Invalid if there is no registered activity renderer for the
     given activity_type. Otherwise returns the given activity_type.
 
     This just uses object_id_validators as a lookup.
     very safe.
 
-    """
+    '''
     if activity_type in object_id_validators:
         return activity_type
     else:
         raise Invalid('%s: %s' % (_('Not found'), _('Activity type')))
 
-def resource_id_exists(value, context):
-
-    model = context['model']
-    session = context['session']
-
-    result = session.query(model.Resource).get(value)
-    if not result:
-        raise Invalid('%s: %s' % (_('Not found'), _('Resource')))
-    return value
 
 # A dictionary mapping activity_type values from activity dicts to functions
 # for validating the object_id values from those same activity dicts.
@@ -259,13 +289,10 @@ object_id_validators = {
     'changed organization' : group_id_exists,
     'deleted organization' : group_id_exists,
     'follow group' : group_id_exists,
-    'new related item': related_id_exists,
-    'deleted related item': related_id_exists,
-    'changed related item': related_id_exists,
     }
 
 def object_id_validator(key, activity_dict, errors, context):
-    """Validate the 'object_id' value of an activity_dict.
+    '''Validate the 'object_id' value of an activity_dict.
 
     Uses the object_id_validators dict (above) to find and call an 'object_id'
     validator function for the given activity_dict's 'activity_type' value.
@@ -277,7 +304,7 @@ def object_id_validator(key, activity_dict, errors, context):
     Raises Invalid if there is no object_id_validator for the activity_dict's
     'activity_type' value.
 
-    """
+    '''
     activity_type = activity_dict[('activity_type',)]
     if object_id_validators.has_key(activity_type):
         object_id = activity_dict[('object_id',)]
@@ -286,11 +313,6 @@ def object_id_validator(key, activity_dict, errors, context):
         raise Invalid('There is no object_id validator for '
             'activity type "%s"' % activity_type)
 
-def extras_unicode_convert(extras, context):
-    for extra in extras:
-        extras[extra] = unicode(extras[extra])
-    return extras
-
 name_match = re.compile('[a-z0-9_\-]*$')
 def name_validator(value, context):
     '''Return the given value if it's a valid name, otherwise raise Invalid.
@@ -309,7 +331,7 @@ def name_validator(value, context):
         a valid name
 
     '''
-    if not isinstance(value, basestring):
+    if not isinstance(value, string_types):
         raise Invalid(_('Names must be strings'))
 
     # check basic textual rules
@@ -317,29 +339,29 @@ def name_validator(value, context):
         raise Invalid(_('That name cannot be used'))
 
     if len(value) < 2:
-        raise Invalid(_('Name must be at least %s characters long') % 2)
+        raise Invalid(_('Must be at least %s characters long') % 2)
     if len(value) > PACKAGE_NAME_MAX_LENGTH:
         raise Invalid(_('Name must be a maximum of %i characters long') % \
                       PACKAGE_NAME_MAX_LENGTH)
     if not name_match.match(value):
-        raise Invalid(_('Url must be purely lowercase alphanumeric '
+        raise Invalid(_('Must be purely lowercase alphanumeric '
                         '(ascii) characters and these symbols: -_'))
     return value
 
 def package_name_validator(key, data, errors, context):
-    model = context["model"]
-    session = context["session"]
-    package = context.get("package")
+    model = context['model']
+    session = context['session']
+    package = context.get('package')
 
-    query = session.query(model.Package.name).filter_by(name=data[key])
+    query = session.query(model.Package.state).filter_by(name=data[key])
     if package:
         package_id = package.id
     else:
-        package_id = data.get(key[:-1] + ("id",))
+        package_id = data.get(key[:-1] + ('id',))
     if package_id and package_id is not missing:
-        query = query.filter(model.Package.id <> package_id)
+        query = query.filter(model.Package.id != package_id)
     result = query.first()
-    if result:
+    if result and result.state != State.DELETED:
         errors[key].append(_('That URL is already in use.'))
 
     value = data[key]
@@ -386,7 +408,7 @@ def group_name_validator(key, data, errors, context):
     else:
         group_id = data.get(key[:-1] + ('id',))
     if group_id and group_id is not missing:
-        query = query.filter(model.Group.id <> group_id)
+        query = query.filter(model.Group.id != group_id)
     result = query.first()
     if result:
         errors[key].append(_('Group name already exists in database'))
@@ -423,7 +445,7 @@ def tag_string_convert(key, data, errors, context):
     and parses tag names. These are added to the data dict, enumerated. They
     are also validated.'''
 
-    if isinstance(data[key], basestring):
+    if isinstance(data[key], string_types):
         tags = [tag.strip() \
                 for tag in data[key].split(',') \
                 if tag.strip()]
@@ -452,7 +474,7 @@ def ignore_not_package_admin(key, data, errors, context):
     if 'ignore_auth' in context:
         return
 
-    if user and new_authz.is_sysadmin(user):
+    if user and authz.is_sysadmin(user):
         return
 
     authorized = False
@@ -480,7 +502,7 @@ def ignore_not_sysadmin(key, data, errors, context):
     user = context.get('user')
     ignore_auth = context.get('ignore_auth')
 
-    if ignore_auth or (user and new_authz.is_sysadmin(user)):
+    if ignore_auth or (user and authz.is_sysadmin(user)):
         return
 
     data.pop(key)
@@ -492,7 +514,7 @@ def ignore_not_group_admin(key, data, errors, context):
     model = context['model']
     user = context.get('user')
 
-    if user and new_authz.is_sysadmin(user):
+    if user and authz.is_sysadmin(user):
         return
 
     authorized = False
@@ -523,14 +545,13 @@ def user_name_validator(key, data, errors, context):
     model = context['model']
     new_user_name = data[key]
 
-    if not isinstance(new_user_name, basestring):
+    if not isinstance(new_user_name, string_types):
         raise Invalid(_('User names must be strings'))
 
     user = model.User.get(new_user_name)
+    user_obj_from_context = context.get('user_obj')
     if user is not None:
         # A user with new_user_name already exists in the database.
-
-        user_obj_from_context = context.get('user_obj')
         if user_obj_from_context and user_obj_from_context.id == user.id:
             # If there's a user_obj in context with the same id as the user
             # found in the db, then we must be doing a user_update and not
@@ -541,6 +562,12 @@ def user_name_validator(key, data, errors, context):
             # name, so you can create a new user with that name or update an
             # existing user's name to that name.
             errors[key].append(_('That login name is not available.'))
+    elif user_obj_from_context:
+        old_user = model.User.get(user_obj_from_context.id)
+        if old_user is not None and old_user.state != model.State.PENDING:
+            errors[key].append(_('That login name can not be modified.'))
+        else:
+            return
 
 def user_both_passwords_entered(key, data, errors, context):
 
@@ -556,12 +583,13 @@ def user_password_validator(key, data, errors, context):
 
     if isinstance(value, Missing):
         pass
-    elif not isinstance(value, basestring):
+    elif not isinstance(value, string_types):
         errors[('password',)].append(_('Passwords must be strings'))
     elif value == '':
         pass
-    elif len(value) < 4:
-        errors[('password',)].append(_('Your password must be 4 characters or longer'))
+    elif len(value) < 8:
+        errors[('password',)].append(_('Your password must be 8 characters or '
+                                       'longer'))
 
 def user_passwords_match(key, data, errors, context):
 
@@ -578,6 +606,11 @@ def user_password_not_empty(key, data, errors, context):
     '''Only check if password is present if the user is created via action API.
        If not, user_both_passwords_entered will handle the validation'''
 
+    # sysadmin may provide password_hash directly for importing users
+    if (data.get(('password_hash',), missing) is not missing and
+            authz.is_sysadmin(context.get('user'))):
+        return
+
     if not ('password1',) in data and not ('password2',) in data:
         password = data.get(('password',),None)
         if not password:
@@ -655,27 +688,27 @@ def tag_not_in_vocabulary(key, tag_dict, errors, context):
         return
 
 def url_validator(key, data, errors, context):
-    """ Checks that the provided value (if it is present) is a valid URL """
+    ''' Checks that the provided value (if it is present) is a valid URL '''
     import urlparse
     import string
 
-    model = context['model']
-    session = context['session']
-
     url = data.get(key, None)
     if not url:
         return
 
-    pieces = urlparse.urlparse(url)
-    if all([pieces.scheme, pieces.netloc]) and \
-       set(pieces.netloc) <= set(string.letters + string.digits + '-.') and \
-       pieces.scheme in ['http', 'https']:
-       return
+    try:
+        pieces = urlparse.urlparse(url)
+        if all([pieces.scheme, pieces.netloc]) and \
+           set(pieces.netloc) <= set(string.letters + string.digits + '-.') and \
+           pieces.scheme in ['http', 'https']:
+           return
+    except ValueError:
+        # url is invalid
+        pass
 
     errors[key].append(_('Please provide a valid URL'))
 
 
-
 def user_name_exists(user_name, context):
     model = context['model']
     session = context['session']
@@ -686,7 +719,7 @@ def user_name_exists(user_name, context):
 
 
 def role_exists(role, context):
-    if role not in new_authz.ROLE_PERMISSIONS:
+    if role not in authz.ROLE_PERMISSIONS:
         raise Invalid(_('role does not exist.'))
     return role
 
@@ -724,9 +757,23 @@ def list_of_strings(key, data, errors, context):
     if not isinstance(value, list):
         raise Invalid(_('Not a list'))
     for x in value:
-        if not isinstance(x, basestring):
+        if not isinstance(x, string_types):
             raise Invalid('%s: %s' % (_('Not a string'), x))
 
+def if_empty_guess_format(key, data, errors, context):
+    value = data[key]
+    resource_id = data.get(key[:-1] + ('id',))
+
+    # if resource_id then an update
+    if (not value or value is Missing) and not resource_id:
+        url = data.get(key[:-1] + ('url',), '')
+        mimetype, encoding = mimetypes.guess_type(url)
+        if mimetype:
+            data[key] = mimetype
+
+def clean_format(format):
+    return h.unified_resource_format(format)
+
 def no_loops_in_hierarchy(key, data, errors, context):
     '''Checks that the parent groups specified in the data would not cause
     a loop in the group hierarchy, and therefore cause the recursion up/down
@@ -746,3 +793,120 @@ def no_loops_in_hierarchy(key, data, errors, context):
             raise Invalid(_('This parent would create a loop in the '
                             'hierarchy'))
 
+
+def filter_fields_and_values_should_have_same_length(key, data, errors, context):
+    convert_to_list_if_string = logic.converters.convert_to_list_if_string
+    fields = convert_to_list_if_string(data.get(('filter_fields',), []))
+    values = convert_to_list_if_string(data.get(('filter_values',), []))
+
+    if len(fields) != len(values):
+        msg = _('"filter_fields" and "filter_values" should have the same length')
+        errors[('filter_fields',)].append(msg)
+        errors[('filter_values',)].append(msg)
+
+
+def filter_fields_and_values_exist_and_are_valid(key, data, errors, context):
+    convert_to_list_if_string = logic.converters.convert_to_list_if_string
+    fields = convert_to_list_if_string(data.get(('filter_fields',)))
+    values = convert_to_list_if_string(data.get(('filter_values',)))
+
+    if not fields:
+        errors[('filter_fields',)].append(_('"filter_fields" is required when '
+                                            '"filter_values" is filled'))
+    if not values:
+        errors[('filter_values',)].append(_('"filter_values" is required when '
+                                            '"filter_fields" is filled'))
+
+    filters = collections.defaultdict(list)
+    for field, value in zip(fields, values):
+        filters[field].append(value)
+
+    data[('filters',)] = dict(filters)
+
+
+def extra_key_not_in_root_schema(key, data, errors, context):
+
+    for schema_key in context.get('schema_keys', []):
+        if schema_key == data[key]:
+            raise Invalid(_('There is a schema field with the same name'))
+
+
+def empty_if_not_sysadmin(key, data, errors, context):
+    '''Only sysadmins may pass this value'''
+    from ckan.lib.navl.validators import empty
+
+    user = context.get('user')
+
+    ignore_auth = context.get('ignore_auth')
+    if ignore_auth or (user and authz.is_sysadmin(user)):
+        return
+
+    empty(key, data, errors, context)
+
+#pattern from https://html.spec.whatwg.org/#e-mail-state-(type=email)
+email_pattern = re.compile(r"^[a-zA-Z0-9.!#$%&'*+\/=?^_`{|}~-]+@[a-zA-Z0-9]"\
+                       "(?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?(?:\.[a-zA-Z0-9]"\
+                       "(?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?)*$")
+
+
+def email_validator(value, context):
+    '''Validate email input '''
+
+    if value:
+        if not email_pattern.match(value):
+            raise Invalid(_('Email {email} is not a valid format').format(email=value))
+    return value
+
+
+def email_is_unique(key, data, errors, context):
+    '''Validate email is unique'''
+    model = context['model']
+    session = context['session']
+
+    users = session.query(model.User) \
+            .filter(model.User.email == data[key]).all()
+    # is there is no users with this email it's free
+    if not users:
+        return
+    else:
+        # allow user to update their own email
+        for user in users:
+            if (user.name == data[("name",)]
+                    or user.id == data[("id",)]):
+                return
+
+    raise Invalid(
+                _('The email address \'{email}\' \
+                    belongs to a registered user.').
+                        format(email=data[key]))
+
+def one_of(list_of_value):
+    ''' Checks if the provided value is present in a list '''
+    def callable(value):
+        if value not in list_of_value:
+            raise Invalid(_('Value must be one of {}'.format(list_of_value)))
+        return value
+    return callable
+
+
+def json_object(value):
+    ''' Make sure value can be serialized as a JSON object'''
+    if value is None or value == '':
+        return
+    try:
+        if not json.dumps(value).startswith('{'):
+            raise Invalid(_('The value should be a valid JSON object'))
+    except ValueError as e:
+        raise Invalid(_('Could not parse the value as a valid JSON object'))
+
+    return value
+
+
+def extras_valid_json(extras, context):
+    try:
+        for extra, value in iteritems(extras):
+            json.dumps(value)
+    except ValueError as e:
+        raise Invalid(_(u'Could not parse extra \'{name}\' as valid JSON').
+                format(name=extra))
+    return extras
