@@ -1,3 +1,5 @@
+# encoding: utf-8
+
 from sqlalchemy import *
 import sqlalchemy.schema
 import uuid
@@ -6,7 +8,7 @@ from migrate import *
 import migrate.changeset
 from migrate.changeset.constraint import ForeignKeyConstraint, PrimaryKeyConstraint
 
-metadata = MetaData()
+metadata = None
 
 def make_uuid():
     return unicode(uuid.uuid4())
@@ -16,6 +18,8 @@ def make_uuid():
 # 3 create foreign key constraints (using cascade!)
 # 4 create uuids for revisions (auto cascades elsewhere!)
 def upgrade(migrate_engine):
+    global metadata
+    metadata = MetaData()
     metadata.bind = migrate_engine
     dropped_fk_constraints = drop_constraints_and_alter_types()
     upgrade2(migrate_engine, dropped_fk_constraints)
@@ -37,7 +41,10 @@ def drop_constraints_and_alter_types():
                     dropped_fk_constraints.append((constraint.columns, foreign_key_cols, constraint.name, table.name))
 
     # 2 alter type of revision id and foreign keys
-                    id_col = constraint.table.columns[constraint.columns[0]]
+                    # Sanity check
+                    if len(constraint.columns.keys()) != 1:
+                        raise ValueError()
+                    id_col = constraint.table.columns[constraint.columns.keys()[0]]
                     id_col.alter(type=UnicodeText)
 
     revision_table = Table('revision', metadata, autoload=True)
@@ -61,13 +68,16 @@ def upgrade2(migrate_engine, dropped_fk_constraints):
 
         # So we create via hand ...
         constraint_columns, foreign_key_cols, constraint_name, table_name = fk_constraint
+        # Sanity check
+        if len(constraint_columns.keys()) != 1:
+            raise ValueError()
+
         oursql = '''ALTER TABLE %(table)s
             ADD CONSTRAINT %(fkeyname)s
             FOREIGN KEY (%(col_name)s)
             REFERENCES revision (id)
-            ON UPDATE CASCADE
             ''' % {'table':table_name, 'fkeyname':constraint_name,
-                    'col_name':constraint_columns[0] }
+                    'col_name':constraint_columns.keys()[0] }
         migrate_engine.execute(oursql)
 
     # 4 create uuids for revisions and in related tables
