@@ -1,30 +1,30 @@
-import sqlalchemy
+# encoding: utf-8
+
 import meta
 import datetime
+import sqlalchemy
+
+import core
+import ckan.model
 import domain_object
 
-class UserFollowingUser(domain_object.DomainObject):
-    '''A many-many relationship between users.
 
-    A relationship between one user (the follower) and another (the object),
-    that means that the follower is currently following the object.
-
-    '''
+class ModelFollowingModel(domain_object.DomainObject):
     def __init__(self, follower_id, object_id):
         self.follower_id = follower_id
         self.object_id = object_id
         self.datetime = datetime.datetime.now()
 
     @classmethod
-    def get(self, follower_id, object_id):
-        '''Return a UserFollowingUser object for the given follower_id and
+    def get(cls, follower_id, object_id):
+        '''Return a ModelFollowingModel object for the given follower_id and
         object_id, or None if no such follower exists.
 
         '''
-        query = meta.Session.query(UserFollowingUser)
-        query = query.filter(UserFollowingUser.follower_id==follower_id)
-        query = query.filter(UserFollowingUser.object_id==object_id)
-        return query.first()
+        query = cls._get(follower_id, object_id)
+        following = cls._filter_following_objects(query)
+        if len(following) == 1:
+            return following[0]
 
     @classmethod
     def is_following(cls, follower_id, object_id):
@@ -32,33 +32,78 @@ class UserFollowingUser(domain_object.DomainObject):
         otherwise.
 
         '''
-        return UserFollowingUser.get(follower_id, object_id) is not None
-
+        return cls.get(follower_id, object_id) is not None
 
     @classmethod
     def followee_count(cls, follower_id):
-        '''Return the number of users followed by a user.'''
-        return meta.Session.query(UserFollowingUser).filter(
-                UserFollowingUser.follower_id == follower_id).count()
+        '''Return the number of objects followed by the follower.'''
+        return cls._get_followees(follower_id).count()
 
     @classmethod
     def followee_list(cls, follower_id):
-        '''Return a list of users followed by a user.'''
-        return meta.Session.query(UserFollowingUser).filter(
-                UserFollowingUser.follower_id == follower_id).all()
-
+        '''Return a list of objects followed by the follower.'''
+        query = cls._get_followees(follower_id).all()
+        followees = cls._filter_following_objects(query)
+        return followees
 
     @classmethod
-    def follower_count(cls, user_id):
-        '''Return the number of followers of a user.'''
-        return meta.Session.query(UserFollowingUser).filter(
-                UserFollowingUser.object_id == user_id).count()
+    def follower_count(cls, object_id):
+        '''Return the number of followers of the object.'''
+        return cls._get_followers(object_id).count()
 
     @classmethod
-    def follower_list(cls, user_id):
-        '''Return a list of followers of a user.'''
-        return meta.Session.query(UserFollowingUser).filter(
-                UserFollowingUser.object_id == user_id).all()
+    def follower_list(cls, object_id):
+        '''Return a list of followers of the object.'''
+        query = cls._get_followers(object_id).all()
+        followers = cls._filter_following_objects(query)
+        return followers
+
+    @classmethod
+    def _filter_following_objects(cls, query):
+        return [q[0] for q in query]
+
+    @classmethod
+    def _get_followees(cls, follower_id):
+        return cls._get(follower_id)
+
+    @classmethod
+    def _get_followers(cls, object_id):
+        return cls._get(None, object_id)
+
+    @classmethod
+    def _get(cls, follower_id=None, object_id=None):
+        follower_alias = sqlalchemy.orm.aliased(cls._follower_class())
+        object_alias = sqlalchemy.orm.aliased(cls._object_class())
+
+        follower_id = follower_id or cls.follower_id
+        object_id = object_id or cls.object_id
+
+        query = meta.Session.query(cls, follower_alias, object_alias)\
+            .filter(sqlalchemy.and_(
+                follower_alias.id == follower_id,
+                cls.follower_id == follower_alias.id,
+                cls.object_id == object_alias.id,
+                follower_alias.state != core.State.DELETED,
+                object_alias.state != core.State.DELETED,
+                object_alias.id == object_id))
+
+        return query
+
+
+class UserFollowingUser(ModelFollowingModel):
+    '''A many-many relationship between users.
+
+    A relationship between one user (the follower) and another (the object),
+    that means that the follower is currently following the object.
+
+    '''
+    @classmethod
+    def _follower_class(cls):
+        return ckan.model.User
+
+    @classmethod
+    def _object_class(cls):
+        return ckan.model.User
 
 
 user_following_user_table = sqlalchemy.Table('user_following_user',
@@ -76,62 +121,20 @@ user_following_user_table = sqlalchemy.Table('user_following_user',
 
 meta.mapper(UserFollowingUser, user_following_user_table)
 
-class UserFollowingDataset(domain_object.DomainObject):
+class UserFollowingDataset(ModelFollowingModel):
     '''A many-many relationship between users and datasets (packages).
 
     A relationship between a user (the follower) and a dataset (the object),
     that means that the user is currently following the dataset.
 
     '''
-    def __init__(self, follower_id, object_id):
-        self.follower_id = follower_id
-        self.object_id = object_id
-        self.datetime = datetime.datetime.now()
+    @classmethod
+    def _follower_class(cls):
+        return ckan.model.User
 
     @classmethod
-    def get(self, follower_id, object_id):
-        '''Return a UserFollowingDataset object for the given follower_id and
-        object_id, or None if no such follower exists.
-
-        '''
-        query = meta.Session.query(UserFollowingDataset)
-        query = query.filter(UserFollowingDataset.follower_id==follower_id)
-        query = query.filter(UserFollowingDataset.object_id==object_id)
-        return query.first()
-
-    @classmethod
-    def is_following(cls, follower_id, object_id):
-        '''Return True if follower_id is currently following object_id, False
-        otherwise.
-
-        '''
-        return UserFollowingDataset.get(follower_id, object_id) is not None
-
-
-    @classmethod
-    def followee_count(cls, follower_id):
-        '''Return the number of datasets followed by a user.'''
-        return meta.Session.query(UserFollowingDataset).filter(
-                UserFollowingDataset.follower_id == follower_id).count()
-
-    @classmethod
-    def followee_list(cls, follower_id):
-        '''Return a list of datasets followed by a user.'''
-        return meta.Session.query(UserFollowingDataset).filter(
-                UserFollowingDataset.follower_id == follower_id).all()
-
-
-    @classmethod
-    def follower_count(cls, dataset_id):
-        '''Return the number of followers of a dataset.'''
-        return meta.Session.query(UserFollowingDataset).filter(
-                UserFollowingDataset.object_id == dataset_id).count()
-
-    @classmethod
-    def follower_list(cls, dataset_id):
-        '''Return a list of followers of a dataset.'''
-        return meta.Session.query(UserFollowingDataset).filter(
-                UserFollowingDataset.object_id == dataset_id).all()
+    def _object_class(cls):
+        return ckan.model.Package
 
 
 user_following_dataset_table = sqlalchemy.Table('user_following_dataset',
@@ -150,60 +153,20 @@ user_following_dataset_table = sqlalchemy.Table('user_following_dataset',
 meta.mapper(UserFollowingDataset, user_following_dataset_table)
 
 
-class UserFollowingGroup(domain_object.DomainObject):
+class UserFollowingGroup(ModelFollowingModel):
     '''A many-many relationship between users and groups.
 
     A relationship between a user (the follower) and a group (the object),
     that means that the user is currently following the group.
 
     '''
-    def __init__(self, follower_id, object_id):
-        self.follower_id = follower_id
-        self.object_id = object_id
-        self.datetime = datetime.datetime.now()
+    @classmethod
+    def _follower_class(cls):
+        return ckan.model.User
 
     @classmethod
-    def get(self, follower_id, object_id):
-        '''Return a UserFollowingGroup object for the given follower_id and
-        object_id, or None if no such relationship exists.
-
-        '''
-        query = meta.Session.query(UserFollowingGroup)
-        query = query.filter(UserFollowingGroup.follower_id == follower_id)
-        query = query.filter(UserFollowingGroup.object_id == object_id)
-        return query.first()
-
-    @classmethod
-    def is_following(cls, follower_id, object_id):
-        '''Return True if follower_id is currently following object_id, False
-        otherwise.
-
-        '''
-        return UserFollowingGroup.get(follower_id, object_id) is not None
-
-    @classmethod
-    def followee_count(cls, follower_id):
-        '''Return the number of groups followed by a user.'''
-        return meta.Session.query(UserFollowingGroup).filter(
-                UserFollowingGroup.follower_id == follower_id).count()
-
-    @classmethod
-    def followee_list(cls, follower_id):
-        '''Return a list of groups followed by a user.'''
-        return meta.Session.query(UserFollowingGroup).filter(
-                UserFollowingGroup.follower_id == follower_id).all()
-
-    @classmethod
-    def follower_count(cls, object_id):
-        '''Return the number of users following a group.'''
-        return meta.Session.query(UserFollowingGroup).filter(
-                UserFollowingGroup.object_id == object_id).count()
-
-    @classmethod
-    def follower_list(cls, object_id):
-        '''Return a list of the users following a group.'''
-        return meta.Session.query(UserFollowingGroup).filter(
-                UserFollowingGroup.object_id == object_id).all()
+    def _object_class(cls):
+        return ckan.model.Group
 
 user_following_group_table = sqlalchemy.Table('user_following_group',
         meta.metadata,
