@@ -1,16 +1,33 @@
+# encoding: utf-8
+
 import datetime
 import urllib2
 import re
 
-from pylons import config
+from ckan.common import config
+from paste.deploy.converters import asbool
+from six import text_type, string_types
 
 from ckan.common import _, json
+import ckan.lib.maintain as maintain
+
+log = __import__('logging').getLogger(__name__)
 
 
 class License(object):
     """Domain object for a license."""
 
     def __init__(self, data):
+        # convert old keys if necessary
+        if 'is_okd_compliant' in data:
+            data['od_conformance'] = 'approved' \
+                if asbool(data['is_okd_compliant']) else ''
+            del data['is_okd_compliant']
+        if 'is_osi_compliant' in data:
+            data['osd_conformance'] = 'approved' \
+                if asbool(data['is_osi_compliant']) else ''
+            del data['is_osi_compliant']
+
         self._data = data
         for (key, value) in self._data.items():
             if key == 'date_created':
@@ -23,20 +40,50 @@ class License(object):
                 self._data[key] = value
 
     def __getattr__(self, name):
+        if name == 'is_okd_compliant':
+            log.warn('license.is_okd_compliant is deprecated - use '
+                     'od_conformance instead.')
+            return self._data['od_conformance'] == 'approved'
+        if name == 'is_osi_compliant':
+            log.warn('license.is_osi_compliant is deprecated - use '
+                     'osd_conformance instead.')
+            return self._data['osd_conformance'] == 'approved'
         return self._data[name]
 
+    @maintain.deprecated("License.__getitem__() is deprecated and will be "
+                         "removed in a future version of CKAN. Instead, "
+                         "please use attribute access.")
     def __getitem__(self, key):
-        return self._data[key]
+        '''NB This method is deprecated and will be removed in a future version
+        of CKAN. Instead, please use attribute access.
+        '''
+        return self.__getattr__(key)
 
     def isopen(self):
-        return self.is_okd_compliant or self.is_osi_compliant
+        if not hasattr(self, '_isopen'):
+            self._isopen = self.od_conformance == 'approved' or \
+                self.osd_conformance == 'approved'
+        return self._isopen
 
+    @maintain.deprecated("License.as_dict() is deprecated and will be "
+                         "removed in a future version of CKAN. Instead, "
+                         "please use attribute access.")
     def as_dict(self):
+        '''NB This method is deprecated and will be removed in a future version
+        of CKAN. Instead, please use attribute access.
+        '''
         data = self._data.copy()
         if 'date_created' in data:
             value = data['date_created']
             value = value.isoformat()
             data['date_created'] = value
+
+        # deprecated keys
+        if 'od_conformance' in data:
+            data['is_okd_compliant'] = data['od_conformance'] == 'approved'
+        if 'osd_conformance' in data:
+            data['is_osi_compliant'] = data['osd_conformance'] == 'approved'
+
         return data
 
 
@@ -71,14 +118,19 @@ class LicenseRegister(object):
         try:
             response = urllib2.urlopen(license_url)
             response_body = response.read()
-        except Exception, inst:
+        except Exception as inst:
             msg = "Couldn't connect to licenses service %r: %s" % (license_url, inst)
-            raise Exception, msg
+            raise Exception(msg)
         try:
             license_data = json.loads(response_body)
-        except Exception, inst:
+        except Exception as inst:
             msg = "Couldn't read response from licenses service %r: %s" % (response_body, inst)
-            raise Exception, inst
+            raise Exception(inst)
+        for license in license_data:
+            if isinstance(license, string_types):
+                license = license_data[license]
+            if license.get('title'):
+                license['title'] = _(license['title'])
         self._create_license_list(license_data, license_url)
 
     def _create_license_list(self, license_data, license_url=''):
@@ -97,7 +149,7 @@ class LicenseRegister(object):
         if default != Exception:
             return default
         else:
-            raise KeyError, "License not found: %s" % key
+            raise KeyError("License not found: %s" % key)
 
     def get(self, key, default=None):
         return self.__getitem__(key, default=default)
@@ -118,7 +170,6 @@ class LicenseRegister(object):
         return len(self.licenses)
 
 
-
 class DefaultLicense(dict):
     ''' The license was a dict but this did not allow translation of the
     title.  This is a slightly changed dict that allows us to have the title
@@ -127,13 +178,13 @@ class DefaultLicense(dict):
     domain_content = False
     domain_data = False
     domain_software = False
-    family = ""
+    family = ''
     is_generic = False
-    is_okd_compliant = False
-    is_osi_compliant = False
-    maintainer = ""
-    status = "active"
-    url = ""
+    od_conformance = 'not reviewed'
+    osd_conformance = 'not reviewed'
+    maintainer = ''
+    status = 'active'
+    url = ''
     title = ''
     id = ''
 
@@ -143,8 +194,8 @@ class DefaultLicense(dict):
             'domain_software',
             'family',
             'is_generic',
-            'is_okd_compliant',
-            'is_osi_compliant',
+            'od_conformance',
+            'osd_conformance',
             'maintainer',
             'status',
             'url',
@@ -155,7 +206,7 @@ class DefaultLicense(dict):
         if key in self.keys:
             value = getattr(self, key)
             if isinstance(value, str):
-                return unicode(value)
+                return text_type(value)
             else:
                 return value
         else:
@@ -165,7 +216,7 @@ class DefaultLicense(dict):
         ''' create a dict of the license used by the licenses api '''
         out = {}
         for key in self.keys:
-            out[key] = unicode(getattr(self, key))
+            out[key] = text_type(getattr(self, key))
         return out
 
 class LicenseNotSpecified(DefaultLicense):
@@ -174,12 +225,12 @@ class LicenseNotSpecified(DefaultLicense):
 
     @property
     def title(self):
-        return _("License Not Specified")
+        return _("License not specified")
 
 class LicenseOpenDataCommonsPDDL(DefaultLicense):
     domain_data = True
     id = "odc-pddl"
-    is_okd_compliant = True
+    od_conformance = 'approved'
     url = "http://www.opendefinition.org/licenses/odc-pddl"
 
     @property
@@ -189,7 +240,7 @@ class LicenseOpenDataCommonsPDDL(DefaultLicense):
 class LicenseOpenDataCommonsOpenDatabase(DefaultLicense):
     domain_data = True
     id = "odc-odbl"
-    is_okd_compliant = True
+    od_conformance = 'approved'
     url = "http://www.opendefinition.org/licenses/odc-odbl"
 
     @property
@@ -199,7 +250,7 @@ class LicenseOpenDataCommonsOpenDatabase(DefaultLicense):
 class LicenseOpenDataAttribution(DefaultLicense):
     domain_data = True
     id = "odc-by"
-    is_okd_compliant = True
+    od_conformance = 'approved'
     url = "http://www.opendefinition.org/licenses/odc-by"
 
     @property
@@ -210,7 +261,7 @@ class LicenseCreativeCommonsZero(DefaultLicense):
     domain_content = True
     domain_data = True
     id = "cc-zero"
-    is_okd_compliant = True
+    od_conformance = 'approved'
     url = "http://www.opendefinition.org/licenses/cc-zero"
 
     @property
@@ -219,7 +270,7 @@ class LicenseCreativeCommonsZero(DefaultLicense):
 
 class LicenseCreativeCommonsAttribution(DefaultLicense):
     id = "cc-by"
-    is_okd_compliant = True
+    od_conformance = 'approved'
     url = "http://www.opendefinition.org/licenses/cc-by"
 
     @property
@@ -229,7 +280,7 @@ class LicenseCreativeCommonsAttribution(DefaultLicense):
 class LicenseCreativeCommonsAttributionShareAlike(DefaultLicense):
     domain_content = True
     id = "cc-by-sa"
-    is_okd_compliant = True
+    od_conformance = 'approved'
     url = "http://www.opendefinition.org/licenses/cc-by-sa"
 
     @property
@@ -239,7 +290,7 @@ class LicenseCreativeCommonsAttributionShareAlike(DefaultLicense):
 class LicenseGNUFreeDocument(DefaultLicense):
     domain_content = True
     id = "gfdl"
-    is_okd_compliant = True
+    od_conformance = 'approved'
     url = "http://www.opendefinition.org/licenses/gfdl"
     @property
     def title(self):
@@ -249,7 +300,7 @@ class LicenseOtherOpen(DefaultLicense):
     domain_content = True
     id = "other-open"
     is_generic = True
-    is_okd_compliant = True
+    od_conformance = 'approved'
 
     @property
     def title(self):
@@ -259,7 +310,7 @@ class LicenseOtherPublicDomain(DefaultLicense):
     domain_content = True
     id = "other-pd"
     is_generic = True
-    is_okd_compliant = True
+    od_conformance = 'approved'
 
     @property
     def title(self):
@@ -269,7 +320,7 @@ class LicenseOtherAttribution(DefaultLicense):
     domain_content = True
     id = "other-at"
     is_generic = True
-    is_okd_compliant = True
+    od_conformance = 'approved'
 
     @property
     def title(self):
@@ -278,7 +329,7 @@ class LicenseOtherAttribution(DefaultLicense):
 class LicenseOpenGovernment(DefaultLicense):
     domain_content = True
     id = "uk-ogl"
-    is_okd_compliant = True
+    od_conformance = 'approved'
     # CS: bad_spelling ignore
     url = "http://reference.data.gov.uk/id/open-government-licence"
 
