@@ -1,8 +1,9 @@
+# encoding: utf-8
+
 """
 Contains miscelaneous set of DB-related functions
 """
 
-import re
 
 _special_characters = '%_'
 def escape_sql_like_special_characters(term, escape='\\'):
@@ -14,4 +15,3 @@ def escape_sql_like_special_characters(term, escape='\\'):
     for ch in escape + _special_characters:
         term = term.replace(ch, escape+ch)
     return term
-    
