@@ -1,3 +1,5 @@
+# encoding: utf-8
+
 import datetime
 from calendar import timegm
 import logging
@@ -6,7 +8,7 @@ logger = logging.getLogger(__name__)
 from sqlalchemy.sql import select, and_, union, or_
 from sqlalchemy import orm
 from sqlalchemy import types, Column, Table
-from pylons import config
+from ckan.common import config
 import vdm.sqlalchemy
 
 import meta
@@ -22,7 +24,9 @@ import ckan.lib.dictization as dictization
 
 __all__ = ['Package', 'package_table', 'package_revision_table',
            'PACKAGE_NAME_MAX_LENGTH', 'PACKAGE_NAME_MIN_LENGTH',
-           'PACKAGE_VERSION_MAX_LENGTH', 'PackageTagRevision', 'PackageRevision']
+           'PACKAGE_VERSION_MAX_LENGTH', 'PackageTag', 'PackageTagRevision',
+           'PackageRevision']
+
 
 PACKAGE_NAME_MAX_LENGTH = 100
 PACKAGE_NAME_MIN_LENGTH = 2
@@ -45,6 +49,7 @@ package_table = Table('package', meta.metadata,
         Column('type', types.UnicodeText, default=u'dataset'),
         Column('owner_org', types.UnicodeText),
         Column('creator_user_id', types.UnicodeText),
+        Column('metadata_created', types.DateTime, default=datetime.datetime.utcnow),
         Column('metadata_modified', types.DateTime, default=datetime.datetime.utcnow),
         Column('private', types.Boolean, default=False),
 )
@@ -65,8 +70,6 @@ class Package(vdm.sqlalchemy.RevisionedObjectMixin,
     def __init__(self, **kw):
         from ckan import model
         super(Package, self).__init__(**kw)
-        resource_group = model.ResourceGroup(label="default")
-        self.resource_groups_all.append(resource_group)
 
     @classmethod
     def search_by_name(cls, text_query):
@@ -76,8 +79,10 @@ class Package(vdm.sqlalchemy.RevisionedObjectMixin,
     @classmethod
     def get(cls, reference):
         '''Returns a package object referenced by its id or name.'''
-        query = meta.Session.query(cls).filter(cls.id==reference)
-        pkg = query.first()
+        if not reference:
+            return None
+
+        pkg = meta.Session.query(cls).get(reference)
         if pkg == None:
             pkg = cls.by_name(reference)
         return pkg
@@ -85,21 +90,17 @@ class Package(vdm.sqlalchemy.RevisionedObjectMixin,
 
     @property
     def resources(self):
-        if len(self.resource_groups_all) == 0:
-            return []
-
-        assert len(self.resource_groups_all) == 1, "can only use resources on packages if there is only one resource_group"
-        return [resource for resource in 
-                self.resource_groups_all[0].resources_all
-                if resource.state <> 'deleted']
+        return [resource for resource in
+                self.resources_all
+                if resource.state != 'deleted']
 
     def related_packages(self):
         return [self]
 
     def add_resource(self, url, format=u'', description=u'', hash=u'', **kw):
         import resource
-        self.resource_groups_all[0].resources_all.append(resource.Resource(
-            resource_group_id=self.resource_groups_all[0].id,
+        self.resources_all.append(resource.Resource(
+            package_id=self.id,
             url=url,
             format=format,
             description=description,
@@ -156,12 +157,10 @@ class Package(vdm.sqlalchemy.RevisionedObjectMixin,
         """
         import ckan.model as model
         query = meta.Session.query(model.Tag)
-        query = query.join(PackageTagRevision)
-        query = query.filter(PackageTagRevision.tag_id == model.Tag.id)
-        query = query.filter(PackageTagRevision.package_id == self.id)
-        query = query.filter(and_(
-            PackageTagRevision.state == 'active',
-            PackageTagRevision.current == True))
+        query = query.join(model.PackageTag)
+        query = query.filter(model.PackageTag.tag_id == model.Tag.id)
+        query = query.filter(model.PackageTag.package_id == self.id)
+        query = query.filter(model.PackageTag.state == 'active')
         if vocab:
             query = query.filter(model.Tag.vocabulary_id == vocab.id)
         else:
@@ -205,7 +204,7 @@ class Package(vdm.sqlalchemy.RevisionedObjectMixin,
         groups = [getattr(group, ref_group_by) for group in self.get_groups()]
         groups.sort()
         _dict['groups'] = groups
-        _dict['extras'] = dict([(key, value) for key, value in self.extras.items()])
+        _dict['extras'] = {key: value for key, value in self.extras.items()}
         _dict['ratings_average'] = self.get_average_rating()
         _dict['ratings_count'] = len(self.ratings)
         _dict['resources'] = [res.as_dict(core_columns_only=False) \
@@ -233,16 +232,18 @@ class Package(vdm.sqlalchemy.RevisionedObjectMixin,
         if type_ in package_relationship.PackageRelationship.get_forward_types():
             subject = self
             object_ = related_package
+            direction = "forward"
         elif type_ in package_relationship.PackageRelationship.get_reverse_types():
             type_ = package_relationship.PackageRelationship.reverse_to_forward_type(type_)
             assert type_
             subject = related_package
             object_ = self
+            direction = "reverse"
         else:
-            raise KeyError, 'Package relationship type: %r' % type_
+            raise KeyError('Package relationship type: %r' % type_)
 
         rels = self.get_relationships(with_package=related_package,
-                                      type=type_, active=False, direction="forward")
+                                      type=type_, active=False, direction=direction)
         if rels:
             rel = rels[0]
             if comment:
@@ -363,7 +364,7 @@ class Package(vdm.sqlalchemy.RevisionedObjectMixin,
             self.license_id = license['id']
         else:
             msg = "Value not a license object or entity: %s" % repr(license)
-            raise Exception, msg
+            raise Exception(msg)
 
     license = property(get_license, set_license)
 
@@ -377,7 +378,7 @@ class Package(vdm.sqlalchemy.RevisionedObjectMixin,
                 Ordered by most recent first.
         '''
         from tag import PackageTag
-        from resource import ResourceGroup, Resource
+        from resource import Resource
         from package_extra import PackageExtra
 
         results = {} # revision:[PackageRevision1, PackageTagRevision1, etc.]
@@ -385,23 +386,16 @@ class Package(vdm.sqlalchemy.RevisionedObjectMixin,
             if not results.has_key(pkg_rev.revision):
                 results[pkg_rev.revision] = []
             results[pkg_rev.revision].append(pkg_rev)
-        for class_ in [ResourceGroup, Resource, PackageExtra, PackageTag]:
+        for class_ in [Resource, PackageExtra, PackageTag]:
             rev_class = class_.__revision_class__
-            if class_ == Resource:
-                q = meta.Session.query(rev_class).join('continuity',
-                                                  'resource_group')
-                obj_revisions = q.filter(ResourceGroup.package_id == self.id).all()
-            else:
-                obj_revisions = meta.Session.query(rev_class).filter_by(package_id=self.id).all()
+            obj_revisions = meta.Session.query(rev_class).filter_by(package_id=self.id).all()
             for obj_rev in obj_revisions:
                 if not results.has_key(obj_rev.revision):
                     results[obj_rev.revision] = []
                 results[obj_rev.revision].append(obj_rev)
 
         result_list = results.items()
-        ourcmp = lambda rev_tuple1, rev_tuple2: \
-                 cmp(rev_tuple2[0].timestamp, rev_tuple1[0].timestamp)
-        return sorted(result_list, cmp=ourcmp)
+        return sorted(result_list, key=lambda x: x[0].timestamp, reverse=True)
 
     @property
     def latest_related_revision(self):
@@ -413,32 +407,23 @@ class Package(vdm.sqlalchemy.RevisionedObjectMixin,
         '''Overrides the diff in vdm, so that related obj revisions are
         diffed as well as PackageRevisions'''
         from tag import PackageTag
-        from resource import ResourceGroup, Resource
+        from resource import Resource
         from package_extra import PackageExtra
 
         results = {} # field_name:diffs
         results.update(super(Package, self).diff(to_revision, from_revision))
         # Iterate over PackageTag, PackageExtra, Resources etc.
-        for obj_class in [ResourceGroup, Resource, PackageExtra, PackageTag]:
+        for obj_class in [Resource, PackageExtra, PackageTag]:
             obj_rev_class = obj_class.__revision_class__
             # Query for object revisions related to this package
-            if obj_class == Resource:
-                obj_rev_query = meta.Session.query(obj_rev_class).\
-                                join('continuity', 'resource_group').\
-                                join('revision').\
-                                filter(ResourceGroup.package_id == self.id).\
-                                order_by(core.Revision.timestamp.desc())
-            else:
-                obj_rev_query = meta.Session.query(obj_rev_class).\
-                                filter_by(package_id=self.id).\
-                                join('revision').\
-                                order_by(core.Revision.timestamp.desc())
+            obj_rev_query = meta.Session.query(obj_rev_class).\
+                            filter_by(package_id=self.id).\
+                            join('revision').\
+                            order_by(core.Revision.timestamp.desc())
             # Columns to include in the diff
             cols_to_diff = obj_class.revisioned_fields()
             cols_to_diff.remove('id')
             if obj_class is Resource:
-                cols_to_diff.remove('resource_group_id')
-            else:
                 cols_to_diff.remove('package_id')
             # Particular object types are better known by an invariant field
             if obj_class is PackageTag:
@@ -501,16 +486,6 @@ class Package(vdm.sqlalchemy.RevisionedObjectMixin,
             groups = [g[0] for g in groupcaps if g[1] == capacity]
         return groups
 
-    @property
-    def metadata_created(self):
-        import ckan.model as model
-        q = meta.Session.query(model.PackageRevision.revision_timestamp)\
-            .filter(model.PackageRevision.id == self.id)\
-            .order_by(model.PackageRevision.revision_timestamp.asc())
-        ts = q.first()
-        if ts:
-            return ts[0]
-
     @staticmethod
     def get_fields(core_only=False, fields_to_ignore=None):
         '''Returns a list of the properties of a package.
@@ -578,6 +553,55 @@ class Package(vdm.sqlalchemy.RevisionedObjectMixin,
         return activity.ActivityDetail(activity_id, self.id, u"Package", activity_type,
             {'package': package_dict })
 
+    def set_rating(self, user_or_ip, rating):
+        '''Record a user's rating of this package.
+
+        The caller function is responsible for doing the commit.
+
+        If a rating is outside the range MAX_RATING - MIN_RATING then a
+        RatingValueException is raised.
+
+        @param user_or_ip - user object or an IP address string
+        '''
+        user = None
+        from user import User
+        from rating import Rating, MAX_RATING, MIN_RATING
+        if isinstance(user_or_ip, User):
+            user = user_or_ip
+            rating_query = meta.Session.query(Rating)\
+                               .filter_by(package=self, user=user)
+        else:
+            ip = user_or_ip
+            rating_query = meta.Session.query(Rating)\
+                               .filter_by(package=self, user_ip_address=ip)
+
+        try:
+            rating = float(rating)
+        except TypeError:
+            raise RatingValueException
+        except ValueError:
+            raise RatingValueException
+        if rating > MAX_RATING or rating < MIN_RATING:
+            raise RatingValueException
+
+        if rating_query.count():
+            rating_obj = rating_query.first()
+            rating_obj.rating = rating
+        elif user:
+            rating = Rating(package=self,
+                            user=user,
+                            rating=rating)
+            meta.Session.add(rating)
+        else:
+            rating = Rating(package=self,
+                            user_ip_address=ip,
+                            rating=rating)
+            meta.Session.add(rating)
+
+
+class RatingValueException(Exception):
+    pass
+
 # import here to prevent circular import
 import tag
 
