@@ -1,12 +1,12 @@
-import sqlalchemy as sa
-from meta import *
-from core import *
-from types import make_uuid
-from datetime import datetime
+# encoding: utf-8
+
+from sqlalchemy import Column, Table
+from sqlalchemy.types import UnicodeText
+import meta
 
 __all__ = ['term_translation_table']
 
-term_translation_table = Table('term_translation', metadata,
+term_translation_table = Table('term_translation', meta.metadata,
     Column('term', UnicodeText, nullable=False),
     Column('term_translation', UnicodeText, nullable=False),
     Column('lang_code', UnicodeText, nullable=False),
