@@ -1,11 +1,16 @@
+# encoding: utf-8
+
 import datetime
 import re
 import os
 from hashlib import sha1, md5
 
+import passlib.utils
+from passlib.hash import pbkdf2_sha512
 from sqlalchemy.sql.expression import or_
 from sqlalchemy.orm import synonym
-from sqlalchemy import types, Column, Table
+from sqlalchemy import types, Column, Table, func
+from six import text_type
 import vdm.sqlalchemy
 
 import meta
@@ -13,11 +18,11 @@ import core
 import types as _types
 import domain_object
 
+
 user_table = Table('user', meta.metadata,
         Column('id', types.UnicodeText, primary_key=True,
                default=_types.make_uuid),
         Column('name', types.UnicodeText, nullable=False, unique=True),
-        Column('openid', types.UnicodeText),
         Column('password', types.UnicodeText),
         Column('fullname', types.UnicodeText),
         Column('email', types.UnicodeText),
@@ -39,24 +44,14 @@ class User(vdm.sqlalchemy.StatefulObjectMixin,
     VALID_NAME = re.compile(r"^[a-zA-Z0-9_\-]{3,255}$")
     DOUBLE_SLASH = re.compile(':\/([^/])')
 
-    @classmethod
-    def by_openid(cls, openid):
-        obj = meta.Session.query(cls).autoflush(False)
-        return obj.filter_by(openid=openid).first()
-
     @classmethod
     def by_email(cls, email):
         return meta.Session.query(cls).filter_by(email=email).all()
 
     @classmethod
     def get(cls, user_reference):
-        # double slashes in an openid often get turned into single slashes
-        # by browsers, so correct that for the openid lookup
-        corrected_openid_user_ref = cls.DOUBLE_SLASH.sub('://\\1',
-                                                         user_reference)
         query = meta.Session.query(cls).autoflush(False)
         query = query.filter(or_(cls.name == user_reference,
-                                 cls.openid == corrected_openid_user_ref,
                                  cls.id == user_reference))
         return query.first()
 
@@ -84,40 +79,61 @@ class User(vdm.sqlalchemy.StatefulObjectMixin,
         return md5(e).hexdigest()
 
     def get_reference_preferred_for_uri(self):
-        '''Returns a reference (e.g. name, id, openid) for this user
+        '''Returns a reference (e.g. name, id) for this user
         suitable for the user\'s URI.
         When there is a choice, the most preferable one will be
-        given, based on readability. This is expected when repoze.who can
-        give a more friendly name for an openid user.
+        given, based on readability.
         The result is not escaped (will get done in url_for/redirect_to).
         '''
         if self.name:
             ref = self.name
-        elif self.openid:
-            ref = self.openid
         else:
             ref = self.id
         return ref
 
     def _set_password(self, password):
-        '''Hash password on the fly.'''
-        if isinstance(password, unicode):
-            password_8bit = password.encode('ascii', 'ignore')
-        else:
-            password_8bit = password
+        '''Hash using pbkdf2
 
-        salt = sha1(os.urandom(60))
-        hash = sha1(password_8bit + salt.hexdigest())
-        hashed_password = salt.hexdigest() + hash.hexdigest()
+        Use passlib to hash the password using pkbdf2, upgrading
+        passlib will also upgrade the number of rounds and salt of the
+        hash as the user logs in automatically. Changing hashing
+        algorithm will require this code to be changed (perhaps using
+        passlib's CryptContext)
+        '''
+        hashed_password = pbkdf2_sha512.encrypt(password)
 
-        if not isinstance(hashed_password, unicode):
+        if not isinstance(hashed_password, text_type):
             hashed_password = hashed_password.decode('utf-8')
         self._password = hashed_password
 
     def _get_password(self):
-        '''Return the password hashed'''
         return self._password
 
+    def _verify_and_upgrade_from_sha1(self, password):
+        if isinstance(password, text_type):
+            password_8bit = password.encode('ascii', 'ignore')
+        else:
+            password_8bit = password
+
+        hashed_pass = sha1(password_8bit + self.password[:40])
+        current_hash = passlib.utils.to_native_str(self.password[40:])
+
+        if passlib.utils.consteq(hashed_pass.hexdigest(), current_hash):
+            #we've passed the old sha1 check, upgrade our password
+            self._set_password(password)
+            self.save()
+            return True
+        else:
+            return False
+
+    def _verify_and_upgrade_pbkdf2(self, password):
+        if pbkdf2_sha512.verify(password, self.password):
+            self._set_password(password)
+            self.save()
+            return True
+        else:
+            return False
+
     def validate_password(self, password):
         '''
         Check the password against existing credentials.
@@ -131,12 +147,17 @@ class User(vdm.sqlalchemy.StatefulObjectMixin,
         '''
         if not password or not self.password:
             return False
-        if isinstance(password, unicode):
-            password_8bit = password.encode('ascii', 'ignore')
+
+        if not pbkdf2_sha512.identify(self.password):
+            return self._verify_and_upgrade_from_sha1(password)
         else:
-            password_8bit = password
-        hashed_pass = sha1(password_8bit + self.password[:40])
-        return self.password[40:] == hashed_pass.hexdigest()
+            current_hash = pbkdf2_sha512.from_string(self.password)
+            if (current_hash.rounds < pbkdf2_sha512.default_rounds or
+                len(current_hash.salt) < pbkdf2_sha512.default_salt_size):
+
+                return self._verify_and_upgrade_pbkdf2(password)
+            else:
+                return pbkdf2_sha512.verify(password, self.password)
 
     password = property(_get_password, _set_password)
 
@@ -160,16 +181,45 @@ class User(vdm.sqlalchemy.StatefulObjectMixin,
     def number_of_edits(self):
         # have to import here to avoid circular imports
         import ckan.model as model
-        revisions_q = meta.Session.query(model.Revision)
-        revisions_q = revisions_q.filter_by(author=self.name)
-        return revisions_q.count()
 
-    def number_administered_packages(self):
+        # Get count efficiently without spawning the SQLAlchemy subquery
+        # wrapper. Reset the VDM-forced order_by on timestamp.
+        return meta.Session.execute(
+            meta.Session.query(
+                model.Revision
+            ).filter_by(
+                author=self.name
+            ).statement.with_only_columns(
+                [func.count()]
+            ).order_by(
+                None
+            )
+        ).scalar()
+
+    def number_created_packages(self, include_private_and_draft=False):
         # have to import here to avoid circular imports
         import ckan.model as model
-        q = meta.Session.query(model.PackageRole)
-        q = q.filter_by(user=self, role=model.Role.ADMIN)
-        return q.count()
+
+        # Get count efficiently without spawning the SQLAlchemy subquery
+        # wrapper. Reset the VDM-forced order_by on timestamp.
+        q = meta.Session.query(
+            model.Package
+        ).filter_by(
+            creator_user_id=self.id
+        )
+
+        if include_private_and_draft:
+            q = q.filter(model.Package.state != 'deleted')
+        else:
+            q = q.filter_by(state='active', private=False)
+
+        return meta.Session.execute(
+            q.statement.with_only_columns(
+                [func.count()]
+            ).order_by(
+                None
+            )
+        ).scalar()
 
     def activate(self):
         ''' Activate the user '''
@@ -224,7 +274,7 @@ class User(vdm.sqlalchemy.StatefulObjectMixin,
 
     @classmethod
     def search(cls, querystr, sqlalchemy_query=None, user_name=None):
-        '''Search name, fullname, email and openid. '''
+        '''Search name, fullname, email. '''
         if sqlalchemy_query is None:
             query = meta.Session.query(cls)
         else:
@@ -233,16 +283,27 @@ class User(vdm.sqlalchemy.StatefulObjectMixin,
         filters = [
             cls.name.ilike(qstr),
             cls.fullname.ilike(qstr),
-            cls.openid.ilike(qstr),
         ]
         # sysadmins can search on user emails
-        import ckan.new_authz as new_authz
-        if user_name and new_authz.is_sysadmin(user_name):
+        import ckan.authz as authz
+        if user_name and authz.is_sysadmin(user_name):
             filters.append(cls.email.ilike(qstr))
 
         query = query.filter(or_(*filters))
         return query
 
+    @classmethod
+    def user_ids_for_name_or_id(self, user_list=[]):
+        '''
+        This function returns a list of ids from an input that can be a list of
+        names or ids
+        '''
+        query = meta.Session.query(self.id)
+        query = query.filter(or_(self.name.in_(user_list),
+                                 self.id.in_(user_list)))
+        return [user.id for user in query.all()]
+
+
 meta.mapper(User, user_table,
     properties={'password': synonym('_password', map_column=True)},
     order_by=user_table.c.name)
