@@ -4,7 +4,7 @@
  *
  * See the snippets/custom_form_fields.html for an example.
  */
-this.ckan.module('custom-fields', function (jQuery, _) {
+this.ckan.module('custom-fields', function (jQuery) {
   return {
     options: {
       /* The selector used for each custom field wrapper */
@@ -17,16 +17,11 @@ this.ckan.module('custom-fields', function (jQuery, _) {
      * Returns nothing.
      */
     initialize: function () {
-      if (!jQuery('html').hasClass('ie7')) {
-        jQuery.proxyAll(this, /_on/);
+      jQuery.proxyAll(this, /_on/);
 
-        var delegated = this.options.fieldSelector + ':last input:first';
-        this.el.on('change', delegated, this._onChange);
-        this.el.on('change', ':checkbox', this._onRemove);
-
-        // Style the remove checkbox like a button.
-        this.$('.checkbox').addClass("btn btn-danger icon-remove");
-      }
+      var delegated = this.options.fieldSelector + ':last input:first';
+      this.el.on('change', delegated, this._onChange);
+      this.el.on('change', ':checkbox', this._onRemove);
     },
 
     /* Creates a new field and appends it to the list. This currently works by
@@ -90,14 +85,14 @@ this.ckan.module('custom-fields', function (jQuery, _) {
      */
     _onChange: function (event) {
       if (event.target.value !== '') {
-        var parent = jQuery(event.target).parents('.control-custom');
+        var parent = jQuery(event.target).parents(this.options.fieldSelector);
         this.newField(parent);
       }
     },
 
     /* Event handler called when the remove checkbox is checked */
     _onRemove: function (event) {
-      var parent = jQuery(event.target).parents('.control-custom');
+      var parent = jQuery(event.target).parents(this.options.fieldSelector);
       this.disableField(parent, event.target.checked);
     }
   };
