@@ -1,51 +1,76 @@
-from nose.tools import assert_equal, assert_raises
-from pylons import config
+# encoding: utf-8
+
+from nose.tools import assert_equal, assert_raises, assert_in, assert_raises
 from email.mime.text import MIMEText
+from email.parser import Parser
+from email.header import decode_header
 import hashlib
+import base64
 
+from ckan.common import config
 import ckan.model as model
+import ckan.lib.helpers as h
 import ckan.lib.mailer as mailer
-from ckan.tests.pylons_controller import PylonsTestCase
-from ckan.tests.mock_mail_server import SmtpServerHarness
-from ckan.lib.create_test_data import CreateTestData
-from ckan.lib.base import g
+from ckan.tests.legacy.mock_mail_server import SmtpServerHarness
+
+import ckan.tests.helpers as helpers
+import ckan.tests.factories as factories
+
+
+class MailerBase(SmtpServerHarness):
 
-class TestMailer(SmtpServerHarness, PylonsTestCase):
     @classmethod
     def setup_class(cls):
+
+        helpers.reset_db()
+
         smtp_server = config.get('smtp.test_server')
         if smtp_server:
             host, port = smtp_server.split(':')
-            port = int(port) + int(str(hashlib.md5(cls.__name__).hexdigest())[0], 16)
+            port = (int(port) +
+                    int(str(hashlib.md5(cls.__name__).hexdigest())[0], 16))
             config['smtp.test_server'] = '%s:%s' % (host, port)
-        CreateTestData.create_user(name='bob', email='bob@bob.net')
-        CreateTestData.create_user(name='mary') #NB No email addr provided
+
+        # Created directly to avoid email validation
+        user_without_email = model.User(name='mary', email=None)
+        model.Session.add(user_without_email)
+        model.Session.commit()
         SmtpServerHarness.setup_class()
-        PylonsTestCase.setup_class()
 
     @classmethod
     def teardown_class(cls):
         SmtpServerHarness.teardown_class()
-        model.repo.rebuild_db()
+        helpers.reset_db()
 
     def setup(self):
         self.clear_smtp_messages()
 
     def mime_encode(self, msg, recipient_name):
-        sender_name = g.site_title
-        sender_url = g.site_url
-        body = mailer.add_msg_niceties(recipient_name, msg, sender_name, sender_url)
-        encoded_body = MIMEText(body.encode('utf-8'), 'plain', 'utf-8').get_payload().strip()
+        text = MIMEText(msg.encode('utf-8'), 'plain', 'utf-8')
+        encoded_body = text.get_payload().strip()
         return encoded_body
 
+    def get_email_body(self, msg):
+        payload = Parser().parsestr(msg).get_payload()
+        return base64.b64decode(payload)
+
+    def get_email_subject(self, msg):
+        header = Parser().parsestr(msg)['Subject']
+        return decode_header(header)[0][0]
+
+
+class TestMailer(MailerBase):
+
     def test_mail_recipient(self):
+        user = factories.User()
+
         msgs = self.get_smtp_messages()
         assert_equal(msgs, [])
 
         # send email
         test_email = {'recipient_name': 'Bob',
-                      'recipient_email':'bob@bob.net',
-                      'subject': 'Meeting', 
+                      'recipient_email': user['email'],
+                      'subject': 'Meeting',
                       'body': 'The meeting is cancelled.',
                       'headers': {'header1': 'value1'}}
         mailer.mail_recipient(**test_email)
@@ -61,15 +86,19 @@ class TestMailer(SmtpServerHarness, PylonsTestCase):
         assert test_email['subject'] in msg[3], msg[3]
         expected_body = self.mime_encode(test_email['body'],
                                          test_email['recipient_name'])
-        assert expected_body in msg[3], '%r not in %r' % (expected_body, msg[3])
+        assert_in(expected_body, msg[3])
 
     def test_mail_user(self):
+
+        user = factories.User()
+        user_obj = model.User.by_name(user['name'])
+
         msgs = self.get_smtp_messages()
         assert_equal(msgs, [])
 
         # send email
-        test_email = {'recipient': model.User.by_name(u'bob'),
-                      'subject': 'Meeting', 
+        test_email = {'recipient': user_obj,
+                      'subject': 'Meeting',
                       'body': 'The meeting is cancelled.',
                       'headers': {'header1': 'value1'}}
         mailer.mail_user(**test_email)
@@ -79,56 +108,128 @@ class TestMailer(SmtpServerHarness, PylonsTestCase):
         assert_equal(len(msgs), 1)
         msg = msgs[0]
         assert_equal(msg[1], config['smtp.mail_from'])
-        assert_equal(msg[2], [model.User.by_name(u'bob').email])
+        assert_equal(msg[2], [user['email']])
         assert test_email['headers'].keys()[0] in msg[3], msg[3]
         assert test_email['headers'].values()[0] in msg[3], msg[3]
         assert test_email['subject'] in msg[3], msg[3]
         expected_body = self.mime_encode(test_email['body'],
-                                         'bob')
-        assert expected_body in msg[3], '%r not in %r' % (expected_body, msg[3])
+                                         user['name'])
+
+        assert_in(expected_body, msg[3])
 
     def test_mail_user_without_email(self):
         # send email
         test_email = {'recipient': model.User.by_name(u'mary'),
-                      'subject': 'Meeting', 
+                      'subject': 'Meeting',
                       'body': 'The meeting is cancelled.',
                       'headers': {'header1': 'value1'}}
         assert_raises(mailer.MailerException, mailer.mail_user, **test_email)
 
-    def test_send_reset_email(self):
+    @helpers.change_config('ckan.site_title', 'My CKAN instance')
+    def test_from_field_format(self):
+
+        msgs = self.get_smtp_messages()
+        assert_equal(msgs, [])
+
         # send email
-        mailer.send_reset_link(model.User.by_name(u'bob'))
+        test_email = {'recipient_name': 'Bob',
+                      'recipient_email': 'Bob@bob.com',
+                      'subject': 'Meeting',
+                      'body': 'The meeting is cancelled.',
+                      'headers': {'header1': 'value1'}}
+        mailer.mail_recipient(**test_email)
+
+        # check it went to the mock smtp server
+        msgs = self.get_smtp_messages()
+        msg = msgs[0]
+
+        expected_from_header = '{0} <{1}>'.format(
+            config.get('ckan.site_title'),
+            config.get('smtp.mail_from')
+        )
+
+        assert_in(expected_from_header, msg[3])
+
+    def test_send_reset_email(self):
+        user = factories.User()
+        user_obj = model.User.by_name(user['name'])
+
+        mailer.send_reset_link(user_obj)
 
         # check it went to the mock smtp server
         msgs = self.get_smtp_messages()
         assert_equal(len(msgs), 1)
         msg = msgs[0]
         assert_equal(msg[1], config['smtp.mail_from'])
-        assert_equal(msg[2], [model.User.by_name(u'bob').email])
+        assert_equal(msg[2], [user['email']])
         assert 'Reset' in msg[3], msg[3]
-        test_msg = mailer.get_reset_link_body(model.User.by_name(u'bob'))
+        test_msg = mailer.get_reset_link_body(user_obj)
         expected_body = self.mime_encode(test_msg,
-                                         u'bob')
-        assert expected_body in msg[3], '%r not in %r' % (expected_body, msg[3])
+                                         user['name'])
 
-        # reset link tested in user functional test
+        assert_in(expected_body, msg[3])
 
     def test_send_invite_email(self):
-        user = model.User.by_name(u'bob')
-        assert user.reset_key is None, user
+        user = factories.User()
+        user_obj = model.User.by_name(user['name'])
+        assert user_obj.reset_key is None, user_obj
+
         # send email
-        mailer.send_invite(user)
+        mailer.send_invite(user_obj)
 
         # check it went to the mock smtp server
         msgs = self.get_smtp_messages()
         assert_equal(len(msgs), 1)
         msg = msgs[0]
         assert_equal(msg[1], config['smtp.mail_from'])
-        assert_equal(msg[2], [model.User.by_name(u'bob').email])
-        test_msg = mailer.get_invite_body(model.User.by_name(u'bob'))
+        assert_equal(msg[2], [user['email']])
+        test_msg = mailer.get_invite_body(user_obj)
         expected_body = self.mime_encode(test_msg,
-                                         u'bob') 
-        assert expected_body in msg[3], '%r not in %r' % (expected_body, msg[3])
-        assert user.reset_key is not None, user
-        
-        # reset link tested in user functional test
+                                         user['name'])
+
+        assert_in(expected_body, msg[3])
+        assert user_obj.reset_key is not None, user
+
+    def test_send_invite_email_with_group(self):
+        user = factories.User()
+        user_obj = model.User.by_name(user['name'])
+
+        group = factories.Group()
+        role = 'member'
+
+        # send email
+        mailer.send_invite(user_obj, group_dict=group, role=role)
+
+        # check it went to the mock smtp server
+        msgs = self.get_smtp_messages()
+        msg = msgs[0]
+        body = self.get_email_body(msg[3])
+        assert_in(group['title'], body)
+        assert_in(h.roles_translated()[role], body)
+
+    def test_send_invite_email_with_org(self):
+        user = factories.User()
+        user_obj = model.User.by_name(user['name'])
+
+        org = factories.Organization()
+        role = 'admin'
+
+        # send email
+        mailer.send_invite(user_obj, group_dict=org, role=role)
+
+        # check it went to the mock smtp server
+        msgs = self.get_smtp_messages()
+        msg = msgs[0]
+        body = self.get_email_body(msg[3])
+        assert_in(org['title'], body)
+        assert_in(h.roles_translated()[role], body)
+
+    @helpers.change_config('smtp.test_server', '999.999.999.999')
+    def test_bad_smtp_host(self):
+        test_email = {'recipient_name': 'Bob',
+                      'recipient_email': 'b@example.com',
+                      'subject': 'Meeting',
+                      'body': 'The meeting is cancelled.',
+                      'headers': {'header1': 'value1'}}
+        assert_raises(mailer.MailerException,
+                      mailer.mail_recipient, **test_email)
