@@ -1,57 +1,266 @@
+# encoding: utf-8
+
 import json
-import httpretty
 import nose
-import sys
-import datetime
-from nose.tools import assert_equal
+from nose.tools import assert_equal, raises
 
-import pylons
-from pylons import config
 import sqlalchemy.orm as orm
-import paste.fixture
+from ckan.tests.helpers import _get_test_app
 
+from ckan.common import config
 import ckan.plugins as p
 import ckan.lib.create_test_data as ctd
 import ckan.model as model
-import ckan.tests as tests
-import ckan.config.middleware as middleware
+import ckan.tests.legacy as tests
+import ckan.tests.helpers as helpers
+import ckan.tests.factories as factories
 
-import ckanext.datastore.db as db
-from ckanext.datastore.tests.helpers import rebuild_all_dbs, set_url_type
+import ckanext.datastore.backend.postgres as db
+from ckanext.datastore.tests.helpers import (
+    set_url_type, DatastoreFunctionalTestBase, DatastoreLegacyTestBase)
+from ckan.plugins.toolkit import ValidationError
 
 
-# avoid hanging tests https://github.com/gabrielfalcao/HTTPretty/issues/34
-if sys.version_info < (2, 7, 0):
-    import socket
-    socket.setdefaulttimeout(1)
+class TestDatastoreCreateNewTests(DatastoreFunctionalTestBase):
+    def test_create_creates_index_on_primary_key(self):
+        package = factories.Dataset()
+        data = {
+            'resource': {
+                'boo%k': 'crime',
+                'author': ['tolstoy', 'dostoevsky'],
+                'package_id': package['id']
+            },
+        }
+        result = helpers.call_action('datastore_create', **data)
+        resource_id = result['resource_id']
+        index_names = self._get_index_names(resource_id)
+        assert resource_id + '_pkey' in index_names
+
+    def test_create_creates_url_with_site_name(self):
+        package = factories.Dataset()
+        data = {
+            'resource': {
+                'boo%k': 'crime',
+                'package_id': package['id']
+            },
+        }
+        result = helpers.call_action('datastore_create', **data)
+        resource_id = result['resource_id']
+        resource = helpers.call_action('resource_show', id=resource_id)
+        url = resource['url']
+        assert url.startswith(config.get('ckan.site_url'))
+
+    def test_create_index_on_specific_fields(self):
+        package = factories.Dataset()
+        data = {
+            'resource': {
+                'boo%k': 'crime',
+                'author': ['tolstoy', 'dostoevsky'],
+                'package_id': package['id']
+            },
+            'fields': [{'id': 'book', 'type': 'text'},
+                       {'id': 'author', 'type': 'text'}],
+            'indexes': ['author']
+        }
+        result = helpers.call_action('datastore_create', **data)
+        resource_id = result['resource_id']
+        assert self._has_index_on_field(resource_id, '"author"')
+
+    def test_create_adds_index_on_full_text_search_when_creating_other_indexes(self):
+        package = factories.Dataset()
+        data = {
+            'resource': {
+                'boo%k': 'crime',
+                'author': ['tolstoy', 'dostoevsky'],
+                'package_id': package['id']
+            },
+            'fields': [{'id': 'book', 'type': 'text'},
+                       {'id': 'author', 'type': 'text'}],
+            'indexes': ['author']
+        }
+        result = helpers.call_action('datastore_create', **data)
+        resource_id = result['resource_id']
+        assert self._has_index_on_field(resource_id, '"_full_text"')
+
+    def test_create_adds_index_on_full_text_search_when_not_creating_other_indexes(self):
+        package = factories.Dataset()
+        data = {
+            'resource': {
+                'boo%k': 'crime',
+                'author': ['tolstoy', 'dostoevsky'],
+                'package_id': package['id']
+            },
+            'fields': [{'id': 'book', 'type': 'text'},
+                       {'id': 'author', 'type': 'text'}],
+        }
+        result = helpers.call_action('datastore_create', **data)
+        resource_id = result['resource_id']
+        assert self._has_index_on_field(resource_id, '"_full_text"')
+
+    def test_create_add_full_text_search_indexes_on_every_text_field(self):
+        package = factories.Dataset()
+        data = {
+            'resource': {
+                'book': 'crime',
+                'author': ['tolstoy', 'dostoevsky'],
+                'package_id': package['id']
+            },
+            'fields': [{'id': 'boo%k', 'type': 'text'},
+                       {'id': 'author', 'type': 'text'}],
+            'lang': 'english',
+        }
+        result = helpers.call_action('datastore_create', **data)
+        resource_id = result['resource_id']
+        assert self._has_index_on_field(resource_id,
+                                        "to_tsvector('english', \"boo%k\")")
+        assert self._has_index_on_field(resource_id,
+                                        "to_tsvector('english', \"author\")")
+
+    def test_create_doesnt_add_more_indexes_when_updating_data(self):
+        resource = factories.Resource()
+        data = {
+            'resource_id': resource['id'],
+            'force': True,
+            'records': [
+                {'book': 'annakarenina', 'author': 'tolstoy'}
+            ]
+        }
+        result = helpers.call_action('datastore_create', **data)
+        previous_index_names = self._get_index_names(resource['id'])
+        data = {
+            'resource_id': resource['id'],
+            'force': True,
+            'records': [
+                {'book': 'warandpeace', 'author': 'tolstoy'}
+            ]
+        }
+        result = helpers.call_action('datastore_create', **data)
+        current_index_names = self._get_index_names(resource['id'])
+        assert_equal(previous_index_names, current_index_names)
+
+    @raises(p.toolkit.ValidationError)
+    def test_create_duplicate_fields(self):
+        package = factories.Dataset()
+        data = {
+            'resource': {
+                'book': 'crime',
+                'author': ['tolstoy', 'dostoevsky'],
+                'package_id': package['id']
+            },
+            'fields': [{'id': 'book', 'type': 'text'},
+                       {'id': 'book', 'type': 'text'}],
+        }
+        result = helpers.call_action('datastore_create', **data)
 
 
-class TestDatastoreCreate(tests.WsgiAppCase):
+    def _has_index_on_field(self, resource_id, field):
+        sql = u"""
+            SELECT
+                relname
+            FROM
+                pg_class
+            WHERE
+                pg_class.relname = %s
+            """
+        index_name = db._generate_index_name(resource_id, field)
+        results = self._execute_sql(sql, index_name).fetchone()
+        return bool(results)
+
+    def _get_index_names(self, resource_id):
+        sql = u"""
+            SELECT
+                i.relname AS index_name
+            FROM
+                pg_class t,
+                pg_class i,
+                pg_index idx
+            WHERE
+                t.oid = idx.indrelid
+                AND i.oid = idx.indexrelid
+                AND t.relkind = 'r'
+                AND t.relname = %s
+            """
+        results = self._execute_sql(sql, resource_id).fetchall()
+        return [result[0] for result in results]
+
+    def _execute_sql(self, sql, *args):
+        engine = db.get_write_engine()
+        session = orm.scoped_session(orm.sessionmaker(bind=engine))
+        return session.connection().execute(sql, *args)
+
+    def test_sets_datastore_active_on_resource_on_create(self):
+        resource = factories.Resource()
+
+        assert_equal(resource['datastore_active'], False)
+
+        data = {
+            'resource_id': resource['id'],
+            'force': True,
+            'records': [
+                {'book': 'annakarenina', 'author': 'tolstoy'}
+            ]
+        }
+
+        helpers.call_action('datastore_create', **data)
+
+        resource = helpers.call_action('resource_show', id=resource['id'])
+
+        assert_equal(resource['datastore_active'], True)
+
+    def test_sets_datastore_active_on_resource_on_delete(self):
+        resource = factories.Resource(datastore_active=True)
+
+        assert_equal(resource['datastore_active'], True)
+
+        data = {
+            'resource_id': resource['id'],
+            'force': True,
+            'records': [
+                {'book': 'annakarenina', 'author': 'tolstoy'}
+            ]
+        }
+
+        helpers.call_action('datastore_create', **data)
+
+        helpers.call_action('datastore_delete', resource_id=resource['id'],
+                            force=True)
+
+        resource = helpers.call_action('resource_show', id=resource['id'])
+
+        assert_equal(resource['datastore_active'], False)
+
+    @raises(p.toolkit.ValidationError)
+    def test_create_exceeds_column_name_limit(self):
+        package = factories.Dataset()
+        data = {
+            'resource': {
+                'package_id': package['id']
+            },
+            'fields': [{
+                'id': 'This is a really long name for a column. Column names '
+                'in Postgres have a limit of 63 characters',
+                'type': 'text'
+            }]
+        }
+        result = helpers.call_action('datastore_create', **data)
+
+
+class TestDatastoreCreate(DatastoreLegacyTestBase):
     sysadmin_user = None
     normal_user = None
 
     @classmethod
     def setup_class(cls):
-
-        wsgiapp = middleware.make_app(config['global_conf'], **config)
-        cls.app = paste.fixture.TestApp(wsgiapp)
-        if not tests.is_datastore_supported():
-            raise nose.SkipTest("Datastore not supported")
-        p.load('datastore')
+        cls.app = _get_test_app()
+        super(TestDatastoreCreate, cls).setup_class()
         ctd.CreateTestData.create()
         cls.sysadmin_user = model.User.get('testsysadmin')
         cls.normal_user = model.User.get('annafan')
-        engine = db._get_engine(
-            {'connection_url': pylons.config['ckan.datastore.write_url']})
+        engine = db.get_write_engine()
         cls.Session = orm.scoped_session(orm.sessionmaker(bind=engine))
         set_url_type(
             model.Package.get('annakarenina').resources, cls.sysadmin_user)
 
-    @classmethod
-    def teardown_class(cls):
-        rebuild_all_dbs(cls.Session)
-        p.unload('datastore')
-
     def test_create_requires_auth(self):
         resource = model.Package.get('annakarenina').resources[0]
         data = {
@@ -154,39 +363,21 @@ class TestDatastoreCreate(tests.WsgiAppCase):
 
     def test_create_invalid_field_name(self):
         resource = model.Package.get('annakarenina').resources[0]
-        data = {
-            'resource_id': resource.id,
-            'fields': [{'id': 'book', 'type': 'text'},
-                       {'id': '_author', 'type': 'text'}]
-        }
-        postparams = '%s=1' % json.dumps(data)
         auth = {'Authorization': str(self.sysadmin_user.apikey)}
-        res = self.app.post('/api/action/datastore_create', params=postparams,
-                            extra_environ=auth, status=409)
-        res_dict = json.loads(res.body)
-        assert res_dict['success'] is False
+        invalid_names = ['_author', '"author', '', ' author', 'author ',
+                         '\tauthor', 'author\n']
 
-        data = {
-            'resource_id': resource.id,
-            'fields': [{'id': 'book', 'type': 'text'},
-                       {'id': '"author', 'type': 'text'}]
-        }
-        postparams = '%s=1' % json.dumps(data)
-        res = self.app.post('/api/action/datastore_create', params=postparams,
-                            extra_environ=auth, status=409)
-        res_dict = json.loads(res.body)
-        assert res_dict['success'] is False
-
-        data = {
-            'resource_id': resource.id,
-            'fields': [{'id': 'book', 'type': 'text'},
-                       {'id': '', 'type': 'text'}]
-        }
-        postparams = '%s=1' % json.dumps(data)
-        res = self.app.post('/api/action/datastore_create', params=postparams,
-                            extra_environ=auth, status=409)
-        res_dict = json.loads(res.body)
-        assert res_dict['success'] is False
+        for field_name in invalid_names:
+            data = {
+                'resource_id': resource.id,
+                'fields': [{'id': 'book', 'type': 'text'},
+                           {'id': field_name, 'type': 'text'}]
+            }
+            postparams = '%s=1' % json.dumps(data)
+            res = self.app.post('/api/action/datastore_create', params=postparams,
+                                extra_environ=auth, status=409)
+            res_dict = json.loads(res.body)
+            assert res_dict['success'] is False
 
     def test_create_invalid_record_field(self):
         resource = model.Package.get('annakarenina').resources[0]
@@ -328,7 +519,6 @@ class TestDatastoreCreate(tests.WsgiAppCase):
         res = res_dict['result']
         assert res['resource_id'] == data['resource_id']
         assert res['fields'] == data['fields'], res['fields']
-        assert res['records'] == data['records']
 
         c = self.Session.connection()
         results = c.execute('select * from "{0}"'.format(resource.id))
@@ -359,9 +549,9 @@ class TestDatastoreCreate(tests.WsgiAppCase):
 
             assert results == results_alias
 
-            sql = (u"select * from _table_metadata "
-                "where alias_of='{0}' and name='{1}'").format(resource.id, alias)
-            results = c.execute(sql)
+            sql = u"select * from _table_metadata " \
+                  "where alias_of=%s and name=%s"
+            results = c.execute(sql, resource.id, alias)
             assert results.rowcount == 1
         self.Session.remove()
 
@@ -471,14 +661,14 @@ class TestDatastoreCreate(tests.WsgiAppCase):
         # new aliases should replace old aliases
         c = self.Session.connection()
         for alias in aliases:
-            sql = (u"select * from _table_metadata "
-                "where alias_of='{0}' and name='{1}'").format(resource.id, alias)
-            results = c.execute(sql)
+            sql = "select * from _table_metadata " \
+                  "where alias_of=%s and name=%s"
+            results = c.execute(sql, resource.id, alias)
             assert results.rowcount == 0
 
-        sql = (u"select * from _table_metadata "
-            "where alias_of='{0}' and name='{1}'").format(resource.id, 'another_alias')
-        results = c.execute(sql)
+        sql = "select * from _table_metadata " \
+              "where alias_of=%s and name=%s"
+        results = c.execute(sql, resource.id, 'another_alias')
         assert results.rowcount == 1
         self.Session.remove()
 
@@ -562,7 +752,6 @@ class TestDatastoreCreate(tests.WsgiAppCase):
         assert res_dict['success'] is True
         res = res_dict['result']
         assert res['fields'] == data['fields'], res['fields']
-        assert res['records'] == data['records']
 
         # Get resource details
         data = {
@@ -684,3 +873,240 @@ class TestDatastoreCreate(tests.WsgiAppCase):
 
         assert res_dict['success'] is False
 
+    def test_datastore_create_with_invalid_data_value(self):
+        """datastore_create() should return an error for invalid data."""
+        resource = factories.Resource(url_type="datastore")
+        data_dict = {
+            "resource_id": resource["id"],
+            "fields": [{"id": "value", "type": "numeric"}],
+            "records": [
+                {"value": 0},
+                {"value": 1},
+                {"value": 2},
+                {"value": 3},
+                {"value": "   "},  # Invalid numeric value.
+                {"value": 5},
+                {"value": 6},
+                {"value": 7},
+            ],
+            "method": "insert",
+        }
+        postparams = '%s=1' % json.dumps(data_dict)
+        auth = {'Authorization': str(self.sysadmin_user.apikey)}
+        res = self.app.post('/api/action/datastore_create', params=postparams,
+                            extra_environ=auth, status=409)
+        res_dict = json.loads(res.body)
+
+        assert res_dict['success'] is False
+        assert res_dict['error']['__type'] == 'Validation Error'
+        assert res_dict['error']['message'].startswith('The data was invalid')
+
+
+class TestDatastoreFunctionCreate(DatastoreFunctionalTestBase):
+    def test_nop_trigger(self):
+        helpers.call_action(
+            u'datastore_function_create',
+            name=u'test_nop',
+            rettype=u'trigger',
+            definition=u'BEGIN RETURN NEW; END;')
+
+    def test_invalid_definition(self):
+        try:
+            helpers.call_action(
+                u'datastore_function_create',
+                name=u'test_invalid_def',
+                rettype=u'trigger',
+                definition=u'HELLO WORLD')
+        except ValidationError as ve:
+            assert_equal(
+                ve.error_dict,
+                {u'definition':
+                    [u'syntax error at or near "HELLO"']})
+        else:
+            assert 0, u'no validation error'
+
+    def test_redefined_trigger(self):
+        helpers.call_action(
+            u'datastore_function_create',
+            name=u'test_redefined',
+            rettype=u'trigger',
+            definition=u'BEGIN RETURN NEW; END;')
+        try:
+            helpers.call_action(
+                u'datastore_function_create',
+                name=u'test_redefined',
+                rettype=u'trigger',
+                definition=u'BEGIN RETURN NEW; END;')
+        except ValidationError as ve:
+            assert_equal(
+                ve.error_dict,
+                {u'name':[
+                    u'function "test_redefined" already exists '
+                    u'with same argument types']})
+        else:
+            assert 0, u'no validation error'
+
+    def test_redefined_with_or_replace_trigger(self):
+        helpers.call_action(
+            u'datastore_function_create',
+            name=u'test_replaceme',
+            rettype=u'trigger',
+            definition=u'BEGIN RETURN NEW; END;')
+        helpers.call_action(
+            u'datastore_function_create',
+            name=u'test_replaceme',
+            or_replace=True,
+            rettype=u'trigger',
+            definition=u'BEGIN RETURN NEW; END;')
+
+
+class TestDatastoreCreateTriggers(DatastoreFunctionalTestBase):
+    def test_create_with_missing_trigger(self):
+        ds = factories.Dataset()
+
+        try:
+            app = self._get_test_app()
+            with app.flask_app.test_request_context():
+                helpers.call_action(
+                    u'datastore_create',
+                    resource={u'package_id': ds['id']},
+                    fields=[{u'id': u'spam', u'type': u'text'}],
+                    records=[{u'spam': u'SPAM'}, {u'spam': u'EGGS'}],
+                    triggers=[{u'function': u'no_such_trigger_function'}])
+        except ValidationError as ve:
+            assert_equal(
+                ve.error_dict,
+                {u'triggers':[
+                    u'function no_such_trigger_function() does not exist']})
+        else:
+            assert 0, u'no validation error'
+
+    def test_create_trigger_applies_to_records(self):
+        ds = factories.Dataset()
+
+        helpers.call_action(
+            u'datastore_function_create',
+            name=u'spamify_trigger',
+            rettype=u'trigger',
+            definition=u'''
+                BEGIN
+                NEW.spam := 'spam spam ' || NEW.spam || ' spam';
+                RETURN NEW;
+                END;''')
+
+        app = self._get_test_app()
+        with app.flask_app.test_request_context():
+            res = helpers.call_action(
+                u'datastore_create',
+                resource={u'package_id': ds['id']},
+                fields=[{u'id': u'spam', u'type': u'text'}],
+                records=[{u'spam': u'SPAM'}, {u'spam': u'EGGS'}],
+                triggers=[{u'function': u'spamify_trigger'}])
+        assert_equal(
+            helpers.call_action(
+                u'datastore_search',
+                fields=[u'spam'],
+                resource_id=res['resource_id'])['records'],
+            [
+                {u'spam': u'spam spam SPAM spam'},
+                {u'spam': u'spam spam EGGS spam'}])
+
+    def test_upsert_trigger_applies_to_records(self):
+        ds = factories.Dataset()
+
+        helpers.call_action(
+            u'datastore_function_create',
+            name=u'more_spam_trigger',
+            rettype=u'trigger',
+            definition=u'''
+                BEGIN
+                NEW.spam := 'spam spam ' || NEW.spam || ' spam';
+                RETURN NEW;
+                END;''')
+
+        app = self._get_test_app()
+        with app.flask_app.test_request_context():
+            res = helpers.call_action(
+                u'datastore_create',
+                resource={u'package_id': ds['id']},
+                fields=[{u'id': u'spam', u'type': u'text'}],
+                triggers=[{u'function': u'more_spam_trigger'}])
+            helpers.call_action(
+                u'datastore_upsert',
+                method=u'insert',
+                resource_id=res['resource_id'],
+                records=[{u'spam': u'BEANS'}, {u'spam': u'SPAM'}])
+        assert_equal(
+            helpers.call_action(
+                u'datastore_search',
+                fields=[u'spam'],
+                resource_id=res['resource_id'])['records'],
+            [
+                {u'spam': u'spam spam BEANS spam'},
+                {u'spam': u'spam spam SPAM spam'}])
+
+    def test_create_trigger_exception(self):
+        ds = factories.Dataset()
+
+        helpers.call_action(
+            u'datastore_function_create',
+            name=u'spamexception_trigger',
+            rettype=u'trigger',
+            definition=u'''
+                BEGIN
+                IF NEW.spam != 'spam' THEN
+                    RAISE EXCEPTION '"%"? Yeeeeccch!', NEW.spam;
+                END IF;
+                RETURN NEW;
+                END;''')
+        try:
+            app = self._get_test_app()
+            with app.flask_app.test_request_context():
+                helpers.call_action(
+                    u'datastore_create',
+                    resource={u'package_id': ds['id']},
+                    fields=[{u'id': u'spam', u'type': u'text'}],
+                    records=[{u'spam': u'spam'}, {u'spam': u'EGGS'}],
+                    triggers=[{u'function': u'spamexception_trigger'}])
+        except ValidationError as ve:
+            assert_equal(
+                ve.error_dict,
+                {u'records':[
+                    u'"EGGS"? Yeeeeccch!']})
+        else:
+            assert 0, u'no validation error'
+
+    def test_upsert_trigger_exception(self):
+        ds = factories.Dataset()
+
+        helpers.call_action(
+            u'datastore_function_create',
+            name=u'spamonly_trigger',
+            rettype=u'trigger',
+            definition=u'''
+                BEGIN
+                IF NEW.spam != 'spam' THEN
+                    RAISE EXCEPTION '"%"? Yeeeeccch!', NEW.spam;
+                END IF;
+                RETURN NEW;
+                END;''')
+        app = self._get_test_app()
+        with app.flask_app.test_request_context():
+            res = helpers.call_action(
+                u'datastore_create',
+                resource={u'package_id': ds['id']},
+                fields=[{u'id': u'spam', u'type': u'text'}],
+                triggers=[{u'function': u'spamonly_trigger'}])
+            try:
+                helpers.call_action(
+                    u'datastore_upsert',
+                    method=u'insert',
+                    resource_id=res['resource_id'],
+                    records=[{u'spam': u'spam'}, {u'spam': u'BEANS'}])
+            except ValidationError as ve:
+                assert_equal(
+                    ve.error_dict,
+                    {u'records':[
+                        u'"BEANS"? Yeeeeccch!']})
+            else:
+                assert 0, u'no validation error'
