@@ -1,28 +1,81 @@
+# encoding: utf-8
+
 import json
 import nose
 import datetime
 
-import pylons
 import sqlalchemy.orm as orm
 
 import ckan.plugins as p
 import ckan.lib.create_test_data as ctd
 import ckan.model as model
-import ckan.tests as tests
+import ckan.tests.legacy as tests
+import ckan.tests.helpers as helpers
+import ckan.tests.factories as factories
 
-import ckanext.datastore.db as db
-from ckanext.datastore.tests.helpers import rebuild_all_dbs, set_url_type
+from ckan.common import config
+
+import ckanext.datastore.backend.postgres as db
+from ckanext.datastore.tests.helpers import (
+    set_url_type, DatastoreFunctionalTestBase, DatastoreLegacyTestBase)
+
+assert_equal = nose.tools.assert_equal
 
 
-class TestDatastoreUpsert(tests.WsgiAppCase):
+class TestDatastoreUpsertNewTests(DatastoreFunctionalTestBase):
+    def test_upsert_doesnt_crash_with_json_field(self):
+        resource = factories.Resource()
+        data = {
+            'resource_id': resource['id'],
+            'force': True,
+            'primary_key': 'id',
+            'fields': [{'id': 'id', 'type': 'text'},
+                       {'id': 'book', 'type': 'json'},
+                       {'id': 'author', 'type': 'text'}],
+        }
+        helpers.call_action('datastore_create', **data)
+        data = {
+            'resource_id': resource['id'],
+            'force': True,
+            'method': 'insert',
+            'records': [
+                {'id': '1',
+                 'book': {'code': 'A', 'title': u'ñ'},
+                 'author': 'tolstoy'}],
+        }
+        helpers.call_action('datastore_upsert', **data)
+
+    def test_upsert_doesnt_crash_with_json_field_with_string_value(self):
+        resource = factories.Resource()
+        data = {
+            'resource_id': resource['id'],
+            'force': True,
+            'primary_key': 'id',
+            'fields': [{'id': 'id', 'type': 'text'},
+                       {'id': 'book', 'type': 'json'},
+                       {'id': 'author', 'type': 'text'}],
+        }
+        helpers.call_action('datastore_create', **data)
+        data = {
+            'resource_id': resource['id'],
+            'force': True,
+            'method': 'insert',
+            'records': [
+                {'id': '1',
+                 'book': u'ñ',
+                 'author': 'tolstoy'}],
+        }
+        helpers.call_action('datastore_upsert', **data)
+
+
+class TestDatastoreUpsert(DatastoreLegacyTestBase):
     sysadmin_user = None
     normal_user = None
 
     @classmethod
     def setup_class(cls):
-        if not tests.is_datastore_supported():
-            raise nose.SkipTest("Datastore not supported")
-        p.load('datastore')
+        cls.app = helpers._get_test_app()
+        super(TestDatastoreUpsert, cls).setup_class()
         ctd.CreateTestData.create()
         cls.sysadmin_user = model.User.get('testsysadmin')
         cls.normal_user = model.User.get('annafan')
@@ -50,15 +103,9 @@ class TestDatastoreUpsert(tests.WsgiAppCase):
         res_dict = json.loads(res.body)
         assert res_dict['success'] is True
 
-        engine = db._get_engine(
-            {'connection_url': pylons.config['ckan.datastore.write_url']})
+        engine = db.get_write_engine()
         cls.Session = orm.scoped_session(orm.sessionmaker(bind=engine))
 
-    @classmethod
-    def teardown_class(cls):
-        rebuild_all_dbs(cls.Session)
-        p.unload('datastore')
-
     def test_upsert_requires_auth(self):
         data = {
             'resource_id': self.data['resource_id']
@@ -238,16 +285,42 @@ class TestDatastoreUpsert(tests.WsgiAppCase):
 
         assert res_dict['success'] is False
 
+    def test_upsert_works_with_empty_list_in_json_field(self):
+        hhguide = u"hitchhiker's guide to the galaxy"
 
-class TestDatastoreInsert(tests.WsgiAppCase):
+        data = {
+            'resource_id': self.data['resource_id'],
+            'method': 'upsert',
+            'records': [{
+                'nested': [],
+                u'b\xfck': hhguide}]
+        }
+
+        postparams = '%s=1' % json.dumps(data)
+        auth = {'Authorization': str(self.sysadmin_user.apikey)}
+        res = self.app.post('/api/action/datastore_upsert', params=postparams,
+                            extra_environ=auth)
+        res_dict = json.loads(res.body)
+        assert res_dict['success'] is True, res_dict
+
+        c = self.Session.connection()
+        results = c.execute('select * from "{0}"'.format(data['resource_id']))
+        record = [r for r in results.fetchall() if r[2] == hhguide]
+        self.Session.remove()
+        assert len(record) == 1, record
+        assert_equal(json.loads(record[0][4].json),
+                     data['records'][0]['nested'])
+
+
+
+class TestDatastoreInsert(DatastoreLegacyTestBase):
     sysadmin_user = None
     normal_user = None
 
     @classmethod
     def setup_class(cls):
-        if not tests.is_datastore_supported():
-            raise nose.SkipTest("Datastore not supported")
-        p.load('datastore')
+        cls.app = helpers._get_test_app()
+        super(TestDatastoreInsert, cls).setup_class()
         ctd.CreateTestData.create()
         cls.sysadmin_user = model.User.get('testsysadmin')
         cls.normal_user = model.User.get('annafan')
@@ -275,15 +348,9 @@ class TestDatastoreInsert(tests.WsgiAppCase):
         res_dict = json.loads(res.body)
         assert res_dict['success'] is True
 
-        engine = db._get_engine(
-            {'connection_url': pylons.config['ckan.datastore.write_url']})
+        engine = db.get_write_engine()
         cls.Session = orm.scoped_session(orm.sessionmaker(bind=engine))
 
-    @classmethod
-    def teardown_class(cls):
-        p.unload('datastore')
-        rebuild_all_dbs(cls.Session)
-
     def test_insert_non_existing_field(self):
         data = {
             'resource_id': self.data['resource_id'],
@@ -341,15 +408,14 @@ class TestDatastoreInsert(tests.WsgiAppCase):
         assert results.rowcount == 3
 
 
-class TestDatastoreUpdate(tests.WsgiAppCase):
+class TestDatastoreUpdate(DatastoreLegacyTestBase):
     sysadmin_user = None
     normal_user = None
 
     @classmethod
     def setup_class(cls):
-        if not tests.is_datastore_supported():
-            raise nose.SkipTest("Datastore not supported")
-        p.load('datastore')
+        cls.app = helpers._get_test_app()
+        super(TestDatastoreUpdate, cls).setup_class()
         ctd.CreateTestData.create()
         cls.sysadmin_user = model.User.get('testsysadmin')
         cls.normal_user = model.User.get('annafan')
@@ -382,15 +448,9 @@ class TestDatastoreUpdate(tests.WsgiAppCase):
         res_dict = json.loads(res.body)
         assert res_dict['success'] is True
 
-        engine = db._get_engine(
-            {'connection_url': pylons.config['ckan.datastore.write_url']})
+        engine = db.get_write_engine()
         cls.Session = orm.scoped_session(orm.sessionmaker(bind=engine))
 
-    @classmethod
-    def teardown_class(cls):
-        p.unload('datastore')
-        rebuild_all_dbs(cls.Session)
-
     def test_update_basic(self):
         c = self.Session.connection()
         results = c.execute('select 1 from "{0}"'.format(self.data['resource_id']))
