@@ -1,20 +1,27 @@
-import sets
+# encoding: utf-8
+
+from six import string_types
+
 import ckan
 from ckan.plugins import SingletonPlugin, implements, IPackageController
-from ckan.plugins import IGroupController, IOrganizationController, ITagController
-import pylons
-import ckan.logic.action.get as action_get
-from pylons import config
+from ckan.plugins import IGroupController, IOrganizationController, ITagController, IResourceController
+
+from ckan.common import request, config, c
+from ckan.logic import get_action
+
+try:
+    long        # Python 2
+except NameError:
+    long = int  # Python 3
 
-LANGS = ['en', 'fr', 'de', 'es', 'it', 'nl', 'ro', 'pt', 'pl']
 
 def translate_data_dict(data_dict):
     '''Return the given dict (e.g. a dataset dict) with as many of its fields
     as possible translated into the desired or the fallback language.
 
     '''
-    desired_lang_code = pylons.request.environ['CKAN_LANG']
-    fallback_lang_code = pylons.config.get('ckan.locale_default', 'en')
+    desired_lang_code = request.environ['CKAN_LANG']
+    fallback_lang_code = config.get('ckan.locale_default', 'en')
 
     # Get a flattened copy of data_dict to do the translation on.
     flattened = ckan.lib.navl.dictization_functions.flatten_dict(
@@ -22,20 +29,26 @@ def translate_data_dict(data_dict):
 
     # Get a simple flat list of all the terms to be translated, from the
     # flattened data dict.
-    terms = sets.Set()
+    terms = set()
     for (key, value) in flattened.items():
         if value in (None, True, False):
             continue
-        elif isinstance(value, basestring):
+        elif isinstance(value, string_types):
             terms.add(value)
         elif isinstance(value, (int, long)):
             continue
         else:
             for item in value:
-                terms.add(item)
+                if isinstance(value, dict):
+                    if key == (u'organization',) and item == 'description':
+                        terms.add(value[item])
+                    else:
+                        terms.add(item)
+                else:
+                    terms.add(item)
 
     # Get the translations of all the terms (as a list of dictionaries).
-    translations = ckan.logic.action.get.term_translation_show(
+    translations = get_action('term_translation_show')(
             {'model': ckan.model},
             {'terms': terms,
                 'lang_codes': (desired_lang_code, fallback_lang_code)})
@@ -68,7 +81,98 @@ def translate_data_dict(data_dict):
             # Don't try to translate values that aren't strings.
             translated_flattened[key] = value
 
-        elif isinstance(value, basestring):
+        elif isinstance(value, string_types):
+            if value in desired_translations:
+                translated_flattened[key] = desired_translations[value]
+            else:
+                translated_flattened[key] = fallback_translations.get(
+                        value, value)
+
+        elif isinstance(value, (int, long, dict)):
+            if key == (u'organization',):
+                translated_flattened[key] = translate_data_dict(value);
+            else:
+                translated_flattened[key] = value
+
+        else:
+            translated_value = []
+            for item in value:
+                if item in desired_translations:
+                    translated_value.append(desired_translations[item])
+                else:
+                    translated_value.append(
+                        fallback_translations.get(item, item)
+                    )
+            translated_flattened[key] = translated_value
+
+    # Finally unflatten and return the translated data dict.
+    translated_data_dict = (ckan.lib.navl.dictization_functions
+            .unflatten(translated_flattened))
+    return translated_data_dict
+
+def translate_resource_data_dict(data_dict):
+    '''Return the given dict with as many of its fields
+    as possible translated into the desired or the fallback language.
+
+    '''
+
+    desired_lang_code = request.environ['CKAN_LANG']
+    fallback_lang_code = config.get('ckan.locale_default', 'en')
+
+    # Get a flattened copy of data_dict to do the translation on.
+    flattened = ckan.lib.navl.dictization_functions.flatten_dict(
+            data_dict)
+
+    # Get a simple flat list of all the terms to be translated, from the
+    # flattened data dict.
+    terms = sets.Set()
+    for (key, value) in flattened.items():
+        if value in (None, True, False):
+            continue
+        elif isinstance(value, string_types):
+            terms.add(value)
+        elif isinstance(value, (int, long)):
+            continue
+        else:
+            for item in value:
+                 terms.add(item)
+
+    # Get the translations of all the terms (as a list of dictionaries).
+    translations = ckan.logic.action.get.term_translation_show(
+            {'model': ckan.model},
+            {'terms': terms,
+                'lang_codes': (desired_lang_code, fallback_lang_code)})
+    # Transform the translations into a more convenient structure.
+    desired_translations = {}
+    fallback_translations = {}
+    for translation in translations:
+        if translation['lang_code'] == desired_lang_code:
+            desired_translations[translation['term']] = (
+                    translation['term_translation'])
+        else:
+            assert translation['lang_code'] == fallback_lang_code
+            fallback_translations[translation['term']] = (
+                    translation['term_translation'])
+
+    # Make a copy of the flattened data dict with all the terms replaced by
+    # their translations, where available.
+    translated_flattened = {}
+    for (key, value) in flattened.items():
+
+        # Don't translate names that are used for form URLs.
+        if key == ('name',):
+            if value in desired_translations:
+                translated_flattened[key] = desired_translations[value]
+            elif value in fallback_translations:
+                translated_flattened[key] = fallback_translations.get(value, value)
+            else:
+                translated_flattened[key] = value
+
+        elif value in (None, True, False):
+            # Don't try to translate values that aren't strings.
+            translated_flattened[key] = value
+
+        elif isinstance(value, string_types):
             if value in desired_translations:
                 translated_flattened[key] = desired_translations[value]
             else:
@@ -88,7 +192,6 @@ def translate_data_dict(data_dict):
                         fallback_translations.get(item, item)
                     )
             translated_flattened[key] = translated_value
-
     # Finally unflatten and return the translated data dict.
     translated_data_dict = (ckan.lib.navl.dictization_functions
             .unflatten(translated_flattened))
@@ -99,21 +202,22 @@ KEYS_TO_IGNORE = ['state', 'revision_id', 'id', #title done seperately
 
 class MultilingualDataset(SingletonPlugin):
     implements(IPackageController, inherit=True)
+    LANGS = config.get('ckan.locale_order', 'en').split(" ")
 
     def before_index(self, search_data):
 
         default_lang = search_data.get(
-            'lang_code', 
-             pylons.config.get('ckan.locale_default', 'en')
+            'lang_code',
+             config.get('ckan.locale_default', 'en')
         )
 
         ## translate title
         title = search_data.get('title')
-        search_data['title_' + default_lang] = title 
-        title_translations = action_get.term_translation_show(
+        search_data['title_' + default_lang] = title
+        title_translations = get_action('term_translation_show')(
                           {'model': ckan.model},
                           {'terms': [title],
-                              'lang_codes': LANGS})
+                           'lang_codes': self.LANGS})
 
         for translation in title_translations:
             title_field = 'title_' + translation['lang_code']
@@ -127,16 +231,16 @@ class MultilingualDataset(SingletonPlugin):
             if not isinstance(value, list):
                 value = [value]
             for item in value:
-                if isinstance(item, basestring):
+                if isinstance(item, string_types):
                     all_terms.append(item)
 
-        field_translations = action_get.term_translation_show(
+        field_translations = get_action('term_translation_show')(
                           {'model': ckan.model},
                           {'terms': all_terms,
-                              'lang_codes': LANGS})
+                           'lang_codes': self.LANGS})
+
+        text_field_items = dict(('text_' + lang, []) for lang in self.LANGS)
 
-        text_field_items = dict(('text_' + lang, []) for lang in LANGS)
-        
         text_field_items['text_' + default_lang].extend(all_terms)
 
         for translation in sorted(field_translations):
@@ -145,12 +249,25 @@ class MultilingualDataset(SingletonPlugin):
 
         for key, value in text_field_items.iteritems():
             search_data[key] = ' '.join(value)
-        
+
         return search_data
 
     def before_search(self, search_params):
-        lang_set = set(LANGS)
-        current_lang = pylons.request.environ['CKAN_LANG']
+        lang_set = set(self.LANGS)
+
+        try:
+            current_lang = request.environ['CKAN_LANG']
+        except TypeError as err:
+            if err.message == ('No object (name: request) has been registered '
+                               'for this thread'):
+                # This happens when this code gets called as part of a paster
+                # command rather then as part of an HTTP request.
+                current_lang = config.get('ckan.locale_default')
+            else:
+                raise
+        except KeyError:
+            current_lang = config.get('ckan.locale_default')
+
         # fallback to default locale if locale not in suported langs
         if not current_lang in lang_set:
             current_lang = config.get('ckan.locale_default')
@@ -177,15 +294,15 @@ class MultilingualDataset(SingletonPlugin):
         if not facets:
             return search_results
 
-        desired_lang_code = pylons.request.environ['CKAN_LANG']
-        fallback_lang_code = pylons.config.get('ckan.locale_default', 'en')
+        desired_lang_code = request.environ['CKAN_LANG']
+        fallback_lang_code = config.get('ckan.locale_default', 'en')
 
         # Look up translations for all of the facets in one db query.
-        terms = sets.Set()
+        terms = set()
         for facet in facets.values():
             for item in facet['items']:
                 terms.add(item['display_name'])
-        translations = ckan.logic.action.get.term_translation_show(
+        translations = get_action('term_translation_show')(
                 {'model': ckan.model},
                 {'terms': terms,
                     'lang_codes': (desired_lang_code, fallback_lang_code)})
@@ -216,16 +333,19 @@ class MultilingualDataset(SingletonPlugin):
         # all the terms in c.fields (c.fields contains the selected facets)
         # and save them in c.translated_fields where the templates can
         # retrieve them later.
-        c = pylons.c
-        desired_lang_code = pylons.request.environ['CKAN_LANG']
-        fallback_lang_code = pylons.config.get('ckan.locale_default', 'en')
-        terms = [value for param, value in c.fields]
-        translations = ckan.logic.action.get.term_translation_show(
+        desired_lang_code = request.environ['CKAN_LANG']
+        fallback_lang_code = config.get('ckan.locale_default', 'en')
+        try:
+            fields = c.fields
+        except AttributeError:
+            return translate_data_dict(dataset_dict)
+        terms = [value for param, value in fields]
+        translations = get_action('term_translation_show')(
                 {'model': ckan.model},
                 {'terms': terms,
                  'lang_codes': (desired_lang_code, fallback_lang_code)})
         c.translated_fields = {}
-        for param, value in c.fields:
+        for param, value in fields:
             matching_translations = [translation for translation in
                     translations if translation['term'] == value and
                     translation['lang_code'] == desired_lang_code]
@@ -275,3 +395,14 @@ class MultilingualTag(SingletonPlugin):
     def before_view(self, data_dict):
         translated_data_dict = translate_data_dict(data_dict)
         return translated_data_dict
+
+class MultilingualResource(SingletonPlugin):
+   '''The MultilinguaResource plugin translate the selected resource name and description on resource
+   preview page.
+
+   '''
+   implements(IResourceController, inherit=True)
+
+   def before_show(self, data_dict):
+        translated_data_dict = translate_resource_data_dict(data_dict)
+        return translated_data_dict
