@@ -1,19 +1,17 @@
+# encoding: utf-8
+
 import requests
 import unittest
 import json
 import httpretty
 
-import paste.fixture
-from pylons import config
+from ckan.tests.helpers import _get_test_app
+from ckan.common import config
 
-import ckan.logic as logic
 import ckan.model as model
-import ckan.tests as tests
-import ckan.plugins as plugins
+import ckan.plugins as p
 import ckan.lib.create_test_data as create_test_data
-import ckan.config.middleware as middleware
 import ckanext.resourceproxy.controller as controller
-
 import ckanext.resourceproxy.plugin as proxy
 
 
@@ -29,33 +27,35 @@ def set_resource_url(url):
     context = {
         'model': model,
         'session': model.Session,
-        'user': model.User.get('testsysadmin').name
+        'user': model.User.get('testsysadmin').name,
+        'use_cache': False,
     }
 
-    resource = logic.get_action('resource_show')(context, {'id': testpackage.resources[0].id})
-    package = logic.get_action('package_show')(context, {'id': testpackage.id})
+    resource = p.toolkit.get_action('resource_show')(
+        context, {'id': testpackage.resources[0].id})
+    package = p.toolkit.get_action('package_show')(
+        context, {'id': testpackage.id})
 
     resource['url'] = url
-    logic.action.update.resource_update(context, resource)
+    p.toolkit.get_action('resource_update')(context, resource)
 
     testpackage = model.Package.get('annakarenina')
-    assert testpackage.resources[0].url == resource['url'], testpackage.resources[0].url
+    assert testpackage.resources[0].url == resource['url']
 
     return {'resource': resource, 'package': package}
 
 
-class TestProxyPrettyfied(tests.WsgiAppCase, unittest.TestCase):
+class TestProxyPrettyfied(unittest.TestCase):
 
     serving = False
 
     @classmethod
     def setup_class(cls):
         cls._original_config = config.copy()
+        cls.app = _get_test_app()
+        if not p.plugin_loaded('resource_proxy'):
+            p.load('resource_proxy')
         config['ckan.plugins'] = 'resource_proxy'
-        wsgiapp = middleware.make_app(config['global_conf'], **config)
-        cls.app = paste.fixture.TestApp(wsgiapp)
-
-        # create test resource
         create_test_data.CreateTestData.create()
 
     @classmethod
@@ -69,8 +69,10 @@ class TestProxyPrettyfied(tests.WsgiAppCase, unittest.TestCase):
         self.data_dict = set_resource_url(self.url)
 
     def register(self, *args, **kwargs):
-        httpretty.HTTPretty.register_uri(httpretty.HTTPretty.GET, *args, **kwargs)
-        httpretty.HTTPretty.register_uri(httpretty.HTTPretty.HEAD, *args, **kwargs)
+        httpretty.HTTPretty.register_uri(httpretty.HTTPretty.GET, *args,
+                                         **kwargs)
+        httpretty.HTTPretty.register_uri(httpretty.HTTPretty.HEAD, *args,
+                                         **kwargs)
 
     @httpretty.activate
     def test_resource_proxy_on_200(self):
@@ -100,7 +102,7 @@ class TestProxyPrettyfied(tests.WsgiAppCase, unittest.TestCase):
         result = self.app.get(proxied_url, status='*')
         # we expect a 409 because the resourceproxy got an error (404)
         # from the server
-        assert result.status == 409, result.status
+        assert result.status_int == 409, result.status
         assert '404' in result.body
 
     @httpretty.activate
@@ -113,7 +115,7 @@ class TestProxyPrettyfied(tests.WsgiAppCase, unittest.TestCase):
 
         proxied_url = proxy.get_proxified_resource_url(self.data_dict)
         result = self.app.get(proxied_url, status='*')
-        assert result.status == 409, result.status
+        assert result.status_int == 409, result.status
         assert 'too large' in result.body, result.body
 
     @httpretty.activate
@@ -126,28 +128,48 @@ class TestProxyPrettyfied(tests.WsgiAppCase, unittest.TestCase):
 
         proxied_url = proxy.get_proxified_resource_url(self.data_dict)
         result = self.app.get(proxied_url, status='*')
-        assert result.status == 409, result.status
+        assert result.status_int == 409, result.status
         assert 'too large' in result.body, result.body
 
     @httpretty.activate
     def test_invalid_url(self):
-        self.data_dict = set_resource_url('javascript:downloadFile(foo)')
+        self.data_dict = set_resource_url('http:invalid_url')
 
         proxied_url = proxy.get_proxified_resource_url(self.data_dict)
         result = self.app.get(proxied_url, status='*')
-        assert result.status == 409, result.status
+        assert result.status_int == 409, result.status
         assert 'Invalid URL' in result.body, result.body
 
-
     def test_non_existent_url(self):
-        self.data_dict = set_resource_url('http://foo.bar')
+        self.data_dict = set_resource_url('http://nonexistent.example.com')
 
         def f1():
             url = self.data_dict['resource']['url']
             requests.get(url)
+
         self.assertRaises(requests.ConnectionError, f1)
 
         proxied_url = proxy.get_proxified_resource_url(self.data_dict)
         result = self.app.get(proxied_url, status='*')
-        assert result.status == 502, result.status
+        assert result.status_int == 502, result.status
         assert 'connection error' in result.body, result.body
+
+    def test_proxied_resource_url_proxies_http_and_https_by_default(self):
+        http_url = 'http://ckan.org'
+        https_url = 'https://ckan.org'
+
+        for url in [http_url, https_url]:
+            data_dict = set_resource_url(url)
+            proxied_url = proxy.get_proxified_resource_url(data_dict)
+            assert proxied_url != url, proxied_url
+
+    def test_resource_url_doesnt_proxy_non_http_or_https_urls_by_default(self):
+        schemes = ['file', 'ws']
+
+        for scheme in schemes:
+            url = '%s://ckan.org' % scheme
+            data_dict = set_resource_url(url)
+            non_proxied_url = proxy.get_proxified_resource_url(data_dict)
+            proxied_url = proxy.get_proxified_resource_url(data_dict, scheme)
+            assert non_proxied_url == url, non_proxied_url
+            assert proxied_url != url, proxied_url
