@@ -1,3 +1,5 @@
+# encoding: utf-8
+
 from logging import getLogger
 
 import ckan.plugins as p
@@ -19,5 +21,9 @@ class StatsPlugin(p.SingletonPlugin):
         return map
 
     def update_config(self, config):
-        p.toolkit.add_template_directory(config, 'templates')
+        templates = 'templates'
+        if p.toolkit.asbool(config.get('ckan.legacy_templates', False)):
+                templates = 'templates_legacy'
+        p.toolkit.add_template_directory(config, templates)
         p.toolkit.add_public_directory(config, 'public')
+        p.toolkit.add_resource('public/ckanext/stats', 'ckanext_stats')
